#include "standard.h"

#include "classftn.h"
#include "constants.h"
#include "dot.h"
#include "error.h"
#include "files.h"
#include "MCMC.h"
#include "nd.h"
#include "random.h"
#include "return.h"
#include "sim_tree.h"
#include "test.h"
#include "tip_seqs.h"
#include "tree.h"

#include <boost/regex.hpp>	



#ifndef TEST_MODE
	int main(int argc, char** argv)
	{
		const bool USE_ARTIFICIAL_CMD_LINE = false;  
		
		Constants::instance()->single_arg_file() = false;
		const string LOGGER_NAME = "main";
		
		if ((argc == 1) && USE_ARTIFICIAL_CMD_LINE) {			
			string cmd_line("  /gobics/home/ingo/workspace/HIVCoal/testing/HIVCoal --sim --iter 1 --classftn-file /gobics/home/ingo/workspace/HIVCoal/input/classifications/CRF3-3-a.txt --classftn-name CRF3-3-a --seqs /c1/scratch/ingo/HIVCoal/temp/sim_tip_seqs_10871 --sites 8 --alpha 0.41565 --theta 0.625982 --stamp 10872 --seed 0  ");  
			boost::regex e("\\s*(\\S.*\\S)\\s*");
			cmd_line = regex_replace(cmd_line, e, "\\1", boost::match_default | boost::format_sed);
			
			vector<string> v; 
		    split(v, cmd_line, is_any_of(" "));

		    argc = v.size();
		    argv = new char*[argc];
		    for (int i = 0; i < v.size(); ++i) {
		    	argv[i] = new char[v[i].length()];
		    	strcpy(argv[i], v[i].c_str());
		    }
		}
		
		//try {
			if (argc == 1) {
				Dot::set_para(1, 1,	   // anc
							  0, 0,    // suc
							  0,       // anc_suc_num
							  0, 0,    // next
							  0, 0,    // prev
							  0, 0,    // seq (tips, interior)
							  1,       // sub
							  0,       // affected
							  0,       // alias_groups
							  0, 0,	   // first_last
							  0, 0,    // mut_rate (int), is log prob
							  0, 0, 0, // acid_prob_grp, acid_prob, br_ac_probs,
							  0, 0,    // old_acid_prob, old_br_ac_probs
							  0, 0, 0, // prior_prob, nodes/sites
							  2.0);    // min_edge_len
				Constants::instance()->messages_on() = true;
				Constants::instance()->compile_dot() = true;
				Constants::instance()->compile_dot_immediately() = false;
				Constants::instance()->use_common_output_file() = true;
				Constants::instance()->delete_spec_output_file() = false;
				Dot::compile_to_printable() = true;
				Constants::instance()->compile_sim_tree() = true;
				Constants::instance()->compile_best_tree() = true;
				Constants::instance()->use_scratch() = false;
				
				PropertyConfigurator::configure(In_Out_Streams::instance()->filename(LOG_CONFIG_FILE));
				LoggerPtr logger(Logger::getLogger(LOGGER_NAME));
				LOG4CXX_INFO(logger, "\n\n\n\n\nEntering main");

				ofstream* out_strm = In_Out_Streams::instance()->ostrm(OUTPUT_FILE);
				LOG4CXX_DEBUG(logger, "appl_id: " << Constants::instance()->appl_id());

				//int num_sites = 6;
				/*simple_tree sim_tree = simple_tree(num_sites, Constants::instance()->num_mutation_rates());
				sim_tree.generate();

				VECTOR(string) seq_of_tip_genomes(sim_tree.num_tips());
				for (int i = 0; i < sim_tree.num_tips(); i++)
					seq_of_tip_genomes[i].resize(num_sites);
				seq_of_tip_genomes = sim_tree.sim_seq();
				Dot::to_dot(&sim_tree);

				optim_para(&sim_tree);*/

				int seed = 0;
				Random::init(seed);
				int num_sites = 100;
				
				/*Tip_Seqs tip_seqs(7, num_sites);
				Classftn cls("input/classifications/Basic7.txt", tip_seqs.num_sites());
				MCMC::Results res_mcmc = cls.log_prob_fix(tip_seqs, 1);
				cout << "ln L: " << setprecision(8) << res_mcmc.log_p()
					 << " (prior: " << res_mcmc.log_p_prior()
					 << ", acids: " << res_mcmc.log_p_acids() << ")" << endl; 
				abort();*/
				
				int num_mut_sims = 1;
				Classftn sim_cls("input/classifications/C1.1.txt", num_sites);
				//Classftn sim_cls("input/classifications/AGHJ02-a.txt", num_sites);
				VECTOR( VECTOR(string) ) sim_seqs;
				boost::tie(sim_seqs, tuples::ignore, tuples::ignore) = sim_cls.sim_seqs(num_mut_sims, false);

				for (int i = 0; i < num_mut_sims; ++i) {
					//Tip_Seqs tip_seqs("/c1/scratch/ingo/HIVCoal/temp/sim_tip_seqs_10400_1.txt");
					Tip_Seqs tip_seqs = sim_seqs[i];
					tip_seqs.save("temp/tip_seqs.txt");
					MCMC::Results res_mcmc;
					
					//Classftn right_cls("input/classifications/AGHJ02-a.txt", tip_seqs.num_sites());
					Classftn right_cls("input/classifications/C1.1.txt", tip_seqs.num_sites());
					right_cls.calc_bps(tip_seqs, false);
					//cout << right_cls << endl;
					Random::init(seed);
					res_mcmc = right_cls.log_prob_var(tip_seqs, 100, 1e-0);
					cout << "ln L: " << setprecision(8) << res_mcmc.log_p()
						 << " (prior: " << res_mcmc.log_p_prior() << ", acids: " << res_mcmc.log_p_acids() << ")" << endl;

					/*classification wrong_cls("input/classifications/CRF3-3-deg-0.txt", tip_seqs.num_sites());
					//wrong_cls.calc_bps(tip_seqs);
					*out_strm << wrong_cls;
					res = wrong_cls.log_prob_var(tip_seqs, 3000, 1e-8);
					cout << "ln L: " << setprecision(8) << res.log_p()
						 << " (prior: " << res.log_p_prior << ", acids: " << res.log_p_acids << ")" << endl;*/
				}
			}
			else {
				Dot::set_para(1, 1,	   // anc
							  0, 0,    // suc
							  0,       // anc_suc_num
							  0, 0,    // next
							  0, 0,    // prev
							  0, 0,    // seq (tips, interior)
							  1,       // sub
							  0,       // affected
							  0,       // alias_groups
							  0, 0,	   // first_last
							  0, 0,    // mut_rate (int), is log prob
							  0, 0, 0, // acid_prob_grp, acid_prob, br_ac_probs,
							  0, 0,    // old_acid_prob, old_br_ac_probs
							  0, 0, 0, // prior_prob, nodes/sites
							  2.0);    // min_edge_len

				Constants::instance()->compile_dot() = false;
				Constants::instance()->use_common_output_file() = false;
				Constants::instance()->delete_spec_output_file() = true;
				Dot::compile_to_printable() = false;
				Constants::instance()->use_scratch() = false;
				
				options_description desc("Allowed options");
				desc.add_options()
					("rec", value<double>(), "recombination rate")
					("theta", value<double>(), "Theta")
					("alpha", value<double>(), "alpha")
					("jphmm-jp", value<double>(), "jump probability")
					("jphmm-off", "do not use jpHMM")
					("unknown", "use unknown segments")
					("seqs", value<string>(), "tip sequences")
					("classftn-file", value<string>(), "classification")
					("classftn-name", value<string>(), "classification")
					("sites", value<int>(), "number of sites")
					("brklen", value<int>(), "break length")
					("eps", value<double>(), "epsilon")
					("seed", value<int>(), "Random seed")
					("stamp", value<int>(), "stamp of return file")
					("sim", "perform simulations")
					("iter", value<int>(), "number of simulations")
					("messages", "print messages")
					("arg", "generate ps of simulated / most probable arg");
				
				variables_map vm;
				store(parse_command_line(argc, argv, desc), vm);
				notify(vm);

				string tip_seqs_prefix;
				string classftn_filename;
				string classftn_name;
				int num_sites = 0;
				int mcmc_break_length = 0;
				double mcmc_eps = 0;
				int Random_seed = 0;
				int num_sims = 1;
				bool use_unknown_segms = false;

				if (vm.count("stamp"))
					Constants::instance()->appl_id() = vm["stamp"].as<int>();
				
				PropertyConfigurator::configure(In_Out_Streams::instance()->filename(LOG_CONFIG_FILE));
				LoggerPtr logger(Logger::getLogger(LOGGER_NAME));
				LOG4CXX_INFO(logger, "\n\n\n\n\nEntering main");

				ofstream* out_strm = In_Out_Streams::instance()->ostrm(OUTPUT_FILE);
				LOG4CXX_DEBUG(logger, "appl_id: " << Constants::instance()->appl_id());
				
				if (vm.count("rec"))
					Constants::instance()->recomb_const() = vm["rec"].as<double>();
				if (vm.count("theta"))
					Constants::instance()->theta() = vm["theta"].as<double>();
				if (vm.count("alpha"))
					Constants::instance()->alpha() = vm["alpha"].as<double>();
				if (vm.count("jphmm-jp"))
					Constants::instance()->jump_prob() = vm["jphmm-jp"].as<double>();
				if (vm.count("jphmm-off"))
					Constants::instance()->use_jphmm() = !vm.count("jphmm-off");
				use_unknown_segms = vm.count("unknown");
				if (vm.count("seqs"))
					tip_seqs_prefix = vm["seqs"].as<string>();
				if (vm.count("classftn-file"))
					classftn_filename = vm["classftn-file"].as<string>();
				if (vm.count("classftn-name")) {
					classftn_name = vm["classftn-name"].as<string>();
					In_Out_Streams::instance()->set_sim_best_filenames(classftn_name);
				}
				else
					In_Out_Streams::instance()->set_sim_best_filenames("");
				*out_strm << "cn: " << classftn_name << endl;
				if (vm.count("sites"))
					num_sites = vm["sites"].as<int>();
				if (vm.count("brklen"))
					mcmc_break_length = vm["brklen"].as<int>();
				if (vm.count("eps"))
					mcmc_eps = vm["eps"].as<double>();
				if (vm.count("seed"))
					Random_seed = vm["seed"].as<int>();
				if (vm.count("stamp"))
					In_Out_Streams::instance()->set_results_filename(vm["stamp"].as<int>());
				else
					In_Out_Streams::instance()->set_results_filename(0);
				if (!vm.count("classftn-file"))
					throw Error("Option --classftn-file required");
				if (!vm.count("seqs"))
					throw Error("Option --seqs required");
				if (vm.count("seed"))
					Random::init(Random_seed);
				else
					Random::init();
				if (vm.count("iter"))
					num_sims = vm["iter"].as<int>();
				Constants::instance()->messages_on() = vm.count("messages");
				Constants::instance()->compile_sim_tree() = vm.count("arg");
				Constants::instance()->compile_best_tree() = vm.count("arg");

				if (vm.count("sim")) {
					if (!(vm.count("sites")))
						throw Error("Option --sim requires option --sites");
					Classftn cls(classftn_filename, num_sites);
					VECTOR( VECTOR(string) ) sim_seqs;
					VECTOR(VECTOR(double) ) sep_dists;
					VECTOR(double) noise_dists;
					boost::tie(sim_seqs, sep_dists, noise_dists) = cls.sim_seqs(num_sims, false);
					for (VECTOR( VECTOR(string) )::iterator it = sim_seqs.begin(); it != sim_seqs.end(); ++it) {
						Tip_Seqs tip_seqs = *it;
						stringstream ss;
						ss << tip_seqs_prefix << "_" << it - sim_seqs.begin() << ".txt";
						string tip_seqs_filename = ss.str();
						tip_seqs.save(tip_seqs_filename);
					}

					ofstream* res_strm = new ofstream(In_Out_Streams::instance()->filename(RESULTS_FILE).c_str());
					*res_strm << ">sep_final   # distance between final nodes (first sub, second sub, distance)" << endl;
					for (int i = 0; i < sep_dists.size(); ++i)
						for (int j = i + 1; j < sep_dists.size(); ++j)
							*res_strm << i << " " << j << " " << sep_dists[i][j] << endl;
					*res_strm << ">noise_final   # distance from final nodes to tips (sub, distance)" << endl;
					for (int i = 0; i < noise_dists.size(); ++i)
						*res_strm << i << " " << noise_dists[i] << endl;
					*res_strm << ">end" << endl;
				}
				else {
					stringstream ss;
					ss << tip_seqs_prefix << ".txt";
					Tip_Seqs tip_seqs(ss.str());
					Classftn cls(classftn_filename, tip_seqs.num_sites());
					bool cls_has_crfs = cls.num_crfs();
					if (Constants::instance()->use_jphmm())
						cls.calc_bps(tip_seqs, use_unknown_segms);

					ofstream* res_strm = new ofstream(In_Out_Streams::instance()->filename(RESULTS_FILE).c_str());
					if (cls_has_crfs && !cls.num_crfs())
						*res_strm << "All CRFs deleted" << endl;
					else {
						if (vm.count("seed"))
							Random::init(Random_seed);
						else
							Random::init();
						MCMC::Results res;
						if (vm.count("brklen") && vm.count("eps"))
							res = cls.log_prob_var(tip_seqs, mcmc_break_length, mcmc_eps);
						else
							if (vm.count("brklen"))
								res = cls.log_prob_var(tip_seqs, mcmc_break_length);
							else
								if (vm.count("eps"))
									res = cls.log_prob_var(tip_seqs, mcmc_eps);
								else
									res = cls.log_prob_var(tip_seqs);
						if (Constants::instance()->messages_on())
							cout << "ln L: " << setprecision(8) << res.log_p()
								 << " (prior: " << res.log_p_prior() 
								 << ", acids: " << res.log_p_acids() << ")" << endl;


						*res_strm << Return(res, cls);
					}
				}
			}
		/*}
		catch (Error err) {
			cout << err;
		}*/

		//LOG4CXX_INFO(logger, "Leaving main");

		return (EXIT_SUCCESS);
	}

#endif
