#include "standard.h"

#include "constants.h"
#include "dot.h"
#include "files.h"
#include "log_acid_prob.h"
#include "nd.h"
#include "nc.h"
#include "part_node.h"
#include "part_tree.h"
#include "sim_nd.h"
#include "sim_nc.h"
#include "sim_tree.h"
#include "tree.h"

set<string> Dot::ids_of_printed_nodes;
set<const Partition_Node*> Dot::printed_part_nodes;
Dot_Para Dot::dot_para;
BOOL Dot::m_compile_to_printable = false;



void To_Dot_Str_Visitor::visit(Rev_Tip_Nd* n) { Rev_Nd* m = n; visit(m); }
void To_Dot_Str_Visitor::visit(Rev_Intern_Nd* n) { Rev_Nd* m = n; visit(m); }
void To_Dot_Str_Visitor::visit(Rev_Coal_Nd* n) { Rev_Nd* m = n; visit(m); }
void To_Dot_Str_Visitor::visit(Rev_Rec_Nd* n) { Rev_Nd* m = n; visit(m); }



void To_Dot_Str_Visitor::visit(Rev_Nd* n)
{
	const string COLOR_ACTIVE = "orange";
	const string COLOR_ORDINARY = "grey87";
	const string COLOR_IMPURE = "yellow";
	const int WIDTH_IMPURE = 1;

	const int MAX_NUM_ROWS = 20;

	stringstream ss[MAX_NUM_ROWS];
	int idx = 0;

	ss[idx] << n->id();
	idx++;

	if (dot_para().seq_tip && n->is_tip() || dot_para().seq_intr && !n->is_tip())
		ss[idx] << n->acid_seq();
	idx++;

	if (dot_para().sub)
	{
		ss[idx] << n->sub_segments_to_string();
	}
	if (dot_para().affected)
	{
		BOOL b = false;
		if (const Rev_Intern_Nd* n = dynamic_cast<const Rev_Intern_Nd*>(this))
			if (n->is_suc_affected())
				b = true;
		if (dot_para().sub &&
			(n->is_anc_affected() || b || n->is_pos_affected() || n->is_time_affected() || n->is_nd_st_affected() || n->is_ac_pr_affected()))
				ss[idx] << " ";
		if (n->is_anc_affected())
			ss[idx] << "A";
		if (b)
			ss[idx] << "S";
		if (n->is_pos_affected())
			ss[idx] << "P";
		if (n->is_time_affected())
			ss[idx] << "T";
		if (n->is_nd_st_affected())
			ss[idx] << "E";
		if (n->is_ac_pr_affected())
			ss[idx] << "B";
	}
	idx++;

	if (dot_para().alias_groups) {
		BOOL is_first = true;
		for (set<int>::const_iterator it = n->act_groups().begin(); it != n->act_groups().end(); ++it) {
			if (is_first)
				is_first = false;
			else
				ss[idx] << " ";
			ss[idx] << *it;
		}
	}
	idx++;

	if (dot_para().acid_prob_grp) {
		BOOL is_first = true;
		for (int i = 0; i < n->num_sites(); i++) {
			if (is_first)
				is_first = false;
			else
				ss[idx] << " ";
			if (n->is_act_site(i))
				ss[idx] << n->site_to_prob()[i]->id();
			else
				ss[idx] << "-";
		}
	}
	idx++;

	if (dot_para().acid_prob) {
		for (int i = 0; i < NUM_NUCL; i++) {
			for (int j = 0; j < n->num_sites(); j++) {
				double p;
				if (n->is_act_site(j))
					p = n->lap(dot_para().mut_rate, j, i);
				else
					p = -10000;
				ss[idx] << left << setprecision(4) << setw(9) << (dot_para().is_log_prob ? p : exp(p));
				if (j < n->num_sites() - 1)
					ss[idx] << " ";
			}
			if (i < NUM_NUCL - 1)
				ss[idx] << "<BR/>";
			else
				if (dot_para().br_ac_probs || dot_para().old_acid_prob || dot_para().old_br_ac_probs)
					ss[idx] << "<BR/> ";
		}
	}
	idx++;

	if (dot_para().br_ac_probs)
	{
		if (!n->is_active())
		{
			for (int i = 0; i < NUM_NUCL; i++) {
				for (int j = 0; j < n->num_sites(); j++) {
					double p;
					if (n->is_act_site(j))
						p = n->lap_anc(dot_para().mut_rate, j, i);
					else
						p = -10000;
					ss[idx] << left << setprecision(4) << setw(9) << (dot_para().is_log_prob ? p : exp(p));
					if (j < n->num_sites() - 1)
						ss[idx] << " ";
				}
				if (i < NUM_NUCL - 1)
					ss[idx] << "<BR/>";
				else
					if (dot_para().old_acid_prob || dot_para().old_br_ac_probs)
						ss[idx] << "<BR/> ";
			}
		}
	}
	idx++;

	if (dot_para().old_acid_prob) {
		for (int i = 0; i < NUM_NUCL; i++) {
			for (int j = 0; j < n->num_sites(); j++) {
				double p;
				if (n->is_act_site(j))
					p = n->old_lap(dot_para().mut_rate, j, i);
				else
					p = -10000;
				ss[idx] << left << setprecision(4) << setw(9) << (dot_para().is_log_prob ? p : exp(p));
				if (j < n->num_sites() - 1)
					ss[idx] << " ";
			}
			if (i < NUM_NUCL - 1)
				ss[idx] << "<BR/>";
			else
				if (dot_para().old_br_ac_probs)
					ss[idx] << "<BR/> ";
		}
	}
	idx++;

	if (dot_para().old_br_ac_probs)
	{
		if (!n->is_active())
		{
			for (int i = 0; i < NUM_NUCL; i++) {
				for (int j = 0; j < n->num_sites(); j++) {
					double p;
					if (n->is_act_site(j))
						p = n->old_lap_anc(dot_para().mut_rate, j, i);
					else
						p = -10000;
					ss[idx] << left << setprecision(4) << setw(9) << (dot_para().is_log_prob ? p : exp(p));
					if (j < n->num_sites() - 1)
						ss[idx] << " ";
				}
				if (i < NUM_NUCL - 1)
					ss[idx] << "<BR/>";
			}
		}
	}
	idx++;

	if (dot_para().prior_prob)
	{
		if (!n->is_tip())
		{
			ss[idx] << "P_pr: "<< setprecision(4) << n->unspec_node_contnr()->P_prior() << " ";
			ss[idx] << "P_ev: "<< setprecision(4) << n->unspec_node_contnr()->P_gen_event() << "<BR/>";
			ss[idx] << "I_coal: " << setprecision(3) << n->unspec_node_contnr()->coal_intensity() << " ";
			ss[idx] << "I_rec: " << setprecision(3) << n->unspec_node_contnr()->recomb_intensity();
		}
	}
	idx++;

	if (dot_para().elig_nodes_sites || dot_para().elig_nodes_sites_old)
	{
		ss[idx] << "Rev_Nds: " << n->unspec_node_contnr()->num_elig_nodes();
		ss[idx] << " ";

		ss[idx] << "Sites: " << n->unspec_node_contnr()->num_cum_elig_sites();
		ss[idx] << " ";
	}
	idx++;

	BOOL b = false;
	if (const Rev_Rec_Nd* n = dynamic_cast<const Rev_Rec_Nd*>(this))
		if (n->is_first_in_NC())
			b = true;

	stringstream ret;
	string main_color = (n->is_active() ? COLOR_ACTIVE : COLOR_ORDINARY);
	ret << "<TD BGCOLOR=\"" << (n->is_pure_node() ? main_color : COLOR_IMPURE) << "\" WIDTH=\"" << WIDTH_IMPURE << "\"></TD>";
	ret << "<TD ";
	ret << "PORT=\"N" << ((n->is_single() || n->is_double() && b) ? 0 : 1) << "\" ";
	ret << "BGCOLOR=\"" << main_color << "\" ";
	ret << ">";
	for (int i = 0; i < MAX_NUM_ROWS; ++i)
		if (ss[i].str().length() > 0)
			ret << ss[i].str() << "<BR/>";
	ret << "</TD>";
	ret << "<TD BGCOLOR=\"" << (n->is_pure_node() ? main_color : COLOR_IMPURE) << "\" WIDTH=\"" << WIDTH_IMPURE << "\"></TD>";
	m_str = ret.str();
}



void Dot::to_dot(Sim_NC* c, set<string>* ids_of_printed_contnrs)
{
	const string COLOR_COAL_ANC = "darkorange2";
	const string COLOR_COAL_SUC = "red";
	const string COLOR_RECOMB_ANC = "purple";
	const string COLOR_RECOMB_SUC = "blue";
	const string COLOR_NEXT = "black";
	const string COLOR_PREV = "grey50";
	const string STYLE_OLD = "dashed";
	const double ARROW_SIZE_HERTG = 0.8;
	const double ARROW_SIZE_POS = 0.5;

	ofstream* Dot_strm = In_Out_Streams::instance()->ostrm(ARG_DOT_FILE);
	pair<set<string>::iterator, BOOL> ret;
	ret = ids_of_printed_contnrs->insert(c->str_id());

	if (ret.second == true)
	{
		VECTOR(Sim_Nd*) nodes = c->unspec_nodes();
		*Dot_strm << c->to_dot_string(dot_para);

		// anc and anc_old
		if (dot_para.anc)
		{
			for (VECTOR(Sim_Nd*)::iterator it = nodes.begin(); it != nodes.end(); ++it)
				if (!(*it)->is_active())
				{
					*Dot_strm << (*it)->to_dot_node_name() << " -> ";

					if ((*it)->parent_contnr())
						*Dot_strm << (*it)->parent_contnr()->to_dot_node_name();
					else
						*Dot_strm << "\"NULL\"";

					*Dot_strm << " [ color = ";
					if ((*it)->has_rec_anc())
						*Dot_strm << COLOR_RECOMB_ANC;
					else
						*Dot_strm << COLOR_COAL_ANC;

					if (dot_para.anc_suc_num && (*it)->has_coal_anc())
						if ((*it)->is_first_child())
							*Dot_strm << ", label = \"1\"";
						else
							*Dot_strm << ", label = \"2\"";

					*Dot_strm <<", arrowsize = " << ARROW_SIZE_HERTG << " ]" << endl;

					//if ((*it)->parent_node())
					//	Dot::to_dot((*it)->parent_contnr(), ids_of_printed_contnrs);
				}
		}

		// suc and suc_old
		if (dot_para.suc)
		{
			VECTOR(Sim_Nd*) children = c->unspec_children();
			for (VECTOR(Sim_Nd*)::iterator it = children.begin(); it != children.end(); ++it)
			{
				*Dot_strm << c->to_dot_node_name() << " -> ";
				if (*it)
					*Dot_strm << (*it)->to_dot_node_name();
				else
					*Dot_strm << "\"NULL\"";
				*Dot_strm << "[ arrowsize = " << ARROW_SIZE_HERTG;

				*Dot_strm << ", color = ";
				if (c->has_rec_suc())
					*Dot_strm << COLOR_RECOMB_SUC;
				else
					*Dot_strm << COLOR_COAL_SUC;

				if (dot_para.anc_suc_num && (*it)->has_coal_suc())
					if ((*it)->is_first_child())
						*Dot_strm << ", label = \"1\"";
					else
						*Dot_strm << ", label = \"2\"";
				*Dot_strm << " ]" << endl;
			}
		}

		// pos_next
		if (dot_para.pos_next)
		{
			if (!c->is_last())
			{
				*Dot_strm << c->to_dot_node_name() << " -> ";
				if (c->next())
					*Dot_strm << c->next()->to_dot_node_name();
				else
					*Dot_strm << "\"NULL\"";
				*Dot_strm << " [ color = " << COLOR_NEXT << ", arrowsize = " << ARROW_SIZE_POS << " ]" << endl;
			}
		}

		// pos_prev
		if (dot_para.pos_prev)
		{
			if (!c->is_first())
			{
				*Dot_strm << c->to_dot_node_name() << " -> ";
				if (c->prev())
					*Dot_strm << c->prev()->to_dot_node_name();
				else
					*Dot_strm << "\"NULL\"";
				*Dot_strm << " [ color = " << COLOR_PREV << ", arrowsize = " << ARROW_SIZE_POS << " ]" << endl;
			}
		}

	}
}



void Dot::to_dot(Rev_NC* c, set<string>* ids_of_printed_contnrs, Dot_Target_File_Type target_file)
{
	const string COLOR_COAL_ANC = "darkorange2";
	const string COLOR_COAL_SUC = "red";
	const string COLOR_RECOMB_ANC = "purple";
	const string COLOR_RECOMB_SUC = "blue";
	const string COLOR_NEXT = "black";
	const string COLOR_PREV = "grey50";
	const string STYLE_OLD = "dashed";
	const double ARROW_SIZE_HERTG = 0.8;
	const double ARROW_SIZE_POS = 0.5;

	Out_File_Name_Abbrev file_abbrv;
	switch (target_file) {
	case ORDINARY:
		file_abbrv = ARG_DOT_FILE;
		break;
	case SIM:
		file_abbrv = SIM_DOT_FILE;
		break;
	case BEST:
		file_abbrv = BEST_DOT_FILE;
		break;
	}
	ofstream* Dot_strm = In_Out_Streams::instance()->ostrm(file_abbrv);
	pair<set<string>::iterator, BOOL> ret;
	ret = ids_of_printed_contnrs->insert(c->str_id());

	if (ret.second == true)
	{
		VECTOR(Rev_Nd*) nodes = c->unspec_nodes();
		*Dot_strm << c->to_dot_string(dot_para);

		// anc and anc_old
		if (dot_para.anc && !c->is_old() || dot_para.anc_old && c->is_old())
		{
			for (VECTOR(Rev_Nd*)::iterator it = nodes.begin(); it != nodes.end(); ++it)
				if (!(*it)->is_active())
				{
					string str;
					if ((*it)->is_old())
						str = ", style = " + STYLE_OLD ;

					*Dot_strm << (*it)->to_dot_node_name() << " -> ";

					if ((*it)->parent_contnr())
						*Dot_strm << (*it)->parent_contnr()->to_dot_node_name();
					else
						*Dot_strm << "\"NULL\"";

					*Dot_strm << " [ color = ";
					if ((*it)->has_rec_anc())
						*Dot_strm << COLOR_RECOMB_ANC;
					else
						*Dot_strm << COLOR_COAL_ANC;

					if (dot_para.anc_suc_num && (*it)->has_coal_anc())
						if ((*it)->is_first_child())
							*Dot_strm << ", label = \"1\"";
						else
							*Dot_strm << ", label = \"2\"";

					*Dot_strm << str << ", arrowsize = " << ARROW_SIZE_HERTG << " ]" << endl;
				}
		}

		// anc_old
		if (dot_para.anc_old)
		{
			for (VECTOR(Rev_Nd*)::iterator it = nodes.begin(); it != nodes.end(); ++it)
			{
				if (!(*it)->is_active() && (*it)->is_anc_affected() &&
					((*it)->old_parent_contnr() != (*it)->parent_contnr()) &&
					!(*it)->is_old_last())
				{
					string str;
					*Dot_strm << (*it)->to_dot_node_name() << " -> ";

					if ((*it)->parent_contnr())
						*Dot_strm << (*it)->old_parent_contnr()->to_dot_node_name();
					else
						*Dot_strm << "\"NULL\"";

					*Dot_strm << " [ color = ";
					if ((*it)->has_rec_anc())
						*Dot_strm << COLOR_RECOMB_ANC;
					else
						*Dot_strm << COLOR_COAL_ANC;

					if (dot_para.anc_suc_num && (*it)->has_coal_anc())
						if ((*it)->was_first_child())
							*Dot_strm << ", label = \"1\"";
						else
							*Dot_strm << ", label = \"2\"";

					*Dot_strm << ", style = " << STYLE_OLD << ", arrowsize = " << ARROW_SIZE_HERTG << " ]" << endl;
				}
			}
		}

		// suc and suc_old
		if (dot_para.suc && !c->is_old() || dot_para.suc_old && c->is_old())
		{
			string str;
			if (c->is_old())
				str = ", style = " + STYLE_OLD;

			VECTOR(Rev_Nd*) children = c->unspec_children();
			for (VECTOR(Rev_Nd*)::iterator it = children.begin(); it != children.end(); ++it)
			{
				*Dot_strm << c->to_dot_node_name() << " -> ";
				if (*it)
					*Dot_strm << (*it)->to_dot_node_name();
				else
					*Dot_strm << "\"NULL\"";
				*Dot_strm << "[ arrowsize = " << ARROW_SIZE_HERTG << str ;

				*Dot_strm << ", color = ";
				if (c->has_rec_suc())
					*Dot_strm << COLOR_RECOMB_SUC;
				else
					*Dot_strm << COLOR_COAL_SUC;

				if (dot_para.anc_suc_num && (*it)->has_coal_suc())
					if ((*it)->is_first_child())
						*Dot_strm << ", label = \"1\"";
					else
						*Dot_strm << ", label = \"2\"";
				*Dot_strm << " ]" << endl;
			}
		}

		// suc_old
		if (dot_para.suc_old)
		{
			if (Rev_Intern_NC* d = dynamic_cast<Rev_Intern_NC*>(c))
				if (d->is_suc_affected())
				{
					VECTOR(Rev_Nd*) old_children = d->unspec_old_children();
					for (VECTOR(Rev_Nd*)::iterator it = old_children.begin(); it != old_children.end(); ++it)
					{
						*Dot_strm << d->to_dot_node_name() << " -> ";
						if (*it)
							*Dot_strm << (*it)->to_dot_node_name();
						else
							*Dot_strm << "\"NULL\"";
						*Dot_strm << "[ arrowsize = " << ARROW_SIZE_HERTG;

						*Dot_strm << ", color = ";
						if (d->has_rec_suc())
							*Dot_strm << COLOR_RECOMB_SUC;
						else
							*Dot_strm << COLOR_COAL_SUC;

						if (dot_para.anc_suc_num && (*it)->has_coal_suc())
							if ((*it)->is_first_child())
								*Dot_strm << ", label = \"1\"";
							else
								*Dot_strm << ", label = \"2\"";
						*Dot_strm << ", style = " << STYLE_OLD << " ]" << endl;
					}
				}
		}

		// pos_next
		if (dot_para.pos_next)
		{
			if (!c->is_last() && !c->is_old())
			{
				*Dot_strm << c->to_dot_node_name() << " -> ";
				if (c->next())
					*Dot_strm << c->next()->to_dot_node_name();
				else
					*Dot_strm << "\"NULL\"";
				*Dot_strm << " [ color = " << COLOR_NEXT << ", arrowsize = " << ARROW_SIZE_POS << " ]" << endl;
			}
		}

		// pos_next_old
		if (dot_para.pos_next_old)
		{
			if (c->is_old())
			{
				*Dot_strm << c->to_dot_node_name() << " -> ";
				if (c->next())
					*Dot_strm << c->next()->to_dot_node_name();
				else
					*Dot_strm << "\"NULL\"";
				*Dot_strm << " [ color = " << COLOR_NEXT << ", style = " << STYLE_OLD << ", arrowsize = "
						  << ARROW_SIZE_POS << " ]" << endl;
			}
			else
			{
				if ((c->is_pos_affected()) && c->is_unmarked())
				{
					if ((c->next() == NULL) || (c->old_next() == NULL) || (c->next() != c->old_next()))
					{
						*Dot_strm << c->to_dot_node_name() << " -> ";
						if ((c->next()) && (c->old_next()))
							*Dot_strm << c->old_next()->to_dot_node_name();
						else
							*Dot_strm << "\"NULL\"";
						*Dot_strm << " [ color = " << COLOR_NEXT << ", style = " << STYLE_OLD << ", arrowsize = "
								  << ARROW_SIZE_POS << " ]" << endl;
					}
				}
			}
		}

		// pos_prev
		if (dot_para.pos_prev)
		{
			if (!c->is_first() && !c->is_old())
			{
				*Dot_strm << c->to_dot_node_name() << " -> ";
				if (c->prev())
					*Dot_strm << c->prev()->to_dot_node_name();
				else
					*Dot_strm << "\"NULL\"";
				*Dot_strm << " [ color = " << COLOR_PREV << ", arrowsize = " << ARROW_SIZE_POS << " ]" << endl;
			}
		}

		// pos_prev_old
		if (dot_para.pos_prev_old)
		{
			if (c->is_old())
			{
				*Dot_strm << c->to_dot_node_name() << " -> ";
				if (c->prev())
					*Dot_strm << c->prev()->to_dot_node_name();
				else
					*Dot_strm << "\"NULL\"";
				*Dot_strm << " [ color = " << COLOR_PREV << ", style = " << STYLE_OLD << ", arrowsize = "
						  << ARROW_SIZE_POS << " ]" << endl;
			}
			else
			{
				if ((c->is_pos_affected()) && c->is_unmarked())
				{
					if ((c->prev() == NULL) || (c->old_prev() == NULL) || (c->prev() != c->old_prev()))
					{
						*Dot_strm << c->to_dot_node_name() << " -> ";
						if ((c->prev()) && (c->old_prev()))
							*Dot_strm << c->old_prev()->to_dot_node_name();
						else
							*Dot_strm << "\"NULL\"";
						*Dot_strm << " [ color = " << COLOR_PREV << ", style = " << STYLE_OLD << ", arrowsize = "
								  << ARROW_SIZE_POS << " ]" << endl;
					}
				}
			}
		}
	}
}



// convert time to name of time Rev_Nd
string Dot::time_to_node_name(const double t)
{
	stringstream ss;
	ss << "Time_";
	ss.setf(ios::fixed, ios::floatfield);
	ss << setprecision(16);
	ss << t;
	string str = ss.str();
	for (unsigned int i = 0; i < str.length(); i++)
		if (str.substr(i,1) == ".")
			str.replace(i, 1, "_");
	return str;
}



// generate Dot file of ARG
void Dot::to_dot(Sim_Tree* tr, string name)
{
	ofstream* Dot_strm = In_Out_Streams::instance()->ostrm(ARG_DOT_FILE);
	*Dot_strm << "digraph { " << endl;
	if (compile_to_printable()) {
		*Dot_strm << "size=\"7, 10\"" << endl;
		*Dot_strm << "page=\"8.5, 11\"" << endl;
	}
	*Dot_strm << "ordering = out" << endl;

	// determine times of generations of the nodes
	set<double> times_of_generation;
	for (Sim_NC* c = tr->first_node_contnr(); c; c = c->next())
	{
		times_of_generation.insert(c->birth_time());
	}

	// show time nodes
	*Dot_strm << "Rev_Nd [ shape = plaintext, fontsize = 15, fontname = \"Courier\" ];" << endl;
    Dot_strm->setf(ios::fixed,ios::floatfield);
	for (set<double>::reverse_iterator it = times_of_generation.rbegin(); it != times_of_generation.rend(); it++)
	{
		*Dot_strm << time_to_node_name(*it);
		*Dot_strm << setprecision(5) << " [ label = "<< *it << "]" << endl;
	}
	BOOL is_first = true;
	double prev;
	for (set<double>::reverse_iterator it = times_of_generation.rbegin(); it != times_of_generation.rend(); it++)
	{
		if (is_first)
		{
			is_first = false;
		}
		else
		{
			*Dot_strm << time_to_node_name(prev) << " -> " << time_to_node_name(*it)
			          << " [ minlen = " << dot_para.min_edge_len << " ]" << endl;
		}
		prev = *it;
	}

	// show normal nodes
	ids_of_printed_nodes.clear();
	*Dot_strm << "Rev_Nd [ shape = box, style = filled, fontsize = 15, fontname = \"Courier\" ];" << endl;
	for (Sim_NC* c = tr->first_node_contnr(); c; c = c->next())
	{
		Dot::to_dot(c, &ids_of_printed_nodes);
	}

	// put normal and time nodes on same level
	for (Sim_NC* c = tr->first_node_contnr(); c; c = c->next())
	{
		*Dot_strm << "{ rank = same; " << time_to_node_name(c->birth_time()) << "; ";
		*Dot_strm << c->to_dot_node_name() << "; }" << endl;
	}

	// show name of figure
	if (name != "")
	{
		*Dot_strm << "\"" << name << "\" [ fontsize = 17, color = magenta, fontname = \"Courier\" ]" << endl;
		*Dot_strm << "{ rank = same; \"" << name << "\"; " << time_to_node_name(tr->last_node()->birth_time())
				  << "; }" << endl;
	}

	// show first and last Rev_Nd of Rev_Tree
	if (dot_para.first_last)
	{
		*Dot_strm << "Rev_Nd [ shape = plaintext, style = solid, fontsize = 20, fontname = \"Times-Roman\" ];" << endl;
		*Dot_strm << "\"first\" -> " << tr->first_node_contnr()->to_dot_node_name() << endl;
		*Dot_strm << "\"last\" -> " << tr->last_node_contnr()->to_dot_node_name() << endl;
	}

	*Dot_strm << "}" << endl << endl;

	In_Out_Streams::instance()->no_arg_Dot()++;
	In_Out_Streams::instance()->is_first_arg_Dot() = true;
}



// generate Dot file of ARG
void Dot::to_dot(Rev_Tree* tr, string name, Dot_Target_File_Type target_file_type)
{
	const int FONTSIZE_NORMAL = 15;
	const int FONTSIZE_TIME = 15;

	Out_File_Name_Abbrev file_abbrv;
	switch (target_file_type) {
	case ORDINARY:
		file_abbrv = ARG_DOT_FILE;
		break;
	case SIM:
		file_abbrv = SIM_DOT_FILE;
		break;
	case BEST:
		file_abbrv = BEST_DOT_FILE;
		break;
	}
	ofstream* Dot_strm = In_Out_Streams::instance()->ostrm(file_abbrv);
	*Dot_strm << "digraph { " << endl;
	if (compile_to_printable()) {
		*Dot_strm << "size=\"7, 10\"" << endl;
		*Dot_strm << "page=\"8.5, 11\"" << endl;
	}

	*Dot_strm << "ordering = out" << endl;

	// determine times of generations of the nodes
	set<double> times_of_generation;
	for (Rev_NC* c = tr->first_node_contnr(); c; c = c->next())
	{
		times_of_generation.insert(c->birth_time());
	}

	Rev_NC* c = tr->old_first_node_contnr();
	while (c)
	{
		times_of_generation.insert(c->birth_time());
		if (c->is_pos_affected())
			c = c->old_next();
		else
			c = c->next();
	}

	// show time nodes
	*Dot_strm << "Node [ shape = plaintext, fontsize = " << FONTSIZE_TIME << ", fontname = \"Courier\" ];" << endl;
    Dot_strm->setf(ios::fixed,ios::floatfield);
	for (set<double>::reverse_iterator it = times_of_generation.rbegin(); it != times_of_generation.rend(); it++)
	{
		*Dot_strm << time_to_node_name(*it);
		*Dot_strm << setprecision(5) << " [ label = "<< *it << "]" << endl;
	}
	BOOL is_first = true;
	double prev;
	for (set<double>::reverse_iterator it = times_of_generation.rbegin(); it != times_of_generation.rend(); it++)
	{
		if (is_first)
		{
			is_first = false;
		}
		else
		{
			*Dot_strm << time_to_node_name(prev) << " -> " << time_to_node_name(*it)
			          << " [ minlen = " << dot_para.min_edge_len << " ]" << endl;
		}
		prev = *it;
	}

	// show normal nodes
	ids_of_printed_nodes.clear();
	*Dot_strm << "Node [ shape = box, style = filled, fontsize = " << FONTSIZE_NORMAL
	   		  << ", fontname = \"Courier\" ];" << endl;
	for (Rev_NC* c = tr->first_node_contnr(); c; c = c->next())
	{
		Dot::to_dot(c, &ids_of_printed_nodes, target_file_type);
	}

	c = tr->old_first_node_contnr();
	while (c)
	{
		Dot::to_dot(c, &ids_of_printed_nodes, target_file_type);
		if (c->is_pos_affected())
			c = c->old_next();
		else
			c = c->next();
	}

	// put normal and time nodes on same level
	for (Rev_NC* c = tr->first_node_contnr(); c; c = c->next())
	{
		*Dot_strm << "{ rank = same; " << time_to_node_name(c->birth_time()) << "; ";
		*Dot_strm << c->to_dot_node_name() << "; }" << endl;
	}

	c = tr->old_first_node_contnr();
	while (c)
	{
		*Dot_strm << "{ rank = same; " << time_to_node_name(c->birth_time()) << "; ";
		*Dot_strm << c->to_dot_node_name() << "; }" << endl;
		if (c->is_pos_affected())
			c = c->old_next();
		else
			c = c->next();
	}

	// show name of figure
	if (name != "")
	{
		*Dot_strm << "\"" << name << "\" [ fontsize = 17, color = magenta, fontname = \"Courier\" ]" << endl;
		*Dot_strm << "{ rank = same; \"" << name << "\"; " << time_to_node_name(tr->last_node()->birth_time())
				  << "; }" << endl;
	}

	// show first and last Rev_Nd of Rev_Tree
	if (dot_para.first_last)
	{
		*Dot_strm << "Rev_Nd [ shape = plaintext, style = solid, fontsize = 20, fontname = \"Times-Roman\" ];" << endl;
		*Dot_strm << "\"first\" -> " << tr->first_node_contnr()->to_dot_node_name() << endl;
		*Dot_strm << "\"last\" -> " << tr->last_node_contnr()->to_dot_node_name() << endl;
	}

	// show old first and old last Rev_Nd of Rev_Tree (before changes)
	if (dot_para.first_last_old)
	{
		*Dot_strm << "Rev_Nd [ shape = plaintext, style = solid, fontsize = 20, fontname = \"Times-Roman\"  ];" << endl;
		if (tr->old_first_node_contnr())
			*Dot_strm << "\"first(old)\" -> " << tr->old_first_node_contnr()->to_dot_node_name() << endl;
		if (tr->old_last_node_contnr())
			*Dot_strm << "\"last(old)\" -> " << tr->old_last_node_contnr()->to_dot_node_name() << endl;
	}

	*Dot_strm << "}" << endl << endl;

	if (target_file_type == ORDINARY) {
		In_Out_Streams::instance()->no_arg_Dot()++;
		In_Out_Streams::instance()->is_first_arg_Dot() = true;
	}
}



string level_to_node_name(int i)
{
	stringstream ss;
	ss << "level_" << i;
	return ss.str();
}



void Dot::to_dot(const Partition_Node* n)
{
	const string COLOR_COAL = "red";
	const string COLOR_RECOMB = "blue";

	ofstream* Dot_strm = In_Out_Streams::instance()->ostrm(PART_TREE_DOT_FILE);

	pair<set<const Partition_Node*>::iterator, BOOL> ret;
	ret = printed_part_nodes.insert(n);

	if (ret.second == true)
	{
		*Dot_strm << "\"" << n->to_string() << "\"" << endl;

		for (set<Suc_Coal>::const_iterator it = n->suc_coal().begin(); it != n->suc_coal().end(); it++)
		{
			*Dot_strm << "\"" << n->to_string() << "\"" << "->" << "\"" << it->target->to_string() << "\""
			          << "[ " << "COLOR = " << COLOR_COAL << " ]" << endl;
			to_dot(it->target);
		}

		for (set<Suc_Rec>::const_iterator it = n->suc_recomb().begin();
		     it != n->suc_recomb().end(); it++)
		{
			*Dot_strm << "\"" << n->to_string() << "\"" << "->" << "\"" << it->target->to_string() << "\""
			          << "[ " << "COLOR = " << COLOR_RECOMB << " ]" << endl;
			to_dot(it->target);
		}
	}

}



void Dot::to_dot(const Partition_Tree* tr)
{
	ofstream* Dot_strm = In_Out_Streams::instance()->ostrm(PART_TREE_DOT_FILE);
	*Dot_strm << "digraph { " << endl;
	*Dot_strm << "ordering = out" << endl;

	// show level nodes
	int max_level = 0;
	for (VECTOR(Partition_Node*)::const_iterator it = tr->part_nodes().begin();
		 it != tr->part_nodes().end(); it++)
		max_level = GSL_MAX(max_level, (*it)->level());

	*Dot_strm << "Node [ shape = plaintext, fontsize = 14 ];" << endl;
	for (int i = 0; i <= max_level; i++)
	{
		*Dot_strm << level_to_node_name(i);
		*Dot_strm << " [ label = \"Level "<< i << "\" ]" << endl;
	}

	for (int i = 0; i < max_level; i++)
		*Dot_strm << level_to_node_name(i) << " -> " << level_to_node_name(i + 1) << endl;

	// show normal nodes
	printed_part_nodes.clear();
	*Dot_strm << "Node [ shape = box, style = filled, fontsize = 15, fontname = \"Courier\" ];" << endl;
	to_dot(tr->root());

	*Dot_strm << "}" << endl << endl;
}



// convert S and P to Dot file
void Dot::to_dot(Rev_NC* start, Rev_NC* end, Rev_NC* third_child)
{
	const double MIN_EDGE_LEN = 1.0;
	const int NODE_WIDTH = 14;

	ofstream* Dot_strm = In_Out_Streams::instance()->ostrm(REARRANGE_COAL_DOT_FILE, true);
	*Dot_strm << "digraph { " << endl;
	*Dot_strm << "ordering = out" << endl;
	*Dot_strm << "ranksep = 1.3;" << endl;

	// show time nodes
	*Dot_strm << "Node [ shape = plaintext, fontsize = 14 ];" << endl;
    Dot_strm->setf(ios::fixed,ios::floatfield);
	Dot_strm->precision(5);
	Rev_NC* c = start;
	BOOL end_reached = false;
	do
	{
		if (*c == *end)
			end_reached = true;
		*Dot_strm << "Time_" << c->str_id()
		          << " [ label = \"" << c->birth_time() << " (" << c->str_id() << ")\" ";
		if ((third_child) && (*c == *third_child))
			*Dot_strm << ", fontcolor = red";
		*Dot_strm << "]" << endl;
		if (*c != *end)
			do
				c = c->next();
			while (!c->is_unmarked() && (*c != *end));
	}
	while (!end_reached);

	// connect time nodes
	c = end;
	BOOL is_first = true;
	end_reached = false;
	do
	{
		if (*c == *start)
			end_reached = true;
		if (!is_first)
			*Dot_strm << " -> ";
		*Dot_strm << "Time_" << c->str_id();
		is_first = false;
		if (*c != *start)
			do
				c = c->prev();
			while (!c->is_unmarked() && (*c != *start));
	}
	while (!end_reached);
	*Dot_strm << " [ minlen = " << MIN_EDGE_LEN << " ]" << endl;

	// show ordinary nodes
	*Dot_strm << "Node [ shape = box, style = filled, fontsize = 14 ];" << endl;
	c = start;
	end_reached = false;
	do
	{
		Dot_strm->precision(5);
		if (*c == *end)
			end_reached = true;
		*Dot_strm << "SS_" << c->str_id() << " [shape = record, width = " << NODE_WIDTH << ", label = \"";
		for (int i = 0; i < S_and_P::num_S; i++)
		{
			*Dot_strm << "<S" << i << "> ";
			if (c->index() == i)
				*Dot_strm << "*S" << i << "*";
			else
				*Dot_strm << "S" << i;
			*Dot_strm << "\\n" << c->s_and_p().S(i);
			if (i < S_and_P::num_S - 1)
				*Dot_strm << "|";
			else
				*Dot_strm << "\"];" << endl;
		}

		*Dot_strm << "{ rank = same; SS_" << c->str_id() << "; "
		          << "Time_" << c->str_id() << "; }" << endl;

		Dot_strm->precision(3);
		if (!end_reached)
			for (int i = 0; i < S_and_P::num_S ; i++)
				for (int j = 0; j < S_and_P::num_S; j++)
				{
					Rev_NC* d = c;
					if (*d != *end)
						do
							d = d->next();
						while (!d->is_unmarked() && (*d != *end));

					*Dot_strm << "SS_" << c->str_id() << ":S" << i << "-> SS_" << d->str_id() << ":S" << j;
					if (c->s_and_p().P(i, j) > 0)
					{
						*Dot_strm << "[ label = \"" << c->s_and_p().P(i, j) << "\" ";
						if (third_child) {
							bool make_red = false;
							if (c->index() == i) {
								if (*d != *third_child) { 
									if (d->index() == j)
										make_red = true;
								}
								else {
									if (((d->index() == S_and_P::THIRD_LEFT) && (j == S_and_P::NONE_LEFT)) ||
									    ((d->index() == S_and_P::ALL_LEFT) && (j == S_and_P::FIRST_TWO_LEFT)))
										make_red = true;
								}
							}
							if (make_red)
								*Dot_strm << ", style = bold, color = red";
						}
						*Dot_strm << "]" << endl;
					}
					else
						*Dot_strm << "[ style = invis ]" << endl;
				}

		if (*c != *end)
			do
				c = c->next();
			while (!c->is_unmarked() && (*c != *end));
	}
	while (!end_reached);

	*Dot_strm << "}" << endl << endl;
}



// set parameters for generation of Dot file of ARG
void Dot::set_para(BOOL anc_para, BOOL anc_old_para, BOOL suc_para, BOOL suc_old_para, BOOL anc_suc_num_para,
		 		   BOOL pos_next_para, BOOL pos_next_old_para, BOOL pos_prev_para, BOOL pos_prev_old_para,
		 		   BOOL seq_tip_para, BOOL seq_intr_para, BOOL sub_para, BOOL affected_para,
		 		   BOOL alias_groups_para, BOOL first_last_para,
		 		   BOOL first_last_old_para, int mut_rate_para, BOOL is_log_prob_para,
		 		   BOOL acid_prob_grp_para, BOOL acid_prob_para, BOOL br_ac_probs_para,
		 		   BOOL old_acid_prob_para,  BOOL old_br_ac_probs_para, BOOL prior_prob_para, BOOL elig_nodes_sites_para,
		 		   BOOL elig_nodes_sites_old_para, double min_edge_len_para)
{
	dot_para.anc = anc_para;
	dot_para.anc_old = anc_old_para;
	dot_para.suc = suc_para;
	dot_para.suc_old = suc_old_para;
	dot_para.anc_suc_num = anc_suc_num_para;
	dot_para.pos_next = pos_next_para;
	dot_para.pos_next_old = pos_next_old_para;
	dot_para.pos_prev = pos_prev_para;
	dot_para.pos_prev_old = pos_prev_old_para;
	dot_para.seq_tip = seq_tip_para;
	dot_para.seq_intr = seq_intr_para;
	dot_para.sub = sub_para;
	dot_para.affected = affected_para;
	dot_para.alias_groups = alias_groups_para;
	dot_para.first_last = first_last_para;
	dot_para.first_last_old = first_last_old_para;
	dot_para.mut_rate = mut_rate_para;
	dot_para.is_log_prob = is_log_prob_para;
	dot_para.acid_prob_grp = acid_prob_grp_para;
	dot_para.acid_prob = acid_prob_para;
	dot_para.br_ac_probs = br_ac_probs_para;
	dot_para.old_acid_prob = old_acid_prob_para;
	dot_para.old_br_ac_probs = old_br_ac_probs_para;
 	dot_para.prior_prob = prior_prob_para;
 	dot_para.elig_nodes_sites = elig_nodes_sites_para;
 	dot_para.elig_nodes_sites_old = elig_nodes_sites_old_para;
 	dot_para.min_edge_len = min_edge_len_para;
}

