#ifndef DOT_H
#define DOT_H

#include "standard.h"

#include "nd_visitor.h"

class Partition_Node;
class Partition_Tree;
class Rev_Nd;
class Rev_Tip_Nd;
class Rev_Intern_Nd;
class Rev_Coal_Nd;
class Rev_Rec_Nd;
class Rev_NC;
class Rev_Tree;
class Sim_NC;
class Sim_Tree;



struct Dot_Para { BOOL anc; BOOL anc_old; BOOL suc; BOOL suc_old; BOOL anc_suc_num; BOOL pos_next; BOOL pos_next_old;
 				  BOOL pos_prev; BOOL pos_prev_old; BOOL seq_tip; BOOL seq_intr; BOOL sub; BOOL affected;
 				  BOOL alias_groups; BOOL first_last; BOOL first_last_old; int mut_rate; BOOL is_log_prob;
 				  BOOL acid_prob_grp; BOOL acid_prob; BOOL br_ac_probs;
				  BOOL old_acid_prob; BOOL old_br_ac_probs; BOOL prior_prob;
				  BOOL elig_nodes_sites; BOOL elig_nodes_sites_old; double min_edge_len; };



class To_Dot_Str_Visitor : public Nd_Visitor
{
private:
	Dot_Para m_dot_para;
	string m_str;
public:
	To_Dot_Str_Visitor(Dot_Para dot_para) : m_dot_para(dot_para) { }

	void visit(Rev_Nd* n);
	void visit(Rev_Tip_Nd* n);
	void visit(Rev_Intern_Nd* n);
	void visit(Rev_Coal_Nd* n);
	void visit(Rev_Rec_Nd* n);

	const Dot_Para& dot_para() const { return m_dot_para; }
	operator string() { return m_str; }
};



class Dot
{
private:
	static BOOL m_compile_to_printable;
public:
	enum Dot_Target_File_Type { ORDINARY, SIM, BEST };

	static set<string> ids_of_printed_nodes;
	static set<const Partition_Node*> printed_part_nodes;
	static Dot_Para dot_para;

	static string time_to_node_name(const double t);
	static void to_dot(Sim_NC* c, set<string>* ids_of_printed_contnrs);
	static void to_dot(Rev_NC* n, set<string>* ids_of_printed_nodes, Dot_Target_File_Type target_file);
	static void to_dot(Sim_Tree* tr, string name = "");
	static void to_dot(Rev_Tree* tr, string name = "", Dot_Target_File_Type target_file = ORDINARY);
	static void to_dot(const Partition_Node* n);
	static void to_dot(const Partition_Tree* tr);
	static void to_dot(Rev_NC* start, Rev_NC* end, Rev_NC* third_child);
	static void to_dot(const Rev_Nd* start, const Rev_Nd* end, const Partition_Tree* part_tree);

	static void set_para(BOOL anc_para, BOOL anc_old_para, BOOL suc_para, BOOL suc_old_para, BOOL anc_suc_num_para,
						 BOOL pos_next_para, BOOL pos_next_old_para, BOOL pos_prev_para, BOOL pos_prev_old_para,
						 BOOL seq_tip_para, BOOL seq_intr_para, BOOL sub_para, BOOL affected_para,
						 BOOL alias_groups_para, BOOL first_last_para,
						 BOOL first_last_old_para, int mut_rate_para, BOOL is_log_prob_para,
						 BOOL acid_prob_grp_para, BOOL acid_prob_para, BOOL br_ac_probs_para,
						 BOOL old_acid_prob_para,  BOOL old_br_ac_probs_para, BOOL prior_prob_para, BOOL elig_nodes_sites_para,
						 BOOL elig_nodes_sites_old_para, double min_edge_len_para);

	static BOOL& compile_to_printable() { return m_compile_to_printable; }
};

#endif
