#include "standard.h"

#include "constants.h"
#include "files.h"

In_Out_Streams* In_Out_Streams::m_instance = 0;



string get_time_stamp()
{
	time_t tt = time ((time_t *) 0);
	struct tm *t;
	t = localtime(&tt);
	stringstream ss;
	ss << ((t->tm_mon + 1 < 10) ? "0" : "") << t->tm_mon + 1  << "-"
	   << ((t->tm_mday < 10) ? "0" : "") << t->tm_mday  << "_"
	   << ((t->tm_hour < 10) ? "0" : "") << t->tm_hour  << ":"
	   << ((t->tm_min < 10) ? "0" : "") << t->tm_min  << ":"
	   << ((t->tm_sec < 10) ? "0" : "") << t->tm_sec;
	return ss.str();
}



string Filename_Plus_Status::filename() const
{
	if (m_is_init)
		return m_filename;
	else
		throw Error("Trying to get filename from uninitialized Filename_Plus_Status");
}



In_Out_Streams::In_Out_Streams()
{	
	m_no_arg_Dot = 0;
	m_is_first_arg_Dot = true;
	m_time_stamp = get_time_stamp();
	

	if (Constants::instance()->use_scratch()) {
		m_main_dir_name = "/c1/scratch/ingo/HIVCoal";		
		m_log_dir_name = "/gobics/home/ingo/workspace/HIVCoal";		
		m_temp_dir_name = "/c1/scratch/ingo/HIVCoal/temp";
		m_input_dir_name = "/gobics/home/ingo/workspace/HIVCoal/input";
		m_output_dir_name = "/c1/scratch/ingo/HIVCoal/output";
		m_jphmm_exe_dir_name = "/gobics/home/ingo/JPHMM/src";
		m_jphmm_results_dir_name = "/c1/scratch/ingo/HIVCoal/";
	}	
	else {
		m_main_dir_name = ".";		
		m_log_dir_name = ".";		
		m_temp_dir_name = "temp";
		m_input_dir_name = "input";
		m_output_dir_name = "output";
		m_jphmm_exe_dir_name = "../ext/JPHMM/src";
		m_jphmm_results_dir_name = ".";
	}	
	
	// init output filenames
	stringstream ss;
	ss << output_dir_name() << "/log/output_" << time_stamp() << "_" << Constants::instance()->appl_id() << ".txt";
	m_abbrv_to_filename_out[SPEC_OUTPUT_FILE].set_filename(ss.str());
	if (Constants::instance()->use_common_output_file()) {
		ss.str("");
		ss << output_dir_name() << "/output.txt";
	}
	m_abbrv_to_filename_out[OUTPUT_FILE].set_filename(ss.str());

	ss.str("");
	ss << temp_dir_name() << "/alignment_" << Constants::instance()->appl_id() << ".txt";
	m_abbrv_to_filename_out[JPHMM_ALIGNMENT_FILE].set_filename(ss.str());
	ss.str("");
	ss << temp_dir_name() << "/query_" << Constants::instance()->appl_id() << ".txt";
	m_abbrv_to_filename_out[JPHMM_QUERY_FILE].set_filename(ss.str());

	ss.str("");
	ss << output_dir_name() << "/trees/sim/sim_tree_" << time_stamp() << "_" << Constants::instance()->appl_id() << ".txt";
	m_abbrv_to_filename_out[SIM_DOT_FILE].set_filename(ss.str());
	ss.str("");
	ss << output_dir_name() << "/trees/sim/sim_tree_" << time_stamp() << "_" << Constants::instance()->appl_id() << ".ps";
	m_abbrv_to_filename_out[SIM_PS_FILE].set_filename(ss.str());
	ss.str("");
	ss << output_dir_name() << "/trees/best/best_tree_" << time_stamp() << "_" << Constants::instance()->appl_id() << ".txt";
	m_abbrv_to_filename_out[BEST_DOT_FILE].set_filename(ss.str());
	ss.str("");
	ss << output_dir_name() << "/trees/best/best_tree_" << time_stamp() << "_" << Constants::instance()->appl_id() << ".ps";
	m_abbrv_to_filename_out[BEST_PS_FILE].set_filename(ss.str());

	ss.str("");
	ss << output_dir_name() << "/arg.txt";
	m_abbrv_to_filename_out[ARG_DOT_FILE].set_filename(ss.str());
	ss.str("");
	ss << output_dir_name() << "/rearrange_coal.txt";
	m_abbrv_to_filename_out[REARRANGE_COAL_DOT_FILE].set_filename(ss.str());
	ss.str("");
	ss << output_dir_name() << "/rearrange_recomb.txt";
	m_abbrv_to_filename_out[REARRANGE_RECOMB_DOT_FILE].set_filename(ss.str());
	ss.str("");
	ss << output_dir_name() << "/part_tree.txt";
	m_abbrv_to_filename_out[PART_TREE_DOT_FILE].set_filename(ss.str());
	ss.str("");
	ss << output_dir_name() << "/mcmc.txt";
	m_abbrv_to_filename_out[MCMC_FILE].set_filename(ss.str());
	ss.str("");
	ss << temp_dir_name() << "/results_hivcoal.txt";
	m_abbrv_to_filename_out[RESULTS_FILE].set_filename(ss.str());
	ss.str("");
	ss << temp_dir_name() << "/log_acid_prob_" << Constants::instance()->appl_id() << ".txt";
	m_abbrv_to_filename_out[LOG_ACID_PROB_FILE].set_filename(ss.str());

	// init input filenames
	ss.str("");
	ss << input_dir_name() << "/EigVal.txt";
	m_abbrv_to_filename_in[EIG_VAL_FILE].set_filename(ss.str());
	ss.str("");
	ss << input_dir_name() << "/EigVec.txt";
	m_abbrv_to_filename_in[EIG_VEC_FILE].set_filename(ss.str());
	ss.str("");
	ss << input_dir_name() << "/InvEigVec.txt";
	m_abbrv_to_filename_in[INV_EIG_VEC_FILE].set_filename(ss.str());
	
	ss.str("");
	ss << input_dir_name() << "/tree.txt";
	m_abbrv_to_filename_in[TREE_FILE].set_filename(ss.str());
	ss.str("");
	ss << main_dir_name() << "/recombination.txt";
	m_abbrv_to_filename_in[JPHMM_RESULTS_FILE].set_filename(ss.str());
	ss.str("");
	ss << input_dir_name() << "/unknown_segments.txt";
	m_abbrv_to_filename_in[UNKNOWN_SUB_FILE].set_filename(ss.str());
	ss.str("");
	ss << log_dir_name() << "/log_config.txt";
	m_abbrv_to_filename_in[LOG_CONFIG_FILE].set_filename(ss.str());

	// init io filenames
	ss.str("");
	ss << temp_dir_name() << "/saved_tree_" << Constants::instance()->appl_id() << ".txt";
	m_abbrv_to_filename_io[SAVED_TREE_FILE].set_filename(ss.str());
	ss.str("");
	ss << output_dir_name() << "/output.txt";
	m_abbrv_to_filename_io[COMMON_OUTPUT_FILE].set_filename(ss.str());
}



In_Out_Streams::~In_Out_Streams()
{
	if (Constants::instance()->use_common_output_file()) {
		ifstream* ifs = In_Out_Streams::instance()->istrm(COMMON_OUTPUT_FILE);
		ofstream* ofs = In_Out_Streams::instance()->ostrm(SPEC_OUTPUT_FILE);
		string str;
		while (getline(*ifs, str))
			*ofs << str << '\n';
	}

	if (Constants::instance()->compile_dot())
		if (Constants::instance()->single_arg_file()) {
			stringstream ss;
			ss << "dot -Tps <" << output_dir_name() << "/arg.txt >" << output_dir_name() << "/arg.ps";
			std::system(ss.str().c_str());
		}
		else
			if (!Constants::instance()->compile_dot_immediately())
				for (int i = 0; i < no_arg_Dot(); ++i) {
					stringstream vv;
					vv << "dot -Tps <" << output_dir_name() << "/arg" << i << ".txt >" << output_dir_name() << "/arg" << i << ".ps";
					std::system(vv.str().c_str());
				}

	VECTOR(string) v;
	v.push_back(In_Out_Streams::instance()->filename(JPHMM_ALIGNMENT_FILE));
	v.push_back(In_Out_Streams::instance()->filename(JPHMM_QUERY_FILE));
	v.push_back(In_Out_Streams::instance()->filename(SAVED_TREE_FILE));
	if (Constants::instance()->delete_spec_output_file())
		v.push_back(In_Out_Streams::instance()->filename(SPEC_OUTPUT_FILE));
	if (Constants::instance()->compile_sim_tree())
		if (exists(In_Out_Streams::instance()->filename(SIM_PS_FILE)) && file_size(In_Out_Streams::instance()->filename(SIM_PS_FILE)))
			v.push_back(In_Out_Streams::instance()->filename(SIM_DOT_FILE));
		else
			v.push_back(In_Out_Streams::instance()->filename(SIM_PS_FILE));
	if (Constants::instance()->compile_best_tree())
		if (exists(In_Out_Streams::instance()->filename(BEST_PS_FILE)) && file_size(In_Out_Streams::instance()->filename(BEST_PS_FILE)))
			v.push_back(In_Out_Streams::instance()->filename(BEST_DOT_FILE));
		else
			v.push_back(In_Out_Streams::instance()->filename(BEST_PS_FILE));
	for (VECTOR(string)::iterator it = v.begin(); it != v.end(); ++it)
		remove(*it);

	for (map<Out_File_Name_Abbrev, ofstream*>::iterator it = m_ostrms.begin(); it != m_ostrms.end(); ++it)
		delete it->second;
	for (map<In_File_Name_Abbrev, ifstream*>::iterator it = m_istrms.begin(); it != m_istrms.end(); ++it)
		delete it->second;
}



void In_Out_Streams::set_results_filename(const int i)
{
	stringstream ss;
	ss << temp_dir_name() << "/results_hivcoal_" << i << ".txt";
	m_abbrv_to_filename_out[RESULTS_FILE].set_filename(ss.str());
}



void In_Out_Streams::set_sim_best_filenames(const string str)
{
	ofstream* out_strm = In_Out_Streams::instance()->ostrm(OUTPUT_FILE);

	stringstream ss;
	ss << output_dir_name() << "/trees/sim/sim_tree_" << time_stamp() << "_" << Constants::instance()->appl_id()
	   << (str.length() ? "_" : "") << str << ".txt";
	m_abbrv_to_filename_out[SIM_DOT_FILE].set_filename(ss.str());
	ss.str("");
	ss << output_dir_name() << "/trees/sim/sim_tree_" << time_stamp() << "_" << Constants::instance()->appl_id()
	   << (str.length() ? "_" : "") << str << ".ps";
	m_abbrv_to_filename_out[SIM_PS_FILE].set_filename(ss.str());
	ss.str("");
	ss << output_dir_name() << "/trees/best/best_tree_" << time_stamp() << "_" << Constants::instance()->appl_id()
	   << (str.length() ? "_" : "") << str << ".txt";
	m_abbrv_to_filename_out[BEST_DOT_FILE].set_filename(ss.str());
	ss.str("");
	ss << output_dir_name() << "/trees/best/best_tree_" << time_stamp() << "_" << Constants::instance()->appl_id()
	   << (str.length() ? "_" : "") << str << ".ps";
	m_abbrv_to_filename_out[BEST_PS_FILE].set_filename(ss.str());
}



void In_Out_Streams::set_jphmm_results_filename(const int i)
{
	stringstream ss;
	ss << jphmm_results_dir_name() << "/recombination_" << i << ".txt";
	m_abbrv_to_filename_in[JPHMM_RESULTS_FILE].set_filename(ss.str());
}



ofstream* In_Out_Streams::ostrm(const Out_File_Name_Abbrev abbrv, BOOL reopen)
{
	if ((abbrv != ARG_DOT_FILE) || Constants::instance()->single_arg_file()) {
		if (m_ostrms.find(abbrv) == m_ostrms.end())
			m_ostrms[abbrv] = new ofstream(filename(abbrv).c_str());
		else
			if (reopen) {
				delete m_ostrms[abbrv];
				m_ostrms[abbrv] = new ofstream(filename(abbrv).c_str());
			}

		return m_ostrms[abbrv];
	}
	else {
		static ofstream* strm;
		if (is_first_arg_Dot()) {
			stringstream ss;
			ss << output_dir_name() << "/arg" << no_arg_Dot() << ".txt";
			strm = new ofstream(ss.str().c_str());
			is_first_arg_Dot() = false;
		}
		return strm;
	}
}



ifstream* In_Out_Streams::istrm(const In_File_Name_Abbrev abbrv)
{
	if (m_istrms.find(abbrv) != m_istrms.end())
		delete m_istrms[abbrv];
	m_istrms[abbrv] = new ifstream(filename(abbrv).c_str());

	return m_istrms[abbrv];
}



ofstream* In_Out_Streams::ostrm(const In_Out_File_Name_Abbrev abbrv, BOOL reopen)
{
	if (m_io_ostrms.find(abbrv) == m_io_ostrms.end())
		m_io_ostrms[abbrv] = new ofstream(filename(abbrv).c_str());
	else
		if (reopen) {
			delete m_io_ostrms[abbrv];
			m_io_ostrms[abbrv] = new ofstream(filename(abbrv).c_str());
		}

	return m_io_ostrms[abbrv];
}



ifstream* In_Out_Streams::istrm(const In_Out_File_Name_Abbrev abbrv)
{
	if ((m_io_istrms.find(abbrv) != m_io_istrms.end()))
		delete m_io_istrms[abbrv];
	m_io_istrms[abbrv] = new ifstream(filename(abbrv).c_str());

	return m_io_istrms[abbrv];
}



In_Out_Streams* In_Out_Streams::instance()
{
	if (!m_instance)
		m_instance = new In_Out_Streams();
	return m_instance;
}




