#ifndef FILES_H_
#define FILES_H_

#include "standard.h"

#include "error.h"

enum Out_File_Name_Abbrev { SPEC_OUTPUT_FILE, OUTPUT_FILE, ARG_DOT_FILE, SIM_DOT_FILE, SIM_PS_FILE, BEST_DOT_FILE, BEST_PS_FILE,
							REARRANGE_COAL_DOT_FILE, REARRANGE_RECOMB_DOT_FILE, PART_TREE_DOT_FILE, MCMC_FILE,
						    JPHMM_ALIGNMENT_FILE, JPHMM_QUERY_FILE, RESULTS_FILE, LOG_ACID_PROB_FILE };
enum In_File_Name_Abbrev { EIG_VAL_FILE, EIG_VEC_FILE, INV_EIG_VEC_FILE, TREE_FILE, JPHMM_RESULTS_FILE, UNKNOWN_SUB_FILE, LOG_CONFIG_FILE };
enum In_Out_File_Name_Abbrev { COMMON_OUTPUT_FILE, SAVED_TREE_FILE };



class Filename_Plus_Status
{
private:
	string m_filename;
	BOOL m_is_init;
public:
	string filename() const;
	BOOL is_init() const { return m_is_init; }
	void set_filename(const string filename) { m_is_init = true; m_filename = filename; }
};



class In_Out_Streams
{
private:
	In_Out_Streams();
	~In_Out_Streams();

	static In_Out_Streams* m_instance;

	int m_no_arg_Dot;
	BOOL m_is_first_arg_Dot;
	string m_time_stamp;
	string m_main_dir_name;
	string m_log_dir_name;	
	string m_temp_dir_name;
	string m_input_dir_name;
	string m_output_dir_name;
	string m_jphmm_exe_dir_name;
	string m_jphmm_results_dir_name;

	map<Out_File_Name_Abbrev, Filename_Plus_Status> m_abbrv_to_filename_out;
	map<In_File_Name_Abbrev, Filename_Plus_Status> m_abbrv_to_filename_in;
	map<In_Out_File_Name_Abbrev, Filename_Plus_Status> m_abbrv_to_filename_io;

	map<Out_File_Name_Abbrev, ofstream*> m_ostrms;
	map<In_File_Name_Abbrev, ifstream*> m_istrms;
	map<In_Out_File_Name_Abbrev, ofstream*> m_io_ostrms;
	map<In_Out_File_Name_Abbrev, ifstream*> m_io_istrms;
public:
	void set_results_filename(const int i);
	void set_sim_best_filenames(const string str);
	void set_jphmm_results_filename(const int i);

	int& no_arg_Dot() { return m_no_arg_Dot; }
	BOOL& is_first_arg_Dot() { return m_is_first_arg_Dot; }
	string time_stamp() { return m_time_stamp; }
	string main_dir_name() { return m_main_dir_name; }	
	string log_dir_name() { return m_log_dir_name; }	
	string temp_dir_name() { return m_temp_dir_name; }
	string input_dir_name() { return m_input_dir_name; }
	string output_dir_name() { return m_output_dir_name; }
	string jphmm_exe_dir_name() { return m_jphmm_exe_dir_name; }
	string jphmm_results_dir_name() { return m_jphmm_results_dir_name; }

	ofstream* ostrm(Out_File_Name_Abbrev abbrv, BOOL reopen = false);
	ifstream* istrm(In_File_Name_Abbrev abbrv);
	ofstream* ostrm(In_Out_File_Name_Abbrev abbrv, BOOL reopen = false);
	ifstream* istrm(In_Out_File_Name_Abbrev abbrv);
 	string filename(Out_File_Name_Abbrev abbrv) { return m_abbrv_to_filename_out.find(abbrv)->second.filename(); }
	string filename(In_File_Name_Abbrev abbrv) { return m_abbrv_to_filename_in.find(abbrv)->second.filename(); }
	string filename(In_Out_File_Name_Abbrev abbrv) { return m_abbrv_to_filename_io.find(abbrv)->second.filename(); }

	static In_Out_Streams* instance();
};

#endif
