#ifndef IO_H_
#define IO_H_

#include <list>
#include <map>
#include <vector>

using namespace std;

class Rev_Nd;
class Rev_NC;
class Rev_Intern_NC;
class Rev_Tree;
template<class N, class C, class PC, class T> class Base_NC;
template<class T> class Bound_Det_Manip;
template<class T> ostream& operator<<(ostream& s, const Bound_Det_Manip<T>& bdm);



// *****************************************************************************
// * Output-Manipulators
// *****************************************************************************

template<class T> class Det_Manip
{
	friend ostream& operator<<<T>(ostream& s, const Bound_Det_Manip<T>& bdm);
private:
	int m_level;
public:
	explicit Det_Manip(int level) : m_level(level) { }
	int level() const { return m_level; }
	Bound_Det_Manip<T> operator()(const T* n) const;
};



template<class T> class Bound_Det_Manip
{
private:
	const Det_Manip<T>& m_det_manip;
	const T* m_data;
public:
	Bound_Det_Manip(const Det_Manip<T>& dm, const T* t) : m_det_manip(dm), m_data(t) { }
	const Det_Manip<T>& det_manip() const { return m_det_manip; }
	const T* data() const { return m_data; }
};



ostream& operator<<(ostream& s, const Bound_Det_Manip<Rev_Nd>& bdm);
ostream& operator<<(ostream& s, const Bound_Det_Manip<Rev_NC>& bdm);
ostream& operator<<(ostream& s, const Bound_Det_Manip<Base_NC<Rev_Nd, Rev_NC, Rev_Intern_NC, Rev_Tree> >& bdm);



template<class N, class C, class PC, class T>
ostream& operator<<(ostream& s, const Bound_Det_Manip<Base_NC<N, C, PC, T> >& bdm)
{
	stringstream ss;
	switch (bdm.det_manip().level())
	{
	case 0:
		ss << bdm.data()->str_id() << endl;
		break;
	default:
		ss << "*** Base_NC ***" << endl;
		ss << bdm.data()->str_id() << endl;
		break;
	}

	return s << ss.str();
}



template<class T> Bound_Det_Manip<T> Det_Manip<T>::operator()(const T* n) const
{
	return Bound_Det_Manip<T>(*this, n);
}



// *****************************************************************************
// * Output of STL-container
// *****************************************************************************

template<class T> ostream& operator<<(ostream& s, const vector<T>& v)
{
	for (typename vector<T>::const_iterator it = v.begin(); it != v.end(); ++it)
		s << *it << " ";
	return s;
}



template<class T> ostream& operator<<(ostream& s, const vector<T*>& v)
{
	for (typename vector<T*>::const_iterator it = v.begin(); it != v.end(); ++it) {
		s << **it << " ";
	}
	return s;
}



template<class T> ostream& operator<<(ostream& s, const list<T>& l)
{
	for (typename list<T>::const_iterator it = l.begin(); it != l.end(); ++it)
		s << *it << " ";
	return s;
}



template<class T> ostream& operator<<(ostream& s, const list<T*>& l)
{
	for (typename list<T*>::const_iterator it = l.begin(); it != l.end(); ++it) {
		s << **it << " ";
	}
	return s;
}



template<class T> ostream& operator<<(ostream& s, const set<T>& l)
{
	typename set<T>::const_iterator it = l.begin();
	s << *it;
	++it;
	for (; it != l.end(); ++it)
		s << " " << *it;
	return s;
}



template<class T> ostream& operator<<(ostream& s, const set<T*>& l)
{
	for (typename set<T*>::const_iterator it = l.begin(); it != l.end(); ++it) {
		s << **it << " ";
	}
	return s;
}



template<class K, class V> ostream& operator<<(ostream& s, const map<K, V>& mp)
{
	for (typename map<K, V>::const_iterator it = mp.begin(); it != mp.end(); ++it)
		s << (*it).first << " => " << (*it).second << "   ";
	return s;
}

#endif
