#ifndef MCMC_H
#define MCMC_H

#include "standard.h"

class Rev_Tree;

static LoggerPtr mcmc_logger(Logger::getLogger("classification.MCMC"));



namespace MCMC {
	class Results
	{
		double m_log_p_prior;
		double m_log_p_acids;
		VECTOR(double) m_log_p_acids_pos;   // log prob of nucleotides position-wise
	public:
		Results() { }
		Results(double log_p_prior, double log_p_acids, VECTOR(double) log_p_acids_pos)
		: m_log_p_prior(log_p_prior), m_log_p_acids(log_p_acids), m_log_p_acids_pos(log_p_acids_pos) { }

		double log_p() const { return log_p_prior() + log_p_acids(); }
		double log_p_prior() const{ return m_log_p_prior; }
		double& log_p_prior() { return m_log_p_prior; }
		double log_p_acids() const { return m_log_p_acids; }
		double& log_p_acids() { return m_log_p_acids; }
		VECTOR(double) log_p_acids_pos() const { return m_log_p_acids_pos; }
	};

	ostream& operator<<(ostream& s, const Results& res);

	Results call(Rev_Tree* tr, const int num_steps);
	Results call(Rev_Tree* tr, const int break_length, const double eps);
}



namespace MCMC_impl {
	MCMC::Results call(Rev_Tree* tr, const BOOL num_steps_fixed, const int num_steps, const int break_length, const double eps);
}


#endif

