#ifndef CLASSFCTN_H
#define CLASSFCTN_H

#include "standard.h"

#include "empty.h"

class Classftn;
namespace MCMC { class Results; }
class Tip_Seqs;
class Rev_Tree;



namespace Classftn_impl {
	struct Tip
	{
		BOOL is_pure;
		int id;
	};

	ostream& operator<<(ostream& s, const Tip& cs);
}



struct Results_Sim_Seq
{
	VECTOR( VECTOR(string) ) seqs;
	VECTOR(VECTOR(double) ) sep_dist;
	VECTOR(double) noise_dist;
};




class Classftn
{
	friend class classftn_changer;
	friend ostream& operator<<(ostream& s, const Classftn& cls);
private:
	struct Crf_To_Delete;

	static const int NUM_MCMC_STEPS = 100;
	static const int MCMC_BREAK_LENGTH = 25;
	static const double MCMC_EPS = 1e-1;

	int m_num_sites;
	VECTOR(Classftn_impl::Tip) m_tips;
	VECTOR( VECTOR(int) ) m_bps;
	VECTOR( VECTOR(int) ) m_segm_subs;
	map<int, set<int> > m_crf_containing_tips;   // key: crf, mapped: index of nodes belonging to CRF 'key'
	map<int, set<int> > m_sub_containing_tips;   // key: sub, mapped: index of nodes belonging to sub 'key'
	static LoggerPtr m_logger;

	static LoggerPtr& logger() { return m_logger; }

	void set_sub_id(int i, int val);
	void set_to_pure(int i, int val);

	int new_sub_id() const;
	void add_tip(BOOL is_pure, int id);
	void resize_aft_seq();
	void resize_aft_bp();
	void recalc_internal();
	void add_bp(int index, double val);
	void add_segment(int index, int val);

	void adjust_after_change();
	MCMC::Results log_prob_impl(const Tip_Seqs& tip_seqs, BOOL num_steps_fixed, int num_steps, int break_length, double eps) const;

	static void get_unknown(int* crf_containing_unknown, VECTOR(pair<int COMMA int> )* boundaries_unknown);
public:
	Classftn();
	Classftn(const string& filename, int num_sites_para);

	BOOL is_pure(int i) const { return m_tips[i].is_pure; }
	int sub_id(int i) const;
	int crf_id(int i) const;
	int num_tips() const { return m_tips.size(); }
	int num_subs() const { return m_sub_containing_tips.size(); }
	int num_crfs() const { return m_crf_containing_tips.size(); }
	int num_sites() const { return m_num_sites; }
	void get_occur_subs(set<int>* occur_subs) const;
	void get_occur_crfs(set<int>* occur_crfs) const;
	BOOL contains_subs() const { return (m_sub_containing_tips.size() > 0 ? true : false); }
	BOOL contains_crfs() const { return (m_crf_containing_tips.size() > 0 ? true : false); }
	map<int, set<int> > sub_containing_tips() const { return m_sub_containing_tips; }

	Classftn_impl::Tip tip(int i) const { return m_tips[i]; }
	int bp(int i, int j) const { return m_bps[i][j]; }
	int num_bps(int i) const { return m_bps[i].size(); }
	int segm_sub(int i, const int j) const { return m_segm_subs[i][j]; }
	int num_segms(int i) const { return m_segm_subs[i].size(); }

	void generate_tree(Rev_Tree* tr, const variant<Tip_Seqs, Empty>& tip_seqs) const;
	tuple<VECTOR( VECTOR(string) ),   // seqs, sep_dists, noise_dists
		  VECTOR( VECTOR(double) ),
		  VECTOR(double) > sim_seqs(int num_sims, BOOL is_optim_para);

	MCMC::Results log_prob_fix(const Tip_Seqs& tip_seqs, int num_steps = NUM_MCMC_STEPS) const;
	MCMC::Results log_prob_var(const Tip_Seqs& tip_seqs, int break_length = MCMC_BREAK_LENGTH,
							  double eps = MCMC_EPS) const;
	MCMC::Results log_prob_var(const Tip_Seqs& tip_seqs, double eps) const;

	void calc_bps(const Tip_Seqs& tip_seqs, BOOL use_unknown_segms);
	void change(const Tip_Seqs& tip_seqs);
};

ostream& operator<<(ostream& s, const Classftn& cls);



struct Classftn::Crf_To_Delete
{
	int crf;   // crf to be deleted
	int sub;   // sub to which the sequences of the crf will be moved
};

#endif
