#include "standard.h"

#include "classftn.h"
#include "cls_changer.h"
#include "files.h"
#include "random.h"

using namespace std;



// delete CRFs in Classftn in case only one sub is left
void classftn_changer::adjust_after_change(Classftn* cls)
{
	cls->recalc_internal();

	if (cls->contains_crfs() && (cls->num_subs() == 1))
	{
		set<int> occur_subs;
		cls->get_occur_subs(&occur_subs);
		int sub = *occur_subs.begin();
		for (int i = 0; i < cls->num_tips(); ++i)
			if (!cls->is_pure(i))
			{
				cls->set_to_pure(i, sub);
			}
		adjust_after_change(cls);
	}
}




void classftn_changer::change(enum_cls_chng_ops op, Classftn* cls)
{
	const bool OUTPUT_DEBUG = 1;
	ofstream* out_strm = In_Out_Streams::instance()->ostrm(OUTPUT_FILE);

	switch (op)
	{
		case PURE_SEQ_TO_PURE_SEQ:
		{
			if (OUTPUT_DEBUG)
				*out_strm << "MOVE_PURE_SEQ" << endl;

			VECTOR(int) elig_targets;
			for (map<int, set<int> >::const_iterator it = cls->m_sub_containing_tips.begin();
			     it != cls->m_sub_containing_tips.end(); ++it)
				for (set<int>::iterator jt = it->second.begin(); jt != it->second.end(); ++jt)
					elig_targets.push_back(*jt);
			int target_tip_idx = elig_targets[gsl_rng_uniform_int(Random::rng(), elig_targets.size())];

			if (OUTPUT_DEBUG)
			{
				*out_strm << "elig_targets: ";
				for (VECTOR(int)::iterator it = elig_targets.begin(); it != elig_targets.end(); ++it)
					*out_strm << *it << " ";
				*out_strm << endl;
				*out_strm << "target idx: " << target_tip_idx << endl << endl;
			}

			VECTOR(int) elig_dest_subs;
			set<int> occur_subs;
			cls->get_occur_subs(&occur_subs);
			for (set<int>::const_iterator it = occur_subs.begin(); it != occur_subs.end(); ++it)
				if (*it != cls->sub_id(target_tip_idx))
					elig_dest_subs.push_back(*it);
			int dest_sub = elig_dest_subs[gsl_rng_uniform_int(Random::rng(), elig_dest_subs.size())];
			cls->set_sub_id(target_tip_idx, dest_sub);

			if (OUTPUT_DEBUG)
			{
				*out_strm << "elig_dest_subs: ";
				for (VECTOR(int)::iterator it = elig_dest_subs.begin(); it != elig_dest_subs.end(); ++it)
					*out_strm << *it << " ";
				*out_strm << endl;
				*out_strm << "destination sub: " << dest_sub << endl << endl;
			}

			break;
		}
		case PART_SUB_TO_NEW_SUB:
		{
			if (OUTPUT_DEBUG)
				*out_strm << "PART_SUB_TO_NEW_SUB" << endl;

			VECTOR(int) elig_target_subs;
			set<int> occur_subs;
			cls->get_occur_subs(&occur_subs);
			for (set<int>::const_iterator it = occur_subs.begin(); it != occur_subs.end(); ++it)
				if (cls->m_sub_containing_tips[*it].size() > 1)
					elig_target_subs.push_back(*it);
			int target_sub = elig_target_subs[gsl_rng_uniform_int(Random::rng(), elig_target_subs.size())];

			set<int>* staying_tips = new set<int>;
			set<int>* moving_tips = new set<int>;
			Random::get_rnd_partition(cls->m_sub_containing_tips[target_sub], staying_tips, moving_tips);
			int new_sub = cls->new_sub_id();
			for (set<int>::iterator it = moving_tips->begin(); it != moving_tips->end(); ++it)
				cls->set_sub_id(*it, new_sub);
			break;
		}
	}

	adjust_after_change(cls);
}
