#include "standard.h"

#include "constants.h"
#include "error.h"
#include "files.h"

Constants* Constants::m_instance = 0;
const string APPL_ID_FILE_NAME = "/gobics/home/ingo/workspace/HIVCoal/misc/appl_id.txt";



Constants::Constants()
{
	// evolution parameter
	m_recomb_const = 0;
	m_theta = 0;
	m_alpha = 0;
	m_num_mutation_rates = 0;
	m_prob_mutation_rates = VECTOR(double)();
	m_base_probs = VECTOR(double)();
	// program control
	m_jump_prob = 0;
	m_use_jphmm = true;
	// administrative
	m_messages_on = true;
	m_use_common_output_file = true;
	m_delete_spec_output_file = false;
	m_appl_id = 0;
	m_single_arg_file = true;
	m_compile_dot = false;
	m_compile_dot_immediately = false;
	m_compile_sim_tree = false;
	m_compile_best_tree = false;
	m_use_scratch = false;
	// other
	m_alphabet = Alphabet();

	ifstream in_strm(APPL_ID_FILE_NAME.c_str());
	if (!in_strm.eof()) {
		int id;
		in_strm >> id;
		m_appl_id = ++id;	
		ofstream out_strm(APPL_ID_FILE_NAME.c_str());
		out_strm << id << endl;
	}
	else {
		stringstream ss;
		ss << "File error in application id file";
		throw Error(ss.str());
	}

	m_recomb_const = 0.0001;
	m_theta = 1.251964 / 2;
	m_alpha = 0.41565;
	m_num_mutation_rates = 2;
	m_base_probs.clear();
	m_base_probs += 0.37039, 0.17354, 0.23231, 0.22376;
	switch (num_mutation_rates())
	{
		case 1:
			m_prob_mutation_rates += 1;
			break;
		case 2:
			m_prob_mutation_rates += 0.5, 0.5;
			break;
		default:
			push_back(m_prob_mutation_rates).repeat(num_mutation_rates(), 1.0 / num_mutation_rates());
			break;
	}
	m_jump_prob = 1e-9;

	VECTOR(char) s;
	s += 'A', 'C', 'G', 'T';
	m_alphabet.init(s);
}



Constants* Constants::instance()
{
	if (!m_instance)
		m_instance = new Constants();
	return m_instance;
}

