#include "standard.h"

#include "crf.h"
#include "files.h"
#include "part_tree.h"
#include "tree.h"

int CRF::m_count = 0;



CRF::CRF(Rev_Tree* tr, int num_bps, VECTOR(int) bps, VECTOR(int) segm_subs)
: m_num_bps(num_bps), m_bps(bps), m_segm_subs(segm_subs), m_id(m_count++), m_num_sites(tr->num_sites())
{
	OUT_FNCTN_HEADER(crf_logger);
	for (VECTOR(int)::const_iterator it = m_segm_subs.begin(); it != m_segm_subs.end(); it++)
		m_occur_subs.insert(*it);
	tr->m_crfs.push_back(this);
	OUT_FNCTN_BOTTOM(crf_logger);
}



CRF::~CRF()
{
	OUT_FNCTN_HEADER(crf_logger);
	delete m_part_tree;
	OUT_FNCTN_BOTTOM(crf_logger);
}



// get generalized breakpoint (gen_bp(-1) = -1, gen_bp(num_bps) = num_sites - 1)
int CRF::gen_bp(const int i) const
{
	if (i == -1)
		return -1;
	else
		if (i == num_bps())
			return num_sites() - 1;
		else
			return m_bps[i];
}



// get subtype at position i
int CRF::sub(const int i) const
{
	for (int j = 0; j < num_segms(); ++j)
		if (i <= gen_bp(j))
			return segm_sub(j);
}



// output crf
ostream& operator<<(ostream& s, const CRF& crf)
{
	s << "*** CRF ***" << endl;

	s << "id: " << crf.id() << endl;
	s << "num_bps: " << crf.num_bps() << endl;

	s << "bp: ";
	for (int i = 0; i < crf.num_bps(); ++i)
		s << crf.bp(i) << " ";
	s << endl;

	s << "segment_subtype: ";
	for (int i = 0; i < crf.num_segms(); ++i)
		s << crf.segm_sub(i) << " ";
	s << endl;

	s << "occuring_subtypes: ";
	for (set<int>::const_iterator it = crf.occur_subs().begin(); it != crf.occur_subs().end(); it++)
		s << *it << " ";
	s << endl;

	return s;
}



Partition_Aliasing::Partition_Aliasing(const VECTOR(CRF*) crfs, const int num_sites)
{
	OUT_FNCTN_HEADER(crf_logger);

	// determine which positions belong to the same subtype in each crf
	VECTOR( VECTOR(set <int> ) ) crf_partitions;   // (index of crf, sub)
	for (VECTOR(CRF*)::const_iterator it = crfs.begin(); it != crfs.end(); ++it) {
		VECTOR(set <int> ) v;
		for (set<int>::iterator jt = (*it)->occur_subs().begin(); jt != (*it)->occur_subs().end(); ++jt) {
			set<int> s;
			for (int i = 0; i < (*it)->num_segms(); ++i) {
				if ((*it)->segm_sub(i) == *jt) {
					for (int j = (*it)->gen_bp(i - 1) + 1; j <= (*it)->gen_bp(i); ++j)
						s.insert(j);
				}
			}
			v.push_back(s);
		}
		crf_partitions.push_back(v);
	}
	LOG4CXX_DEBUG(crf_logger, "determined which positions belong to the same subtype in each crf");

	// intersect all partitions of the crfs
	set<int> s;
	for (int i = 0; i < num_sites; ++i)
		s.insert(i);
	VECTOR( set<int> ) part_set(1, s);
	for (VECTOR( VECTOR(set <int> ) )::iterator it = crf_partitions.begin(); it != crf_partitions.end(); ++it) {
		VECTOR( set<int> ) v;
		for (VECTOR( set<int> )::iterator jt = part_set.begin(); jt != part_set.end(); ++jt)
			for (VECTOR( set<int> )::iterator kt = it->begin(); kt != it->end(); ++kt) {
				VECTOR(int) w(GSL_MIN(jt->size(), kt->size()));
				VECTOR(int)::iterator lt = set_intersection(jt->begin(), jt->end(), kt->begin(), kt->end(),
														    w.begin());
				if (lt - w.begin() > 0) {
					set<int> s(w.begin(), lt);
					v.push_back(s);
				}
			}
		part_set = v;
	}
	LOG4CXX_DEBUG(crf_logger, "intersected all partitions of the crfs");

	// save result in a vector
	for (VECTOR( set<int> )::iterator it = part_set.begin(); it != part_set.end(); ++it) {
		VECTOR( pair<int COMMA int> ) v;
		int begin = 0;
		int last = 0;
		BOOL is_first = true;
		for (set<int>::iterator jt = it->begin(); jt != it->end(); ++jt) {
			if (is_first) {
				begin = *jt;
				is_first = false;
			}
			else
				if (*jt != last + 1) {
					v.push_back(pair<int, int>(begin, last));
					begin = *jt;
				}
			last = *jt;
		}
		v.push_back(pair<int, int>(begin, last));
		m_partition.push_back(v);
	}
	LOG4CXX_DEBUG(crf_logger, "saved result in a vector");

	OUT_FNCTN_BOTTOM(crf_logger);
}



// output Partition_Aliasing
ostream& operator<<(ostream& s, const Partition_Aliasing& part)
{
	s << "*** PARTITION_ALIASING ***" << endl;

	for (int i = 0; i < part.m_partition.size(); ++i)
		for (int j = 0; j < part.m_partition[i].size(); ++j)
			s << "Group " << setw(3) << i << " Segment " << setw(3) << j << ": " << part.m_partition[i][j].first
			  << " - " << part.m_partition[i][j].second << endl;

	return s;
}
