#ifndef CRF_H
#define CRF_H

#include "standard.h"

class Partition_Tree;
class Rev_Tree;

static LoggerPtr crf_logger(Logger::getLogger("crf.crf"));



class CRF
{
private:
	static int m_count;
	set<int> m_occur_subs;
	const int m_id;
	const int m_num_bps;
	const VECTOR(int) m_bps;
	const VECTOR(int) m_segm_subs;
	const int m_num_sites;
	Partition_Tree* m_part_tree;
public:
	CRF(Rev_Tree* tr, int num_bps, VECTOR(int) bp, VECTOR(int) segment_subtype);
	~CRF();

	Partition_Tree* part_tree() const { return m_part_tree; }
	Partition_Tree*& ref_part_tree() { return m_part_tree; }
	int id() const { return m_id; }
	int num_bps() const { return m_num_bps; }
	int bp(const int i) const { return m_bps[i]; }
	int gen_bp(const int i) const;
	int segm_sub(const int i) const { return m_segm_subs[i]; }
	int sub(const int i) const;
	int num_sites() const { return m_num_sites; }
	int num_segms() const { return m_num_bps + 1; }
	const set<int>& occur_subs() const { return m_occur_subs; }
};

ostream& operator<<(ostream& s, const CRF& crf);



class Partition_Aliasing
{
	friend ostream& operator<<(ostream& s, const Partition_Aliasing& part);
private:
	VECTOR( VECTOR( pair<int COMMA int> ) ) m_partition;
public:
	Partition_Aliasing(const VECTOR(CRF*) crfs, const int num_sites);

	int begin_segm(const int i, const int j) const { return m_partition[i][j].first; }
	int end_segm(const int i, const int j) const { return m_partition[i][j].second; }
	int num_groups() const { return m_partition.size(); }
	int num_segms(const int i) const { return m_partition[i].size(); }
};

ostream& operator<<(ostream& s, const Partition_Aliasing& part);

#endif
