#ifndef PART_NODE_H
#define PART_NODE_H

#include "standard.h"

using namespace std;

class Part_Node_Conn;
class Partition_Tree;
class Partition_Node;



struct Subs
{
	set<int> coal_subs;
	set<int> sep_subs;
	set< set<int> > grouped_subs;

	Subs() {}
	Subs(set<int> coal_subs_para, set<int> sep_subs_para, set< set<int> > grouped_subs_para)
	{
		coal_subs = coal_subs_para;
		for (set<int>::iterator it = sep_subs_para.begin(); it != sep_subs_para.end(); it++)
			sep_subs.insert(*it);
		for (set< set<int> >::iterator it = grouped_subs_para.begin(); it != grouped_subs_para.end(); it++)
		{
			grouped_subs.insert(*it);
		}
	}
};



struct Suc_Coal
{
	friend BOOL operator<(const Suc_Coal& s, const Suc_Coal& t);

	Partition_Node* target;
	int source_sub;
};



struct Suc_Rec
{
	friend BOOL operator<(const Suc_Rec& s, const Suc_Rec& t);

	Partition_Node* target;
	set<int> source_subs;
	set<int> first_target_subs;
	set<int> second_target_subs;
};

ostream& operator<<(ostream& s, const Suc_Rec& suc);



class Partition_Node
{
	friend class Partition_Tree;
	friend ostream& operator<<(ostream& s, const Partition_Node& n);
private:
	int m_id;
	Subs m_subs;
	set<Suc_Coal> m_suc_coal;
	set<Suc_Rec> m_suc_recomb;   // pair: node, number of possibilities to divide the chosen subset of subtypes
	const int m_level;
	static LoggerPtr s_logger;

	Partition_Node() : m_level(0) { }
	Partition_Node(const set<int>& coal_subs, const set<int>& sep_subs, const set< set<int> >& grouped_subs,
				   int level, BOOL is_first = false);
	Partition_Node(Subs subs, int level) : m_subs(subs), m_level(level) { }
	static LoggerPtr& logger() { return s_logger; }
	void generate_suc(Partition_Tree* part_tree);
public:
	int level() { return m_level; }
	const set<Suc_Coal>& suc_coal() const { return m_suc_coal; }
	const set<Suc_Rec>& suc_recomb() const { return m_suc_recomb; }
	void get_all_sucs(set<Partition_Node*>& all_sucs) const;
	const int id() const { return m_id; }
	const Subs& subs() const { return m_subs; }
	void conns(VECTOR(Part_Node_Conn)* conns, Part_Node_Conn* curr_conn, int req_length) const;
	string to_string() const;
	BOOL operator==(Subs subs);
};

ostream& operator<<(ostream& s, const Partition_Node& n);

#endif
