#ifndef SUB_PART_H
#define SUB_PART_H

#include "standard.h"

#include "prt_nd_conn.h"

using namespace std;

class Rev_Nd;
class Rev_Rec_Nd;
class Partition_Node;
struct Subs;



class Partition_Tree
{
	friend class Partition_Node;
	friend class Rev_Tree;
private:
	Partition_Node* m_last;
	VECTOR(Partition_Node*) m_part_nodes;
	VECTOR( VECTOR(int) ) m_all_sucs;
	VECTOR(Part_Node_Conn) m_conns;
	map<int, Rev_Rec_Nd*> m_nodes_sep;
	map<set<int>, Rev_Nd*> m_nodes_grouped;
	int m_max_level;
	set<int> m_subs_only_in_crf;
	static LoggerPtr s_logger;
	
	static LoggerPtr& logger() { return s_logger; }
	Partition_Node* root() { return (m_part_nodes.size() > 0) ? m_part_nodes[0] : 0; }
	Rev_Rec_Nd*& ref_node(int i) { return m_nodes_sep[i]; }
public:
	Partition_Tree(const set<int>& occur_subs, const set<int>& subs_only_in_crfs);
	~Partition_Tree();

	const Partition_Node* root() const { return (m_part_nodes.size() > 0) ? m_part_nodes[0] : 0; }
	const Partition_Node* last() const { return (m_part_nodes.size() > 0) ? m_last : 0; }
	const VECTOR(Partition_Node*)& part_nodes() const { return m_part_nodes; }
	const int num_part_nodes() const { return m_part_nodes.size(); }
	const VECTOR(int)& all_sucs(int no_part_node) const { return m_all_sucs[no_part_node]; }
	const VECTOR(Part_Node_Conn)& conns() const { return m_conns; }
	Rev_Rec_Nd* node(int i) const { return m_nodes_sep.find(i)->second; }
	Rev_Nd* node(const set<int>& s) const;
	int max_level() const { return m_max_level; };
	BOOL sub_only_in_crfs(const int i) const { return m_subs_only_in_crf.find(i) != m_subs_only_in_crf.end(); }

	void set_node(const set<int>& s, Rev_Nd* n);
	void set_root(Rev_Nd* n);

	Partition_Node* find_node(Subs subs);
};

ostream& operator<<(ostream& s, const Partition_Tree& tr);

#endif
