#include "standard.h"

#include "error.h"
#include "files.h"
#include "prt_nd_conn.h"

static LoggerPtr prt_nd_conn_logger(Logger::getLogger("crf.prt_nd_conn"));



Conn_Point::Conn_Point(int id, BOOL is_coal, const set<int>& subs, const set<int>& first_subs, const set<int>& second_subs)
: m_id(id), m_is_coal(is_coal), m_subs(subs), m_first_subs(first_subs), m_second_subs(second_subs)
{
	OUT_FNCTN_HEADER(prt_nd_conn_logger);
	OUT_FNCTN_BOTTOM(prt_nd_conn_logger);
}


int Conn_Point::sub() const
{
	if (subs().size() != 1)
		throw ERROR(prt_nd_conn_logger, "Conn_Point: Trying to get single sub, but subs has not size 1");
	return *(subs().begin());
}



const set<int>& Conn_Point::first_subs() const
{
	if (m_first_subs.size() == 0)
		throw ERROR(prt_nd_conn_logger, "Conn_Point: first_subs is empty");
	return m_first_subs;
}



const set<int>& Conn_Point::second_subs() const
{
	if (m_second_subs.size() == 0)
		throw ERROR(prt_nd_conn_logger, "Conn_Point: second_subs is empty");
	return m_second_subs;
}



ostream& operator<<(ostream& s, const Conn_Point& cp)
{
	s << "id: " << cp.m_id << endl;
	s << "is_coal: " << cp.m_is_coal << endl;
	s << "subs: " << cp.m_subs << endl;
	if (!cp.m_is_coal) {
		s << "first_subs: " << cp.m_first_subs << endl;
		s << "first_subs: " << cp.first_subs() << endl;
		s << "second_subs: " << cp.m_second_subs << endl;
	}
	return s;
}



void Part_Node_Conn::push_back(int id, BOOL is_coal, const set<int>& subs,
	                           const set<int>& first_subs, const set<int>& second_subs)
{
	ofstream* out_strm = In_Out_Streams::instance()->ostrm(OUTPUT_FILE);

	Conn_Point s(id, is_coal, subs, first_subs, second_subs);
	if (is_coal && (subs.size() > 1))
		throw ERROR(prt_nd_conn_logger, "Part_Node_Conn: Trying to add a coalescent event to which more than one subtype is associated");
	m_conn_points.push_back(s);
}



void Part_Node_Conn::push_back(const int id, const int sub)
{
	set<int> subs;
	subs.insert(sub);
	set<int> first_subs, second_subs;
	Conn_Point s(id, true, subs, first_subs, second_subs);
	m_conn_points.push_back(s);
}



// delete last element of the connection
void Part_Node_Conn::pop_back()
{
	m_conn_points.pop_back();
}



// return last element of the connection
const Conn_Point& Part_Node_Conn::back() const
{
	return m_conn_points.back();
}



ostream& operator<<(ostream& s, const Part_Node_Conn& c)
{
	s << "*** PART_NODE_CONN ***" << endl;
	for (VECTOR(Conn_Point)::const_iterator it = c.m_conn_points.begin(); it != c.m_conn_points.end(); it++) {
		s << it->id() << " ";
		if (it->is_coal())
			s << "C" << it->subs();
		else
			s << "R" << it->first_subs() << " " << it->second_subs();
	}
	s << endl;

	return s;
}
