#include "standard.h"

#include "error.h"

using namespace log4cxx;
using namespace std;

const int MAX_LEN_HEADER_BOTTOM = 80;
const int MAX_LEN_FUNCTION_INFO = 70;



Error::Error(LoggerPtr logger, const string& file_name, const string& function_name, int no_line, const string& msg)
: m_file_name(file_name),  m_function_name(function_name), m_no_line(no_line), m_message(msg)
{
	stringstream ss;
	ss << file_name << ":" << function_name << ":" << no_line << " - "<< msg;
	cerr << ss.str() << endl;
	LOG4CXX_ERROR(logger, ss.str());
}



ostream& operator<<(ostream& s, const Error& err)
{
	s << err.message() << endl;
}



void output_function_header(LoggerPtr logger, const string& file_name, const string& function_name, const string& det_fnctn_name)
{
	const string DESCRIPTION = " BEGIN OF: ";

	stringstream ss;
	ss << file_name << " - " << det_fnctn_name;
	string str(ss.str());
	if ((int)str.length() > MAX_LEN_FUNCTION_INFO) {
		stringstream ss;
		ss << file_name << " - " << function_name;
		str = ss.str().substr(0, MAX_LEN_FUNCTION_INFO);
	}
	ss.str("");
	ss << string(10, '*') << DESCRIPTION << str << " "
	   << string(GSL_MAX(0, MAX_LEN_HEADER_BOTTOM - static_cast<ptrdiff_t>(DESCRIPTION.length())
			                - static_cast<ptrdiff_t>(str.length())), '*') << endl;
	LOG4CXX_INFO(logger, ss.str());
}



void output_function_bottom(LoggerPtr logger, const string& file_name, const string& function_name, const string& det_fnctn_name)
{
	const string DESCRIPTION = " END OF: ";

	stringstream ss;
	ss << file_name << " - " << det_fnctn_name;
	string str(ss.str());
	if ((int)str.length() > MAX_LEN_FUNCTION_INFO) {
		stringstream ss;
		ss << file_name << " - " << function_name;
		str = ss.str().substr(0, MAX_LEN_FUNCTION_INFO);
	}
	ss.str("");
	ss << string(10, '+') << DESCRIPTION << str << " "
	   << string(GSL_MAX(0, MAX_LEN_HEADER_BOTTOM - static_cast<ptrdiff_t>(DESCRIPTION.length())
						    - static_cast<ptrdiff_t>(str.length())), '+') << endl << endl;
	LOG4CXX_INFO(logger, ss.str());
}

