#include "standard.h"

#include "constants.h"
#include "error.h"
#include "log_acid_prob.h"

LoggerPtr Log_Acid_Prob::s_logger(Logger::getLogger("node.log_acid_prob"));



Log_Acid_Prob::Log_Acid_Prob (const VECTOR( VECTOR(double) )* log_p)
{
	OUT_FNCTN_HEADER(logger());
	m_id = m_count_id++;
	m_log_p.resize(Constants::instance()->num_mutation_rates());
	for (int i = 0; i < log_p->size(); ++i)
		m_log_p[i].resize(NUM_NUCL);
	m_log_p = *log_p;

	m_log_p_anc.resize(log_p->size());
	for (int i = 0; i < log_p->size(); ++i)
		m_log_p_anc[i].resize((*log_p)[i].size());
	OUT_FNCTN_BOTTOM(logger());
}



Log_Acid_Prob::Log_Acid_Prob (const int i)
{
	OUT_FNCTN_HEADER(logger());
	m_id = m_count_id++;
	m_log_p.resize(Constants::instance()->num_mutation_rates());
	for (VECTOR( VECTOR(double) )::iterator it = m_log_p.begin(); it != m_log_p.end(); ++it) {
		it->resize(NUM_NUCL, -10000);
		(*it)[i] = 0;
	}

	m_log_p_anc.resize(Constants::instance()->num_mutation_rates());
	for (VECTOR( VECTOR(double) )::iterator it = m_log_p_anc.begin(); it != m_log_p_anc.end(); ++it)
		it->resize(NUM_NUCL, -10000);
	OUT_FNCTN_BOTTOM(logger());
}



Log_Acid_Prob::Log_Acid_Prob(const Log_Acid_Prob& lap)
{
	OUT_FNCTN_HEADER(logger());
	m_id = m_count_id++;
	m_log_p.resize(Constants::instance()->num_mutation_rates());
	for (int i = 0; i < lap.m_log_p.size(); ++i)
		m_log_p[i].resize(NUM_NUCL);
	m_log_p = lap.m_log_p;

	m_log_p_anc.resize(Constants::instance()->num_mutation_rates());
	for (int i = 0; i < lap.m_log_p_anc.size(); ++i)
		m_log_p_anc[i].resize(NUM_NUCL);
	m_log_p_anc = lap.m_log_p_anc;
	OUT_FNCTN_BOTTOM(logger());
}



ostream& operator<<(ostream& s, const Log_Acid_Prob& lap)
{
	s << "*** LOG_ACID_PROB ***" << endl;
	s << "id: " << lap.id() << endl;
	for (int i = 0; i < lap.m_log_p.size(); ++i)
		for (int j = 0; j < lap.m_log_p[i].size(); ++j)
			s << "m_log_p[" << i << "][" << j << "] = " << lap.m_log_p[i][j] << endl;
	for (int i = 0; i < lap.m_log_p_anc.size(); ++i)
		for (int j = 0; j < lap.m_log_p_anc[i].size(); ++j)
			s << "m_log_p_anc[" << i << "][" << j << "] = " << lap.m_log_p_anc[i][j] << endl;
	return s;
}



