#include "standard.h"

#include "base_nd.h"
#include "changes.h"
#include "constants.h"
#include "crf.h"
#include "dot.h"
#include "empty.h"
#include "error.h"
#include "log_acid_prob.h"
#include "nd.h"
#include "nd_visitor.h"
#include "nc.h"
#include "files.h"
#include "random.h"
#include "subst_prob.h"
#include "sum_probs.h"
#include "tree.h"

template<class N, class C, class T>
long Base_Nd<N, C, T>::m_count_id = 0;
int Log_Acid_Prob::m_count_id = 0;
LoggerPtr Rev_Nd::m_logger(Logger::getLogger("node.nd"));



// *****************************************************************************
// * Rev_Nd
// *****************************************************************************

BOOL Rev_Nd::is_nd_st_affected() const { return unspec_node_contnr()->is_nd_st_affected(); }
Rev_Intern_NC* Rev_Nd::old_parent_contnr() const { return apply_visitor(Parent_Contnr_Visitor<Rev_Coal_NC, Rev_Rec_NC, Rev_Intern_NC>(), m_anc_old); }
Rev_Coal_Nd* Rev_Nd::old_parent_node() const { return apply_visitor(Parent_Node_Visitor<Rev_Coal_Nd, Rev_Coal_NC, Rev_Rec_NC>(), m_anc_old); }
Rev_Rec_Nd* Rev_Nd::first_old_parent_node() const { return apply_visitor(First_Parent_Node_Visitor<Rev_Rec_Nd, Rev_Coal_NC, Rev_Rec_NC>(), m_anc_old); }
Rev_Rec_Nd* Rev_Nd::second_old_parent_node() const { return apply_visitor(Second_Parent_Node_Visitor<Rev_Rec_Nd, Rev_Coal_NC, Rev_Rec_NC>(), m_anc_old); }
Rev_Nd* Rev_Nd::old_next() const { return var_next(unspec_node_contnr()->old_next()); }
Rev_Nd* Rev_Nd::old_prev() const { return var_next(unspec_node_contnr()->old_prev()); }
int Rev_Nd::old_num_elig_nodes() const { return unspec_node_contnr()->old_num_elig_nodes(); }
int Rev_Nd::old_num_cum_elig_sites() const { return unspec_node_contnr()->old_num_cum_elig_sites(); }
void Rev_Nd::set_nd_st_affected() { unspec_node_contnr()->set_nd_st_affected(); }
void Rev_Nd::accept(Nd_Visitor* v) { v->visit(this); }

ostream& operator<<(ostream& s, const Rev_Nd& n) { s << Det_Manip<Rev_Nd>(0)(&n); return s; }



void Rev_Nd::init_old_lap()
{
	OUT_FNCTN_HEADER(m_logger);
	m_is_anc_affected = false;
	m_is_ac_pr_affected = false;

	// initialize m_log_acid_prob
	m_old_log_acid_prob.resize(tree()->num_mutation_rates());
	for (int i = 0; i < tree()->num_mutation_rates(); ++i)
	{
		m_old_log_acid_prob[i].resize(num_sites());
		for (int j = 0; j < num_sites(); ++j)
		{
			m_old_log_acid_prob[i][j].resize(NUM_NUCL);
			for (int k = 0; k < NUM_NUCL; ++k)
				m_old_log_acid_prob[i][j][k] = 0.0;
		}
	}

	// initialize m_log_brnch_acid_prob
	m_old_log_brnch_acid_prob.resize(2);
	for (int l = 0; l < 2; ++l)
	{
		m_old_log_brnch_acid_prob[l].resize(tree()->num_mutation_rates());
		for (int i = 0; i < tree()->num_mutation_rates(); ++i)
		{
			m_old_log_brnch_acid_prob[l][i].resize(num_sites());
			for (int j = 0; j < num_sites(); ++j)
			{
				m_old_log_brnch_acid_prob[l][i][j].resize(NUM_NUCL);
				for (int k = 0; k < NUM_NUCL; ++k)
					m_old_log_brnch_acid_prob[l][i][j][k] = 0.0;
			}
		}
	}

	m_log_acid_prob_ptr_old.resize(tree()->part_alias()->num_groups());
	m_site_to_prob_old.resize(num_sites());

	VECTOR( VECTOR(double) ) w(Constants::instance()->num_mutation_rates());
	for (VECTOR( VECTOR(double) )::iterator it = w.begin(); it != w.end(); ++it)
		it->resize(NUM_NUCL, -10000);
	m_site_to_prob_old.resize(num_sites());
	m_log_acid_prob_ptr_old.resize(tree()->part_alias()->num_groups());
	for (int i = 0; i < tree()->part_alias()->num_groups(); ++i) {
		Log_Acid_Prob* lap = new Log_Acid_Prob(&w);
		m_log_acid_prob_ptr_old[i].push_back(lap);
		for (int sgm = 0; sgm < tree()->part_alias()->num_segms(i); ++sgm)
			for (int site = tree()->part_alias()->begin_segm(i, sgm);
				 site <= tree()->part_alias()->end_segm(i, sgm); ++site) {
				m_site_to_prob_old[site] = lap;
			}
	}

	m_is_lap_init = true;
	OUT_FNCTN_BOTTOM(m_logger);
}



// initialize type, subtype, crf id and occuring subtypes of a node
void Rev_Nd::init_crf_sub(const enum_node_type type, const int crf_sub_id, const set<int> occur_subs)
{
	OUT_FNCTN_HEADER(m_logger);
	m_type = type;
	m_occur_subs = occur_subs;
	if (type == PURE_NODE)
		m_subtype = crf_sub_id;
	else
		if ((type == CRF_NODE) || (type == RECOMB_NODE))
			m_crf_id = crf_sub_id;
	m_is_crf_sub_init = true;
	OUT_FNCTN_BOTTOM(m_logger);
}



/*void Rev_Nd::init_act_segms(const VECTOR(BOOL) is_act_segment)
{
	m_is_act_segment = is_act_segment;
	m_is_act_segms_init = true;
}*/   //needless?



void Rev_Nd::init()
{
	OUT_FNCTN_HEADER(m_logger);
	LOG4CXX_DEBUG(m_logger, "id: " << id());
	if (!tree()->part_alias_calced())
		throw Error("Trying to construct node although the partition for aliasing has not been generated");
	init_old_lap();
	OUT_FNCTN_BOTTOM(m_logger);
}



Rev_Nd::Rev_Nd()
{
	OUT_FNCTN_HEADER(m_logger);
	init();
	OUT_FNCTN_BOTTOM(m_logger);

}



Rev_Nd::~Rev_Nd()
{
	OUT_FNCTN_HEADER(m_logger);
	for (VECTOR( VECTOR(Log_Acid_Prob*) )::iterator it = m_log_acid_prob_ptr_old.begin();
		 it != m_log_acid_prob_ptr_old.end(); ++it)
		for (VECTOR(Log_Acid_Prob*)::iterator jt = it->begin(); jt != it->end(); ++jt)
			checked_delete(*jt);
	OUT_FNCTN_BOTTOM(m_logger);
}



// recalculates occuring subs of node and its container
void Rev_Nd::reset_occur_subs(const set<int>& occur_subs)
{
	m_occur_subs = occur_subs;
}



BOOL Rev_Nd::is_old_first() const
{
	return (tree()->old_first_node() && (id() == tree()->old_first_node()->id()));
}



BOOL Rev_Nd::is_old_last() const
{
	return (tree()->old_last_node() && (id() == tree()->old_last_node()->id()));
}



void Rev_Nd::determine_new_anc()
{
	OUT_FNCTN_HEADER(m_logger);
	LOG4CXX_DEBUG(m_logger, "id: " << id());

	set_anc_affected();
	Rev_Coal_Nd* n = parent_node();
	BOOL is_first = is_first_child();
	while (n && n->has_coal_anc() && n->is_old()) {
		is_first = n->is_first_child();
		n = n->parent_node();
	}

	if (n && !n->is_old()) {
		ref_parent_contnr() = n->node_contnr();
		coal_parent_contnr()->set_suc_affected();
		if (is_first) {
			set_to_first_coal_child();
			parent_node()->first_child_node() = this;
		}
		else {
			set_to_second_coal_child();
			parent_node()->second_child_node() = this;
		}
	}
	else {
		set_active();
		tree()->set_last_node_contnr(unspec_node_contnr());
	}
	OUT_FNCTN_BOTTOM(m_logger);
}



void Rev_Nd::set_coal_anc(Rev_Coal_NC* parent, BOOL is_first_child)
{
	OUT_FNCTN_HEADER(m_logger);
	LOG4CXX_DEBUG(m_logger, "id: " << id());
	LOG4CXX_DEBUG(m_logger, "parent: " << parent->str_id());
	LOG4CXX_DEBUG(m_logger, "is_first_child: " << is_first_child << endl);

	if (is_active())
		tree()->m_num_act_nodes--;

	set_anc_affected();
	parent->set_suc_affected();

	ref_parent_contnr() = parent;
	if (is_first_child)
	{
		parent->first_child() = this;
		set_to_first_coal_child();
	}
	else
	{
		parent->second_child() = this;
		set_to_second_coal_child();
	}
	OUT_FNCTN_BOTTOM(m_logger);
}



void Rev_Nd::set_recomb_anc(Rev_Rec_NC* parent)
{
	if (type() != CRF_NODE)
		throw ERROR(logger(), "Trying to assign recombination event to non-CRF_NODE");

	if (is_active())
		tree()->m_num_act_nodes--;

	parent->set_suc_affected();

	ref_parent_contnr() = parent;
	set_to_rec_child();
	parent->child() = this;
}



int Rev_Nd::crf_id() const
{
	if (has_crf())
		return m_crf_id;
	else
		throw ERROR(logger(), "Trying to get CRF id from non-CRF Rev_Nd");
}



int Rev_Nd::sub() const
{
	if (is_pure_node())
		return m_subtype;
	else
		throw ERROR(logger(), "Trying to get subtype of impure Rev_Nd");
}



void Rev_Nd::set_anc_affected()
{
	if (is_unmarked() && !is_anc_affected()) {
		m_anc_old = anc();
		m_anc_type_old = anc_type();
		m_is_anc_affected = true;
		tree()->changes()->add_to_anc_aff(this);
		tree()->changes()->set_rearranging_done();
	}
}



void Rev_Nd::set_ac_pr_affected()
{
	OUT_FNCTN_HEADER(m_logger);

	if (!is_ac_pr_affected() && is_unmarked() && log_acid_probs_calced())
	{
		for (VECTOR( VECTOR(Log_Acid_Prob*) )::iterator it = m_log_acid_prob_ptr_old.begin();
			 it != m_log_acid_prob_ptr_old.end(); ++it) {
			for (VECTOR(Log_Acid_Prob*)::iterator jt = it->begin(); jt != it->end(); ++jt)
				checked_delete(*jt);
			it->clear();
		}

		for (int i = 0; i < tree()->part_alias()->num_groups(); ++i) {
			map<Log_Acid_Prob*, Log_Acid_Prob*> mp;
			for (VECTOR(Log_Acid_Prob*)::const_iterator jt = log_acid_prob_ptr()[i].begin();
				 jt != log_acid_prob_ptr()[i].end(); ++jt) {
				Log_Acid_Prob* lap = new Log_Acid_Prob(**jt);
				m_log_acid_prob_ptr_old[i].push_back(lap);
				mp[*jt] = lap;
			}

			for (int sgm = 0; sgm < tree()->part_alias()->num_segms(i); ++sgm)
				for (int site = tree()->part_alias()->begin_segm(i, sgm);
					 site <= tree()->part_alias()->end_segm(i, sgm); ++site) {
					m_site_to_prob_old[site] = mp[site_to_prob()[site]];
				}
		}

		m_is_ac_pr_affected = true;
		tree()->changes()->add_to_ac_pr_aff(this);
	}
	OUT_FNCTN_BOTTOM(m_logger);
}



double Rev_Nd::old_lap(const int mut_rate, const int site, const int acid) const
{
	if (!is_act_site(site)) {
		stringstream ss;
		ss << "Trying to get log acid probs of node" << id() << " at non-active site" << site;;
		throw Error(ss.str());
	}
	return m_site_to_prob_old[site]->log_p(mut_rate, acid);
}



double Rev_Nd::old_lap_anc(const int mut_rate, const int site, const int acid) const
{
	if (!is_act_site(site)) {
		stringstream ss;
		ss << "Trying to get log acid probs of node" << id() << " at non-active site" << site;;
		throw Error(ss.str());
	}
	return m_site_to_prob_old[site]->log_p_anc(mut_rate, acid);
}



// restore log acid probs
void Rev_Nd::restore_lap()
{
	Prob_Nd<Rev_Nd, Rev_NC, Rev_Tree>::restore_lap(&m_log_acid_prob_ptr_old, &m_site_to_prob_old);
	m_is_ac_pr_affected = false;
}



// restore ancestor
void Rev_Nd::restore_anc()
{
	Nd_with_Anc<Rev_Nd, Rev_Coal_Nd, Rev_Rec_Nd, Rev_NC, Rev_Coal_NC, Rev_Rec_NC, Rev_Intern_NC, Rev_Tree>::restore_anc(m_anc_type_old, m_anc_old);
	m_is_anc_affected = false;
}



string Rev_Nd::sub_segments_to_string() const
{
	stringstream ss;
	switch (type())
	{
		case CRF_NODE:
			for (int i = 0; i < tree()->crf(crf_id())->num_segms(); ++i)
				ss << tree()->crf(crf_id())->segm_sub(i);
			break;
		case RECOMB_NODE:
			for (int i = 0; i < tree()->crf(crf_id())->num_segms(); i++)
				if (is_act_segment()[i])
					ss << tree()->crf(crf_id())->segm_sub(i);
				else
					ss << "-";
			break;
		case PURE_NODE:
			ss << m_subtype;
			break;
		case FINAL_NODE:
			BOOL is_first = true;
			for (set<int>::iterator it = occur_subs().begin(); it != occur_subs().end(); it++)
			if (is_first)
			{
				ss << *it;
				is_first = false;
			}
			else
				ss << " " << *it;
			break;
	}
	return ss.str();
}



string Rev_Nd::to_dot_string(Dot_Para dot_para) const
{
	const string COLOR_ACTIVE = "orange";
	const string COLOR_ORDINARY = "grey87";
	const string COLOR_IMPURE = "yellow";
	const int WIDTH_IMPURE = 1;

	const int MAX_NUM_ROWS = 20;

	stringstream ss[MAX_NUM_ROWS];
	int idx = 0;

	ss[idx] << id();
	idx++;

	if (dot_para.seq_tip && is_tip() || dot_para.seq_intr && !is_tip())
		ss[idx] << acid_seq();
	idx++;

	if (dot_para.sub)
	{
		ss[idx] << sub_segments_to_string();
	}
	if (dot_para.affected)
	{
		BOOL b = false;
		if (const Rev_Intern_Nd* n = dynamic_cast<const Rev_Intern_Nd*>(this))
			if (n->is_suc_affected())
				b = true;
		if (dot_para.sub &&
			(is_anc_affected() || b || is_pos_affected() || is_time_affected() || is_nd_st_affected() || is_ac_pr_affected()))
				ss[idx] << " ";
		if (is_anc_affected())
			ss[idx] << "A";
		if (b)
			ss[idx] << "S";
		if (is_pos_affected())
			ss[idx] << "P";
		if (is_time_affected())
			ss[idx] << "T";
		if (is_nd_st_affected())
			ss[idx] << "E";
		if (is_ac_pr_affected())
			ss[idx] << "B";
	}
	idx++;

	if (dot_para.alias_groups) {
		BOOL is_first = true;
		for (set<int>::const_iterator it = act_groups().begin(); it != act_groups().end(); ++it) {
			if (is_first)
				is_first = false;
			else
				ss[idx] << " ";
			ss[idx] << *it;
		}
	}
	idx++;

	if (dot_para.acid_prob_grp) {
		BOOL is_first = true;
		for (int i = 0; i < num_sites(); i++) {
			if (is_first)
				is_first = false;
			else
				ss[idx] << " ";
			if (is_act_site(i))
				ss[idx] << site_to_prob()[i]->id();
			else
				ss[idx] << "-";
		}
	}
	idx++;

	if (dot_para.acid_prob) {
		for (int i = 0; i < NUM_NUCL; i++) {
			for (int j = 0; j < num_sites(); j++) {
				double p;
				if (is_act_site(j))
					p = lap(dot_para.mut_rate, j, i);
				else
					p = -10000;
				ss[idx] << left << setprecision(4) << setw(9) << (dot_para.is_log_prob ? p : exp(p));
				if (j < num_sites() - 1)
					ss[idx] << " ";
			}
			if (i < NUM_NUCL - 1)
				ss[idx] << "<BR/>";
			else
				if (dot_para.br_ac_probs || dot_para.old_acid_prob || dot_para.old_br_ac_probs)
					ss[idx] << "<BR/> ";
		}
	}
	idx++;

	if (dot_para.br_ac_probs)
	{
		if (!is_active())
		{
			for (int i = 0; i < NUM_NUCL; i++) {
				for (int j = 0; j < num_sites(); j++) {
					double p;
					if (is_act_site(j))
						p = lap_anc(dot_para.mut_rate, j, i);
					else
						p = -10000;
					ss[idx] << left << setprecision(4) << setw(9) << (dot_para.is_log_prob ? p : exp(p));
					if (j < num_sites() - 1)
						ss[idx] << " ";
				}
				if (i < NUM_NUCL - 1)
					ss[idx] << "<BR/>";
				else
					if (dot_para.old_acid_prob || dot_para.old_br_ac_probs)
						ss[idx] << "<BR/> ";
			}
		}
	}
	idx++;

	if (dot_para.old_acid_prob) {
		for (int i = 0; i < NUM_NUCL; i++) {
			for (int j = 0; j < num_sites(); j++) {
				double p;
				if (is_act_site(j))
					p = old_lap(dot_para.mut_rate, j, i);
				else
					p = -10000;
				ss[idx] << left << setprecision(4) << setw(9) << (dot_para.is_log_prob ? p : exp(p));
				if (j < num_sites() - 1)
					ss[idx] << " ";
			}
			if (i < NUM_NUCL - 1)
				ss[idx] << "<BR/>";
			else
				if (dot_para.old_br_ac_probs)
					ss[idx] << "<BR/> ";
		}
	}
	idx++;

	if (dot_para.old_br_ac_probs)
	{
		if (!is_active())
		{
			for (int i = 0; i < NUM_NUCL; i++) {
				for (int j = 0; j < num_sites(); j++) {
					double p;
					if (is_act_site(j))
						p = old_lap_anc(dot_para.mut_rate, j, i);
					else
						p = -10000;
					ss[idx] << left << setprecision(4) << setw(9) << (dot_para.is_log_prob ? p : exp(p));
					if (j < num_sites() - 1)
						ss[idx] << " ";
				}
				if (i < NUM_NUCL - 1)
					ss[idx] << "<BR/>";
			}
		}
	}
	idx++;

	if (dot_para.prior_prob)
	{
		if (!is_tip())
		{
			ss[idx] << "P_pr: "<< setprecision(4) << unspec_node_contnr()->P_prior() << " ";
			ss[idx] << "P_ev: "<< setprecision(4) << unspec_node_contnr()->P_gen_event() << "<BR/>";
			ss[idx] << "I_coal: " << setprecision(3) << unspec_node_contnr()->coal_intensity() << " ";
			ss[idx] << "I_rec: " << setprecision(3) << unspec_node_contnr()->recomb_intensity();
		}
	}
	idx++;

	if (dot_para.elig_nodes_sites || dot_para.elig_nodes_sites_old)
	{
		ss[idx] << "Rev_Nds: " << unspec_node_contnr()->num_elig_nodes();
		//if (dot_para.elig_nodes_sites_old && is_nd_st_affected())
		//	ss[idx] << "(" << old_num_elig_nodes() << ")";
		ss[idx] << " ";

		ss[idx] << "Sites: " << unspec_node_contnr()->num_cum_elig_sites();
		//if (dot_para.elig_nodes_sites_old && is_nd_st_affected())
		//	ss[idx] << "(" << old_num_cum_elig_sites() << ")";
		ss[idx] << " ";
	}
	idx++;

	BOOL b = false;
	if (const Rev_Rec_Nd* n = dynamic_cast<const Rev_Rec_Nd*>(this))
		if (n->is_first_in_NC())
			b = true;

	stringstream ret;
	string main_color = (is_active() ? COLOR_ACTIVE : COLOR_ORDINARY);
	ret << "<TD BGCOLOR=\"" << (is_pure_node() ? main_color : COLOR_IMPURE) << "\" WIDTH=\"" << WIDTH_IMPURE << "\"></TD>";
	ret << "<TD ";
	ret << "PORT=\"N" << ((is_single() || is_double() && b) ? 0 : 1) << "\" ";
	ret << "BGCOLOR=\"" << main_color << "\" ";
	ret << ">";
	for (int i = 0; i < MAX_NUM_ROWS; ++i)
		if (ss[i].str().length() > 0)
			ret << ss[i].str() << "<BR/>";
	ret << "</TD>";
	ret << "<TD BGCOLOR=\"" << (is_pure_node() ? main_color : COLOR_IMPURE) << "\" WIDTH=\"" << WIDTH_IMPURE << "\"></TD>";
	return ret.str();
}



// check integrity
void Rev_Nd::check() const
{
	Nd_with_Anc<Rev_Nd, Rev_Coal_Nd, Rev_Rec_Nd, Rev_NC, Rev_Coal_NC, Rev_Rec_NC, Rev_Intern_NC, Rev_Tree>::check();

	BOOL is_ok = true;
	ostringstream oss;
	oss << "Node" << id() << ": ";

	if (!m_is_lap_init || !m_is_crf_sub_init) {
		is_ok = false;
		oss << "Rev_Nd in illegal state" << endl;
	}

	// crf_id and subtype ok?
	if (is_pure_node())
		if (m_subtype < 0) {
			is_ok = false;
			oss << "pure node has negative subtype" << endl;
		}
	if (is_crf_node())
		if (m_crf_id < 0) {
			is_ok = false;
			oss << "crf node has negative CRF id" << endl;
		}

	// occuring subtypes match type of node?
	if (is_pure_node())
		if (occur_subs().size() != 1) {
			is_ok = false;
			oss << "pure node has number of occuring subtypes unequal 1" << endl;
		}
	if (is_crf_node())
		if (occur_subs().size() <= 1) {
			is_ok = false;
			oss << "crf node has number of occuring subtypes not larger than 1" << endl;
		}
	if (is_rec_node())
		if (occur_subs().size() <= 1) {
			is_ok = false;
			oss << "recomb node has number of occuring subtypes not larger than 1" << endl;
		}
	if (is_final_node())
		if (occur_subs().size() <= 1) {
			is_ok = false;
			oss << "final node has number of occuring subtypes not larger than 1" << endl;
		}

	if (!is_ok)
		throw ERROR(logger(), oss.str());
}



// *****************************************************************************
// * Rev_Intern_Nd
// *****************************************************************************

BOOL Rev_Intern_Nd::is_suc_affected() const { return rev_intern_node_contnr()->is_suc_affected(); }
BOOL Rev_Intern_Nd::is_pos_affected() const { return unspec_node_contnr()->is_pos_affected(); }
BOOL Rev_Intern_Nd::is_time_affected() const { return unspec_node_contnr()->is_time_affected(); }
BOOL Rev_Intern_Nd::is_unmarked() const { return unspec_node_contnr()->is_unmarked(); }
BOOL Rev_Intern_Nd::is_new() const { return unspec_node_contnr()->is_new(); }
BOOL Rev_Intern_Nd::is_old() const { return unspec_node_contnr()->is_old(); }
void Rev_Intern_Nd::change_time(double new_time) { unspec_node_contnr()->change_time(new_time); }
void Rev_Intern_Nd::set_unmarked() { unspec_node_contnr()->set_unmarked(); }
void Rev_Intern_Nd::set_suc_affected() { rev_intern_node_contnr()->set_suc_affected(); }
void Rev_Intern_Nd::set_pos_affected() { unspec_node_contnr()->set_pos_affected(); }
void Rev_Intern_Nd::accept(Nd_Visitor* v) { v->visit(this); }



void Rev_Intern_Nd::init_end()
{
	OUT_FNCTN_HEADER(logger());
	VECTOR( VECTOR(double) ) w(Constants::instance()->num_mutation_rates());
	for (VECTOR( VECTOR(double) )::iterator it = w.begin(); it != w.end(); ++it)
		it->resize(NUM_NUCL, -10000);
	ref_log_acid_prob_ptr().resize(tree()->part_alias()->num_groups());
	ref_site_to_prob().resize(num_sites());
	for (int i = 0; i < tree()->part_alias()->num_groups(); ++i)
		if (is_act_group(i)) {
			Log_Acid_Prob* lap = new Log_Acid_Prob(&w);
			ref_log_acid_prob_ptr()[i].push_back(lap);
			for (int sgm = 0; sgm < tree()->part_alias()->num_segms(i); ++sgm)
				for (int site = tree()->part_alias()->begin_segm(i, sgm);
					 site <= tree()->part_alias()->end_segm(i, sgm); ++site) {
					ref_site_to_prob()[site] = lap;
				}
		}
	OUT_FNCTN_BOTTOM(logger());
}



// check integrity
void Rev_Intern_Nd::check() const
{
	Rev_Nd::check();

	BOOL is_ok = true;
	stringstream oss;

	if (!rev_intern_node_contnr()) {
		is_ok = false;
		oss << "rev_intern_node_contnr is NULL";
	}

	if (!is_ok)
		throw ERROR(logger(), oss.str());
}



// *****************************************************************************
// * Rev_Tip_Nd
// *****************************************************************************

Rev_NC* Rev_Tip_Nd::unspec_node_contnr() const { return node_contnr(); }
string Rev_Tip_Nd::acid_seq() const { return Nd_with_Seq<Rev_Nd, Rev_NC, Rev_Tree>::acid_seq(); }
void Rev_Tip_Nd::accept(Nd_Visitor* v) { v->visit(this); }



void Rev_Tip_Nd::init(int num_sites, int crf_sub_id, BOOL is_pure)
{
	OUT_FNCTN_HEADER(logger());

	LOG4CXX_DEBUG(logger(), "crf_sub_id: " << crf_sub_id);
	LOG4CXX_DEBUG(logger(), "is_pure: " << boolalpha << is_pure << endl);

	if (tree()->num_sites() != num_sites)
		throw ERROR(logger(), "A tip has a wrong number of sites");

	if (tree()->all_tips_added())
		throw ERROR(logger(), "Rev_Tip_Nd constructor: Trying to construct tip while non-tips in Rev_Tree");

	set<int> occur_subs;
	enum_node_type type;
	if (is_pure)
	{
		type = PURE_NODE;
		occur_subs.insert(crf_sub_id);
	}
	else
	{
		type = CRF_NODE;
		for (set<int>::iterator it = tree()->crf(crf_sub_id)->occur_subs().begin();
			 it != tree()->crf(crf_sub_id)->occur_subs().end(); ++it)
			occur_subs.insert(*it);
	}
	init_crf_sub(type, crf_sub_id, occur_subs);

	set<int> act_groups;
	for (int i = 0; i < tree()->part_alias()->num_groups(); ++i)
		act_groups.insert(i);
	init_act_groups(act_groups);

	OUT_FNCTN_BOTTOM(logger());
}



// initialize log acid probs
void Rev_Tip_Nd::init_prob()
{
	if (!is_init())
		throw ERROR(logger(), "Trying to initialize log acid probs of Rev_Tip_Nd although its sequence is not given");

	if (!m_is_prob_init) {
		ref_log_acid_prob_ptr().resize(tree()->part_alias()->num_groups());
		ref_site_to_prob().resize(num_sites(), 0);
		for (set<int>::iterator it = act_groups().begin(); it != act_groups().end(); ++it) {
			map<int, Log_Acid_Prob*> acid_to_prob;
			for (int j = 0; j < tree()->part_alias()->num_segms(*it); ++j)
				for (int k = tree()->part_alias()->begin_segm(*it, j);
					 k <= tree()->part_alias()->end_segm(*it, j); ++k) {
					if (acid_to_prob.find(seq_pos(k)) == acid_to_prob.end()) {
						Log_Acid_Prob* lap = new Log_Acid_Prob(seq_pos(k));
						acid_to_prob[seq_pos(k)] = lap;
						ref_log_acid_prob_ptr()[*it].push_back(lap);
					}
					ref_site_to_prob()[k] = acid_to_prob[seq_pos(k)];
				}
		}
	}

	m_is_prob_init = true;
}



Rev_Tip_Nd::Rev_Tip_Nd(Rev_Tree* tr, Rev_Tip_NC* node_contnr, string acid_seq, int crf_sub_id, BOOL is_pure)
: Base_Nd<Rev_Nd, Rev_NC, Rev_Tree>(tr, acid_seq.length(), Empty()),
Prob_Nd<Rev_Nd, Rev_NC, Rev_Tree>(),
Nd_with_Anc<Rev_Nd, Rev_Coal_Nd, Rev_Rec_Nd, Rev_NC, Rev_Coal_NC, Rev_Rec_NC, Rev_Intern_NC, Rev_Tree>(),
Rev_Nd(),
Nd_with_NC<Rev_Nd, Rev_NC, Rev_Tip_NC, Rev_Tree>(node_contnr),
Nd_with_Seq<Rev_Nd, Rev_NC, Rev_Tree>(Constants::instance()->alphabet().indices(acid_seq)),
m_is_prob_init(false)
{
	OUT_FNCTN_HEADER(logger());
	init(acid_seq.length(), crf_sub_id, is_pure);
	OUT_FNCTN_BOTTOM(logger());
}



Rev_Tip_Nd::Rev_Tip_Nd(Rev_Tree* tr, Rev_Tip_NC* node_contnr, int num_sites, int crf_sub_id, BOOL is_pure)
: Base_Nd<Rev_Nd, Rev_NC, Rev_Tree>(tr, num_sites, Empty()),
Prob_Nd<Rev_Nd, Rev_NC, Rev_Tree>(),
Nd_with_Anc<Rev_Nd, Rev_Coal_Nd, Rev_Rec_Nd, Rev_NC, Rev_Coal_NC, Rev_Rec_NC, Rev_Intern_NC, Rev_Tree>(),
Rev_Nd(),
Nd_with_NC<Rev_Nd, Rev_NC, Rev_Tip_NC, Rev_Tree>(node_contnr),
Nd_with_Seq<Rev_Nd, Rev_NC, Rev_Tree>(num_sites),
m_is_prob_init(false)
{
	OUT_FNCTN_HEADER(logger());
	init(num_sites, crf_sub_id, is_pure);
	OUT_FNCTN_BOTTOM(logger());
}



string Rev_Tip_Nd::to_tree_output_string() const
{
	stringstream ss;
	ss << "T              " << flush;
	return ss.str();
}



void Rev_Tip_Nd::check() const
{
	Rev_Nd::check();
	Tip_Nd<Rev_Nd, Rev_NC, Rev_Tip_NC, Rev_Tree>::check();
	Nd_with_Seq<Rev_Nd, Rev_NC, Rev_Tree>::check();

	BOOL is_ok = true;
	ostringstream oss;

	/* ...*/

	if (!is_ok)
		throw ERROR(logger(), oss.str());
}



// *****************************************************************************
// * Rev_Coal_Nd
// *****************************************************************************

Rev_Nd* Rev_Coal_Nd::first_old_child_node() const { return node_contnr()->first_old_child_node(); }
Rev_Nd* Rev_Coal_Nd::second_old_child_node() const { return node_contnr()->second_old_child_node(); }
Rev_NC* Rev_Coal_Nd::unspec_node_contnr() const { return node_contnr(); }
void Rev_Coal_Nd::set_old() { node_contnr()->set_old(); }
void Rev_Coal_Nd::accept(Nd_Visitor* v) { v->visit(this); }



Rev_Coal_Nd::Rev_Coal_Nd(Rev_Tree* tr, Rev_Coal_NC* node_contnr, Rev_Nd* first_old_node, Rev_Nd* second_old_node)
: Base_Nd<Rev_Nd, Rev_NC, Rev_Tree>(tr, first_old_node, second_old_node, Empty()),
Prob_Nd<Rev_Nd, Rev_NC, Rev_Tree>(),
Nd_with_Anc<Rev_Nd, Rev_Coal_Nd, Rev_Rec_Nd, Rev_NC, Rev_Coal_NC, Rev_Rec_NC, Rev_Intern_NC, Rev_Tree>(first_old_node, second_old_node),
Rev_Nd(),
Nd_with_NC<Rev_Nd, Rev_NC, Rev_Coal_NC, Rev_Tree>(node_contnr),
Rev_Intern_Nd(node_contnr)
{
	OUT_FNCTN_HEADER(logger());

	if (!tree()->part_alias_calced())
		throw Error("Trying to construct node although the partition for aliasing has not been generated");

	if (!tree()->all_tips_added())
		throw ERROR(logger(), "Constuctor Rev_Intern_Nd: Trying to construct non-tip although set_to_all_tips_added() has not been called ");

	LOG4CXX_DEBUG(logger(), "first_old_node: " << first_old_node->id());
	LOG4CXX_DEBUG(logger(), "second_old_node: " << second_old_node->id() << endl);

	LOG4CXX_DEBUG(logger(), "sub/crf/type");
	set<int> occur_subs;
	enum_node_type type;
	int crf_sub_id;
	if ((first_old_node->type() == PURE_NODE) && (second_old_node->type() == PURE_NODE) &&
	    (first_old_node->sub() == second_old_node->sub()))
	{
		type = PURE_NODE;
		crf_sub_id = first_old_node->sub();
		occur_subs.insert(crf_sub_id);
	}
	else
		if (((first_old_node->type() == PURE_NODE) || (first_old_node->type() == FINAL_NODE))
			&& ((second_old_node->type() == PURE_NODE) || (second_old_node->type() == FINAL_NODE)))
			{
				type = FINAL_NODE;
				if (first_old_node->type() == PURE_NODE)
					occur_subs.insert(first_old_node->sub());
				else
					occur_subs.insert(first_old_node->occur_subs().begin(),
							 				 first_old_node->occur_subs().end());
				if (second_old_node->type() == PURE_NODE)
					occur_subs.insert(second_old_node->sub());
				else
					occur_subs.insert(second_old_node->occur_subs().begin(),
											 second_old_node->occur_subs().end());
			}
		else
			if ((first_old_node->type() == CRF_NODE) && (second_old_node->type() == CRF_NODE)
			    && (first_old_node->crf_id() == second_old_node->crf_id()))
			{
				type = CRF_NODE;
				crf_sub_id = first_old_node->crf_id();
				occur_subs = set<int>(first_old_node->occur_subs().begin(), first_old_node->occur_subs().end());
			}
			else
				throw ERROR(logger(), "Constuctor Rev_Intern_Nd: trying to coalesce illegal pair of nodes");

	init_crf_sub(type, crf_sub_id, occur_subs);

	VECTOR(int) v(tree()->part_alias()->num_groups());
	first_old_node->act_groups().begin();
	VECTOR(int)::iterator it =
		set_union(first_old_node->act_groups().begin(), first_old_node->act_groups().end(),
			      second_old_node->act_groups().begin(), second_old_node->act_groups().end(),
			      v.begin());
	set<int> act_groups;
	for (VECTOR(int)::iterator jt = v.begin(); jt != it; ++jt) {
		act_groups.insert(*jt);
	}
	init_act_groups(act_groups);

	init_end();
	OUT_FNCTN_BOTTOM(logger());
}



// recalculates act_sites/act_groups
void Rev_Coal_Nd::recalc_act_sites_grps()
{
	Base_Nd<Rev_Nd, Rev_NC, Rev_Tree>::recalc_act_sites(first_child_node(), second_child_node());
	Prob_Nd<Rev_Nd, Rev_NC, Rev_Tree>::recalc_act_grps(first_child_node(), second_child_node());
}



// recalculates occuring subs of node and its container
void Rev_Coal_Nd::recalc_occur_subs()
{
	set<int> occur_subs;
	occur_subs.insert(first_child_node()->occur_subs().begin(), first_child_node()->occur_subs().end());
	occur_subs.insert(second_child_node()->occur_subs().begin(), second_child_node()->occur_subs().end());
	reset_occur_subs(occur_subs);
}



// calculates log acid probs for a node based on the log acid probs of its children
void Rev_Coal_Nd::calc_log_acid_probs()
{
	OUT_FNCTN_HEADER(logger());
	LOG4CXX_DEBUG(logger(), "COAL");
	Rev_Nd* first_child = first_child_node();
	Rev_Nd* second_child = second_child_node();
	BOOL is_completely_changed;   // has the log acid prob of the node to be calculated completely again?
	if (is_time_affected() || is_suc_affected() || !log_acid_probs_calced())
		is_completely_changed = true;
	else
		is_completely_changed = false;

	for (set<int>::const_iterator it = act_groups().begin(); it != act_groups().end(); ++it) {   // iterate over all groups of the partition
		BOOL first_changed = first_child->is_changed_group(*it);
		BOOL second_changed = second_child->is_changed_group(*it);

		// calculate probs contributed by the first child
		if (is_completely_changed || first_changed) {
			if (log_acid_probs_calced())   // only set affected if it is not the first time the log acid prob is calculated
				first_child->set_ac_pr_affected();
			for (VECTOR(Log_Acid_Prob*)::const_iterator jt = first_child->log_acid_prob_ptr()[*it].begin();
				 jt != first_child->log_acid_prob_ptr()[*it].end(); ++jt) {
				for (int i = 0; i < NUM_NUCL; i++)
					for (int mr = 0; mr < tree()->num_mutation_rates(); ++mr) {
						double r[NUM_NUCL];
						for (int j = 0; j < NUM_NUCL; j++)
							r[j] = first_child->get_log_subst_prob(mr, i, j) + (*jt)->log_p(mr, j);
						double q = 0;
						sum_probs(&q, r);
						(*jt)->log_p_anc(mr, i) = q;
						LOG4CXX_DEBUG(logger(), "first_child "
									  << " idx " << jt - first_child->log_acid_prob_ptr()[*it].begin()
									  << " i " << i << " mr " << mr << " q " << q);
					}
			}

		}

		// calculate probs contributed by the second child
		if (is_completely_changed || second_changed) {
			if (log_acid_probs_calced())   // only set affected if it is not the first time the log acid prob is calculated
				second_child->set_ac_pr_affected();
			for (VECTOR(Log_Acid_Prob*)::const_iterator jt = second_child->log_acid_prob_ptr()[*it].begin();
				 jt != second_child->log_acid_prob_ptr()[*it].end(); ++jt) {
				for (int i = 0; i < NUM_NUCL; i++)
					for (int mr = 0; mr < tree()->num_mutation_rates(); ++mr) {
						double r[NUM_NUCL];
						for (int j = 0; j < NUM_NUCL; j++)
							r[j] = second_child->get_log_subst_prob(mr, i, j) + (*jt)->log_p(mr, j);
						double q = 0;
						sum_probs(&q, r);
						(*jt)->log_p_anc(mr, i) = q;

						LOG4CXX_DEBUG(logger(), "second_child "
									  << " idx " << jt - second_child->log_acid_prob_ptr()[*it].begin()
									  << " i " << i << " mr " << mr << " q " << q);
					}
			}
		}

		LOG4CXX_DEBUG(logger(), "");

		// calculate probs of current node
		if (is_completely_changed || first_changed || second_changed)
		{
			if (log_acid_probs_calced())   // only set affected if it is not the first time the log acid prob is calculated
				set_ac_pr_affected();
			ref_changed_groups().insert(*it);

			if (!first_child->is_act_group(*it)) {   // only second child contributes (first child has inactive group)
				LOG4CXX_DEBUG(logger(), "first_child missing");
				for (VECTOR(Log_Acid_Prob*)::const_iterator jt = log_acid_prob_ptr()[*it].begin();
					 jt != log_acid_prob_ptr()[*it].end(); ++jt)
					checked_delete(*jt);
				ref_log_acid_prob_ptr()[*it].clear();
				map<Log_Acid_Prob*, Log_Acid_Prob*> mp;
				for (VECTOR(Log_Acid_Prob*)::const_iterator jt = second_child->log_acid_prob_ptr()[*it].begin();
					 jt != second_child->log_acid_prob_ptr()[*it].end(); ++jt) {
					Log_Acid_Prob* lap = new Log_Acid_Prob((*jt)->log_p_anc());
					ref_log_acid_prob_ptr()[*it].push_back(lap);
					mp[*jt] = lap;
				}

				for (int sgm = 0; sgm < tree()->part_alias()->num_segms(*it); ++sgm)
					for (int site = tree()->part_alias()->begin_segm(*it, sgm);
						 site <= tree()->part_alias()->end_segm(*it, sgm); ++site) {
						ref_site_to_prob()[site] = mp[second_child->site_to_prob()[site]];
						LOG4CXX_DEBUG(logger(), "site: " << site);
					}
				LOG4CXX_DEBUG(logger(), "");
			}
			else
				if (!second_child->is_act_group(*it)) {   // only first child contributes (second child has inactive group)
					LOG4CXX_DEBUG(logger(), "second_child missing");
					for (VECTOR(Log_Acid_Prob*)::const_iterator jt = log_acid_prob_ptr()[*it].begin();
						 jt != log_acid_prob_ptr()[*it].end(); ++jt)
						checked_delete(*jt);
					ref_log_acid_prob_ptr()[*it].clear();
					map<Log_Acid_Prob*, Log_Acid_Prob*> mp;
					for (VECTOR(Log_Acid_Prob*)::const_iterator jt = first_child->log_acid_prob_ptr()[*it].begin();
						 jt != first_child->log_acid_prob_ptr()[*it].end(); ++jt) {
						Log_Acid_Prob* lap = new Log_Acid_Prob((*jt)->log_p_anc());
						ref_log_acid_prob_ptr()[*it].push_back(lap);
						mp[*jt] = lap;
					}

					for (int sgm = 0; sgm < tree()->part_alias()->num_segms(*it); ++sgm)
						for (int site = tree()->part_alias()->begin_segm(*it, sgm);
							 site <= tree()->part_alias()->end_segm(*it, sgm); ++site) {
							ref_site_to_prob()[site] = mp[first_child->site_to_prob()[site]];
							LOG4CXX_DEBUG(logger(), "site: " << site);
						}
					LOG4CXX_DEBUG(logger(), "");
				}
				else {   // both children contribute (both have active group)
					LOG4CXX_DEBUG(logger(), "both children present");
					for (VECTOR(Log_Acid_Prob*)::const_iterator jt = log_acid_prob_ptr()[*it].begin();
						 jt != log_acid_prob_ptr()[*it].end(); ++jt)
						checked_delete(*jt);
					ref_log_acid_prob_ptr()[*it].clear();
					map< pair<Log_Acid_Prob*, Log_Acid_Prob*>, Log_Acid_Prob*> mp;
					for (int sgm = 0; sgm < tree()->part_alias()->num_segms(*it); ++sgm)
						for (int site = tree()->part_alias()->begin_segm(*it, sgm);
							 site <= tree()->part_alias()->end_segm(*it, sgm); ++site) {
							Log_Acid_Prob* lap;
							pair<Log_Acid_Prob*, Log_Acid_Prob*> curr_pair(first_child->site_to_prob()[site],
																		   second_child->site_to_prob()[site]);
							map< pair<Log_Acid_Prob*, Log_Acid_Prob*>, Log_Acid_Prob*>::iterator jt =
								mp.find(curr_pair);
							if (jt == mp.end()) {
								VECTOR( VECTOR(double) ) v(Constants::instance()->num_mutation_rates());
								for (int mr = 0; mr < Constants::instance()->num_mutation_rates(); ++mr)
									v[mr].resize(NUM_NUCL);
								for (int i = 0; i < NUM_NUCL; i++)
									for (int mr = 0; mr < tree()->num_mutation_rates(); ++mr) {
										v[mr][i] = first_child->site_to_prob()[site]->log_p_anc(mr, i)
												   + second_child->site_to_prob()[site]->log_p_anc(mr, i);
									}
								lap = new Log_Acid_Prob(&v);
								ref_log_acid_prob_ptr()[*it].push_back(lap);
								mp[curr_pair] = lap;
							}
							else
								lap = jt->second;
							ref_site_to_prob()[site] = lap;
							LOG4CXX_DEBUG(logger(), "site: " << site);
						}
					LOG4CXX_DEBUG(logger(), "");
				}
		}
	}

	LOG4CXX_DEBUG(logger(), "log_acid_prob_ptr (" << id() << ")");
	stringstream ss;
	for (set<int>::iterator it = act_groups().begin(); it != act_groups().end(); ++it)
		for (VECTOR(Log_Acid_Prob*)::const_iterator jt = log_acid_prob_ptr()[*it].begin();
			 jt != log_acid_prob_ptr()[*it].end(); ++jt) {
			ss << "grp " << *it << " log_acid_prob (id) " << (*jt)->id() << endl;
			ss << **jt << endl;
		}
	LOG4CXX_DEBUG(logger(), ss.str());
	LOG4CXX_DEBUG(logger(), "site_to_prob");
	ss.str("");
	for (int i = 0; i < num_sites(); ++i) {
		if (is_act_site(i))
			ss << "i " << i << " log_acid_prob(id) " << site_to_prob()[i]->id() << endl;
	}
	LOG4CXX_DEBUG(logger(), ss.str());

	set_log_acid_probs_calced();
	OUT_FNCTN_BOTTOM(logger());
}



// calculate mutation matrix recursively
void Rev_Coal_Nd::calc_log_subst_probs()
{
	for (int i = 0; i < tree()->num_mutation_rates(); ++i)
	{
		get_subst_prob_matrix(first_child_node()->log_subst_prob()[i], tree()->decomp(), tree()->mutation_rate(i),
							  birth_time() - first_child_node()->birth_time());
		get_subst_prob_matrix(second_child_node()->log_subst_prob()[i], tree()->decomp(), tree()->mutation_rate(i),
				              birth_time() - second_child_node()->birth_time());
	}
	first_child_node()->calc_log_subst_probs();
	second_child_node()->calc_log_subst_probs();
}



void Rev_Coal_Nd::correct_after_move_hook_node(Rev_Coal_Nd* dest, BOOL is_move_down)
{
	Prob_Nd<Rev_Nd, Rev_NC, Rev_Tree>::correct_after_move_hook_node(dest, is_move_down);
}



// check integrity
void Rev_Coal_Nd::check() const
{
	Rev_Intern_Nd::check();
	Coal_Nd<Rev_Nd, Rev_NC, Rev_Coal_NC, Rev_Tree>::check();

	BOOL is_ok = true;
	ostringstream oss;

	// check integrity of the order of pure and final nodes
	if (is_final_node())
		for (Rev_Nd* n = next(); n; n = n->next())
			if (n->is_pure_node() && (occur_subs().find(n->sub()) != occur_subs().end()))
			{
				is_ok = false;
				oss << "pure node: " << n << " (sub: " << n->sub() << ")"
				   << " older than final impure node " << *this << " (subs: ";
				BOOL is_first = true;
				FOREACH (int i, occur_subs()) {
					if (!is_first)
						oss << " ";
					is_first = false;
					oss << i;
				}
				oss << ")" << endl;
			}

	if (!is_ok)
		throw ERROR(logger(), oss.str());
}



string Rev_Coal_Nd::to_tree_output_string() const
{
	stringstream ss;
	ss << "C " << flush;
	ss.width(3);
	ss << first_child_node()->id() << flush;
	ss << "(C) " << flush;
	ss.width(3);
	ss << second_child_node()->id() << flush;
	ss << "(C)" << flush;
	return ss.str();
}



// *****************************************************************************
// * Rev_Rec_Nd
// *****************************************************************************

Rev_Nd*& Rev_Rec_Nd::child_node() const { return Rec_Nd<Rev_Nd, Rev_NC, Rev_Rec_NC, Rev_Tree>::child_node(); }
BOOL Rev_Rec_Nd::is_first_in_NC() const { return Rec_Nd<Rev_Nd, Rev_NC, Rev_Rec_NC, Rev_Tree>::is_first_in_NC(); }
Rev_Nd* Rev_Rec_Nd::old_child_node() const { return node_contnr()->old_child_node(); }
Rev_NC* Rev_Rec_Nd::unspec_node_contnr() const { return node_contnr(); }
void Rev_Rec_Nd::set_old() { node_contnr()->set_old(); }
void Rev_Rec_Nd::check() const { Rev_Intern_Nd::check(); Rec_Nd<Rev_Nd, Rev_NC, Rev_Rec_NC, Rev_Tree>::check(); }
void Rev_Rec_Nd::accept(Nd_Visitor* v) { v->visit(this); }



Rev_Rec_Nd::Rev_Rec_Nd(Rev_Tree* tr, Rev_Rec_NC* node_contnr, Rev_Nd* old_node, set<int> subtypes_para, BOOL is_first_node)
: Base_Nd<Rev_Nd, Rev_NC, Rev_Tree>(tr, old_node->num_sites(), Empty()),
Prob_Nd<Rev_Nd, Rev_NC, Rev_Tree>(),
Nd_with_Anc<Rev_Nd, Rev_Coal_Nd, Rev_Rec_Nd, Rev_NC, Rev_Coal_NC, Rev_Rec_NC, Rev_Intern_NC, Rev_Tree>(old_node),
Rev_Nd(),
Nd_with_NC<Rev_Nd, Rev_NC, Rev_Rec_NC, Rev_Tree>(node_contnr),
Rec_Nd<Rev_Nd, Rev_NC, Rev_Rec_NC, Rev_Tree>(is_first_node),
Rev_Intern_Nd(node_contnr)
{
	OUT_FNCTN_HEADER(logger());

	if (!tree()->part_alias_calced())
		throw Error("Trying to construct node although the partition for aliasing has not been generated");
	if (!tree()->all_tips_added())
		throw Error("Rev_Nd constructor: Trying to construct non-tip although set_to_all_tips_added() has not been called ");

	LOG4CXX_DEBUG(logger(), "old_node: " << old_node->id());
	stringstream ss;
	ss << "subtypes: ";
	for (set<int>::iterator it = subtypes_para.begin(); it != subtypes_para.end(); ++it)
		ss << *it << " ";
	LOG4CXX_DEBUG(logger(), ss.str());
	LOG4CXX_DEBUG(logger(), "is_first_node: " << boolalpha << is_first_node << endl);

	LOG4CXX_DEBUG(logger(), "sub/crf");
	set<int> occur_subs = subtypes_para;
	enum_node_type type;
	int crf_sub_id;
	if (occur_subs.size() == 1)
	{
		type = PURE_NODE;
		crf_sub_id = *occur_subs.begin();
	}
	else
	{
		type = RECOMB_NODE;
		crf_sub_id = old_node->crf_id();
	}
	init_crf_sub(type, crf_sub_id, occur_subs);

	LOG4CXX_DEBUG(logger(), "act_sites");
	int crf_id = old_node->crf_id();
	is_act_segment().resize(tree()->crf(crf_id)->num_segms());
	VECTOR(BOOL) is_act_site(num_sites());
	int left_most_act_site, right_most_act_site;

	int i = 0;
	BOOL is_first = true;
	for (int j = 0; j < tree()->crf(crf_id)->num_segms(); ++j, ++i)
	{
		int left, right;
		if (i > 0)
			left = tree()->crf(crf_id)->bp(i - 1) + 1;
		else
			left = 0;
		if (i < tree()->crf(crf_id)->num_segms() - 1)
			right = tree()->crf(crf_id)->bp(i);
		else
			right = num_sites() - 1;

		if (subtypes_para.find(tree()->crf(crf_id)->segm_sub(j)) == subtypes_para.end())
		{
			is_act_segment()[i] = false;
			for (int j = left; j <= right; j++)
				is_act_site[j] = false;
		}
		else
		{
			is_act_segment()[i] = true;
			for (int j = left; j <= right; j++)
				is_act_site[j] = true;
			if (is_first)
			{
				left_most_act_site = left;
				is_first = false;
			}
			right_most_act_site = right;
		}
	}
	init_act_sites(is_act_site, left_most_act_site, right_most_act_site);

	LOG4CXX_DEBUG(logger(), "act_groups");
	set<int> act_groups;
	for (int i = 0; i < tree()->part_alias()->num_groups(); ++i)
		if (occur_subs.find(tree()->crf(crf_id)->sub(tree()->part_alias()->begin_segm(i, 0))) != occur_subs.end())
			act_groups.insert(i);
	init_act_groups(act_groups);

	init_end();
	OUT_FNCTN_BOTTOM(logger());
}



// hook a Rev_Rec_Node in the rest of the tree (see hook_in.pdf)
Rev_Coal_Nd* Rev_Rec_Nd::hook_in(Rev_Nd* n, double time)
{
	OUT_FNCTN_HEADER(logger());
	LOG4CXX_DEBUG(logger(), "this: " << id());
	LOG4CXX_DEBUG(logger(), "n: " << n->id());
	LOG4CXX_DEBUG(logger(), "time: " << time << endl);

	Rev_Coal_NC* p = n->parent_node()->node_contnr();
	Rev_Coal_NC* new_node_contnr;
	if (n->is_first_child())
		new_node_contnr = new Rev_Coal_NC(tree(), time, n, this);
	else
		new_node_contnr = new Rev_Coal_NC(tree(), time, this, n);
	new_node_contnr->node()->set_coal_anc(p, n->is_first_child());

	OUT_FNCTN_BOTTOM(logger());
	return new_node_contnr->node();
}



// calculates log acid probs for a node based on the log acid probs of its child
void Rev_Rec_Nd::calc_log_acid_probs()
{
	OUT_FNCTN_HEADER(logger());
	LOG4CXX_DEBUG(logger(), "REC");
	Rev_Nd* child = child_node();
	BOOL is_completely_changed;   // has the log acid prob of the node to be calculated completely again?
	if (is_time_affected() || is_suc_affected() || !log_acid_probs_calced())
		is_completely_changed = true;
	else
		is_completely_changed = false;

	for (set<int>::iterator it = act_groups().begin(); it != act_groups().end(); ++it) {
		BOOL child_changed = child->is_changed_group(*it);
		if (is_completely_changed || child_changed) {
			if (log_acid_probs_calced())   // only set affected if it is not the first time the log acid prob is calculated
				child->set_ac_pr_affected();

			// calculate probs contributed by the child
			for (VECTOR(Log_Acid_Prob*)::const_iterator jt = child->log_acid_prob_ptr()[*it].begin();
				 jt != child->log_acid_prob_ptr()[*it].end(); ++jt) {
				for (int i = 0; i < NUM_NUCL; i++)
					for (int mr = 0; mr < tree()->num_mutation_rates(); ++mr) {
						double r[NUM_NUCL];
						for (int j = 0; j < NUM_NUCL; j++)
							r[j] = child->get_log_subst_prob(mr, i, j) + (*jt)->log_p(mr, j);
						double q = 0;
						sum_probs(&q, r);
						(*jt)->log_p_anc(mr, i) = q;

						LOG4CXX_DEBUG(logger(), "child "
									  << " idx " << jt - child->log_acid_prob_ptr()[*it].begin()
									  << " i " << i << " mr " << mr << " q " << q);
					}
			}

			if (log_acid_probs_calced())   // only set affected if it is not the first time the log acid prob is calculated
				set_ac_pr_affected();
			ref_changed_groups().insert(*it);

			// calculate probs of current node
			for (VECTOR(Log_Acid_Prob*)::const_iterator jt = log_acid_prob_ptr()[*it].begin();
				 jt != log_acid_prob_ptr()[*it].end(); ++jt)
				checked_delete(*jt);
			ref_log_acid_prob_ptr()[*it].clear();
			map<Log_Acid_Prob*, Log_Acid_Prob*> mp;
			for (VECTOR(Log_Acid_Prob*)::const_iterator jt = child->log_acid_prob_ptr()[*it].begin();
				 jt != child->log_acid_prob_ptr()[*it].end(); ++jt) {
				Log_Acid_Prob* lap = new Log_Acid_Prob((*jt)->log_p_anc());
				ref_log_acid_prob_ptr()[*it].push_back(lap);
				mp[*jt] = lap;
			}

			for (int sgm = 0; sgm < tree()->part_alias()->num_segms(*it); ++sgm)
				for (int site = tree()->part_alias()->begin_segm(*it, sgm);
					 site <= tree()->part_alias()->end_segm(*it, sgm); ++site)
					ref_site_to_prob()[site] = mp[child->site_to_prob()[site]];
		}
	}

	LOG4CXX_DEBUG(logger(), "log_acid_prob_ptr (" << id() << ")");
	stringstream ss;
	for (set<int>::const_iterator it = act_groups().begin(); it != act_groups().end(); ++it)
		for (VECTOR(Log_Acid_Prob*)::const_iterator jt = log_acid_prob_ptr()[*it].begin();
			 jt != log_acid_prob_ptr()[*it].end(); ++jt) {
			ss << "grp " << *it << " log_acid_prob (id) " << (*jt)->id() << endl;
			ss << **jt << endl;
		}
	LOG4CXX_DEBUG(logger(), ss.str());
	LOG4CXX_DEBUG(logger(), "site_to_prob");
	ss.str("");
	for (int i = 0; i < num_sites(); ++i) {
		if (is_act_site(i))
			ss << "i " << i << " log_acid_prob(id) " << site_to_prob()[i]->id() << endl;
	}
	LOG4CXX_DEBUG(logger(), ss.str());

	set_log_acid_probs_calced();
	OUT_FNCTN_BOTTOM(logger());
}



// calculate mutation matrix recursively
void Rev_Rec_Nd::calc_log_subst_probs()
{
	for (int i = 0; i < tree()->num_mutation_rates(); ++i)
		get_subst_prob_matrix(child_node()->log_subst_prob()[i], tree()->decomp(), tree()->mutation_rate(i),
							  birth_time() - child_node()->birth_time());
	child_node()->calc_log_subst_probs();
}



string Rev_Rec_Nd::to_tree_output_string() const
{
	stringstream ss;
	if (is_first_in_NC())
	{
		ss << "R1" << flush;
		ss.width(3);
		ss << child_node()->id() << flush;
		ss << "(C) " << flush;
		ss.width(3);
		ss << child_node()->second_parent_node()->id() << flush;
		ss << "(S)" << flush;
	}
	else
	{
		ss << "R2" << flush;
		ss.width(3);
		ss << child_node()->id() << flush;
		ss << "(C) " << flush;
		ss.width(3);
		ss << child_node()->first_parent_node()->id() << flush;
		ss << "(S)" << flush;
	}
	return ss.str();
}

