#include "standard.h"

#include "dot.h"
#include "files.h"
#include "sim_nc.h"
#include "sim_nd.h"
#include "sim_tree.h"
#include "tree.h"

using namespace std;

LoggerPtr Sim_NC::m_logger(Logger::getLogger("node.sim_nc"));



// *****************************************************************************
// * Sim_NC
// *****************************************************************************

VECTOR(Sim_Nd*) Sim_NC::sim_nodes() const { return unspec_nodes(); }
void Sim_NC::set_num_cum_elig_sites(int i) { Base_NC<Sim_Nd, Sim_NC, Sim_Intern_NC, Sim_Tree>::ref_num_cum_elig_sites() = i; }
void Sim_NC::set_num_elig_nodes(int i) { Base_NC<Sim_Nd, Sim_NC, Sim_Intern_NC, Sim_Tree>::ref_num_elig_nodes() = i; }



void Sim_NC::connect_prev_and_next()
{
	OUT_FNCTN_HEADER(logger());

	Sim_NC* c = tree()->first_node_contnr();
	if (!c) {
		tree()->ref_first_node_contnr() = this;
		tree()->ref_last_node_contnr() = this;
		prev() = 0;
		next() = 0;
	}
	else {
		while (!c->is_last() && (c->birth_time() <= birth_time()))
			c = c->next();
		if (c->birth_time() > birth_time())
		{
			next() = c;
			prev() = c->prev();
			c->prev()->next() = this;
			c->prev() = this;
		}
		else
		{
			tree()->last_node_contnr()->next() = this;
			prev() = tree()->last_node_contnr();
			tree()->m_last_node_contnr = this;
			next() = 0;
		}
	}

	OUT_FNCTN_BOTTOM(logger());
}



string Sim_NC::to_dot_string(Dot_Para dot_para) const
{
	const string COLOR_TIP = "cyan";
	const string COLOR_ORDINARY = "grey78";
	const int WIDTH_RECOMB = 1;

	// determine color of container
	string color;
	if (is_tip())
		color = COLOR_TIP;
	else
		color = COLOR_ORDINARY;

	// write Dot output for container and its nodes
	stringstream ss;
	ss << to_dot_node_name() << " [color=\"" << color << "\" label=<" << endl;
	ss << "<TABLE CELLSPACING=\"0\" BORDER=\"0\">" << endl;
	ss << "<TR>";
	if (const Sim_Tip_NC* c = dynamic_cast<const Sim_Tip_NC*>(this))
		ss << c->node()->to_dot_string(dot_para);
	else
		if (const Sim_Coal_NC* c = dynamic_cast<const Sim_Coal_NC*>(this))
			ss << c->node()->to_dot_string(dot_para);
		else
			if (const Sim_Rec_NC* c = dynamic_cast<const Sim_Rec_NC*>(this))
			{
				ss << c->first_node()->to_dot_string(dot_para);
				ss << "<TD BGCOLOR=\"black\" WIDTH=\"" << WIDTH_RECOMB << "\"></TD>";
				ss << c->second_node()->to_dot_string(dot_para);
			}
	ss << "</TR>" << endl;
	ss << "</TABLE>>];" << endl;

	return ss.str();
}



// *****************************************************************************
// * Sim_Tip_NC
// *****************************************************************************

Sim_Tip_NC::Sim_Tip_NC(Sim_Tree* tr, int id)
: Base_NC<Sim_Nd, Sim_NC, Sim_Intern_NC, Sim_Tree>(tr),
Tip_NC<Sim_Nd, Sim_Tip_Nd, Sim_NC, Sim_Intern_NC, Sim_Tree>()
{
	OUT_FNCTN_HEADER(logger());
	Sim_Tip_Nd* new_node = new Sim_Tip_Nd(tr, this, tr->num_sites(), id);
	init_node(new_node);
	connect_prev_and_next();
	OUT_FNCTN_BOTTOM(logger());
}



VECTOR(Sim_Nd*) Sim_Tip_NC::unspec_nodes() const
{
	VECTOR(Sim_Nd*) v;
	v.push_back(NC_with_1_Nd<Sim_Nd, Sim_Tip_Nd, Sim_NC, Sim_Intern_NC, Sim_Tree>::node());
	return v;
}



// *****************************************************************************
// * Sim_Coal_NC
// *****************************************************************************

Sim_Coal_NC::Sim_Coal_NC(Sim_Tree* tr, double birth_time, Sim_Nd* first_old_node, Sim_Nd* second_old_node, int id)
: Base_NC<Sim_Nd, Sim_NC, Sim_Intern_NC, Sim_Tree>(tr, birth_time, first_old_node, second_old_node),
Coal_NC<Sim_Nd, Sim_Coal_Nd, Sim_NC, Sim_Coal_NC, Sim_Intern_NC, Sim_Tree>(birth_time, this, first_old_node, second_old_node)
{
	OUT_FNCTN_HEADER(logger());
	Sim_Coal_Nd* new_node = new Sim_Coal_Nd(tr, this, birth_time, first_old_node, second_old_node, id);
	init_node(new_node);
	connect_prev_and_next();
	OUT_FNCTN_BOTTOM(logger());
}



VECTOR(Sim_Nd*) Sim_Coal_NC::unspec_nodes() const
{
	VECTOR(Sim_Nd*) v;
	v.push_back(node());
	return v;
}



// *****************************************************************************
// * Sim_Rec_NC
// *****************************************************************************

Sim_Rec_NC::Sim_Rec_NC(Sim_Tree* tr, double birth_time,
        	   		   Sim_Nd* old_node, VECTOR(int) bp, int id_left_parent, int id_right_parent)
: Base_NC<Sim_Nd, Sim_NC, Sim_Intern_NC, Sim_Tree>(tr, birth_time, old_node),
Rec_NC<Sim_Nd, Sim_Rec_Nd, Sim_NC, Sim_Rec_NC, Sim_Intern_NC, Sim_Tree>(birth_time, this, old_node)
{
	OUT_FNCTN_HEADER(logger());
	Sim_Rec_Nd* first_new_node = new Sim_Rec_Nd(tr, this, birth_time, old_node, bp, true, id_left_parent);
	Sim_Rec_Nd* second_new_node = new Sim_Rec_Nd(tr, this, birth_time, old_node, bp, false, id_right_parent);
	init_nodes(first_new_node, second_new_node);
	connect_prev_and_next();
	OUT_FNCTN_BOTTOM(logger());
}



VECTOR(Sim_Nd*) Sim_Rec_NC::unspec_nodes() const
{
   	VECTOR(Sim_Nd*) v;
   	v.push_back(first_node());
   	v.push_back(second_node());
   	return v;
}







