#ifndef SIM_NODE_CONTNR_H_
#define SIM_NODE_CONTNR_H_

#include "standard.h"

#include "base_nc.h"

using namespace std;

class Sim_Nd;
class Sim_Coal_Nd;
class Sim_Rec_Nd;
class Sim_Tip_Nd;
class Sim_NC;
class Sim_Intern_NC;
class Sim_Tree;
template<class N, class C, class PC, class T> class Base_NC;



class Sim_NC : public virtual Base_NC<Sim_Nd, Sim_NC, Sim_Intern_NC, Sim_Tree>
{
private:
	static LoggerPtr m_logger;
protected:
	Sim_NC() { }
	static LoggerPtr& logger() { return m_logger; }
public:
	void set_num_cum_elig_sites(int i);
	void set_num_elig_nodes(int i);
	void connect_prev_and_next();
	VECTOR(Sim_Nd*) sim_nodes() const;
	string to_dot_string(Dot_Para dot_para) const;
	void check() const { Base_NC<Sim_Nd, Sim_NC, Sim_Intern_NC, Sim_Tree>::check(); }
};



class Sim_Tip_NC : public virtual Sim_NC, public virtual Tip_NC<Sim_Nd, Sim_Tip_Nd, Sim_NC, Sim_Intern_NC, Sim_Tree>
{
private:
	VECTOR(Sim_Nd*) unspec_nodes() const;
public:
	Sim_Tip_NC(Sim_Tree* tr, int id);
	using Sim_NC::logger;
	void check() const { Sim_NC::check(); Tip_NC<Sim_Nd, Sim_Tip_Nd, Sim_NC, Sim_Intern_NC, Sim_Tree>::check(); }
};



class Sim_Intern_NC : public virtual Sim_NC
{
private:
protected:
	Sim_Intern_NC() { }
public:
	void check() const { Sim_NC::check(); }
};



class Sim_Coal_NC
: public virtual Sim_Intern_NC,
public virtual Coal_NC<Sim_Nd, Sim_Coal_Nd, Sim_NC, Sim_Coal_NC, Sim_Intern_NC, Sim_Tree>
{
private:
	VECTOR(Sim_Nd*) unspec_nodes() const;
public:
	Sim_Coal_NC(Sim_Tree* tr, double birth_time, Sim_Nd* first_old_node, Sim_Nd* second_old_node, int id_para);
	using Sim_NC::logger;
	void check() const { Sim_Intern_NC::check(); Coal_NC<Sim_Nd, Sim_Coal_Nd, Sim_NC, Sim_Coal_NC, Sim_Intern_NC, Sim_Tree>::check(); }	
};



class Sim_Rec_NC
: public virtual Sim_Intern_NC,
public virtual Rec_NC<Sim_Nd, Sim_Rec_Nd, Sim_NC, Sim_Rec_NC, Sim_Intern_NC, Sim_Tree>
{
private:
	VECTOR(Sim_Nd*) unspec_nodes() const;
public:
	Sim_Rec_NC(Sim_Tree* tr, double birth_time,
			   Sim_Nd* old_node, VECTOR(int) bp, int id_left_parent, int id_right_parent);
	using Sim_NC::logger;
	void check() const { Sim_Intern_NC::check(); Rec_NC<Sim_Nd, Sim_Rec_Nd, Sim_NC, Sim_Rec_NC, Sim_Intern_NC, Sim_Tree>::check(); }	
};





#endif
