#include "standard.h"

#include "constants.h"
#include "dot.h"
#include "error.h"
#include "files.h"
#include "random.h"
#include "sim_nc.h"
#include "sim_nd.h"
#include "sim_tree.h"
#include "subst_prob.h"
#include "sim_tree.h"

using namespace std;

class Rev_Tree;
class Rev_NC;

LoggerPtr Sim_Nd::m_logger(Logger::getLogger("node.sim_nd"));



// *****************************************************************************
// * Sim_Nd
// *****************************************************************************

BOOL Sim_Nd::is_first() const {	return(id() == tree()->first_node()->id()); }
BOOL Sim_Nd::is_last() const { return(id() == tree()->last_node()->id()); }



string Sim_Nd::to_dot_string(Dot_Para dot_para) const
{
	const string COLOR_ACTIVE = "orange";
	const string COLOR_ORDINARY = "grey87";
	const int MAX_NUM_ROWS = 10;

	stringstream ss[MAX_NUM_ROWS];
	int idx = 0;

	ss[idx] << id();
	idx++;

	if (dot_para.seq_tip && is_tip() || dot_para.seq_intr && !is_tip())
		ss[idx] << Base_Nd<Sim_Nd, Sim_NC, Sim_Tree>::acid_seq();
	idx++;

	BOOL b = false;
	if (const Sim_Rec_Nd* n = dynamic_cast<const Sim_Rec_Nd*>(this))
		if (n->is_first_in_NC())
			b = true;
	stringstream ret;
	string main_color = (is_active() ? COLOR_ACTIVE : COLOR_ORDINARY);
	ret << "<TD ";
	ret << "PORT=\"N" << ((is_single() || is_double() && b) ? 0 : 1) << "\" ";
	ret << "BGCOLOR=\"" << main_color << "\" ";
	ret << ">";
	for (int i = 0; i < MAX_NUM_ROWS; ++i)
		if (ss[i].str().length() > 0)
			ret << ss[i].str() << "<BR/>";
	ret << "</TD>";
	return ret.str();
}



// calculate probability of acids
double Sim_Nd::acid_prob(int site, int acid)
{
	abort();
	/*mp_real p;

	switch (suc_type)
	{
	case COAL_SUC:
	{
		p = 1;
		if (smpl_suc.children.first->m_is_act_site[site])
		{
			mp_real q = 0;
			for (int i = 0; i < NUM_NUCL; i++)
			{
				q += smpl_suc.children.first->get_subst_prob(acid, i) * smpl_suc.children.first->acid_prob(site, i);
			}
			p *= q;
		}
		if (this->smpl_suc.children.second->m_is_act_site[site])
		{
			mp_real q = 0;
			for (int i = 0; i < NUM_NUCL; i++)
				q += smpl_suc.children.second->get_subst_prob(acid, i) * smpl_suc.children.second->acid_prob(site, i);
			p *= q;
		}
		break;
	}
	case RECOMB_SUC_FIRST:
	case RECOMB_SUC_SECOND:
		if (smpl_suc.child->m_is_act_site[site])
		{
			p = 0;
			for (int i = 0; i < NUM_NUCL; i++)
				p += smpl_suc.child->get_subst_prob(acid, i) * smpl_suc.child->acid_prob(site, i);
		}
		else
			p = 1;
		break;
	case TIP:
		if (acid == get_acid(site))
			p = 1;
		else
			p = 0;
		break;
	default:
		p = 0;
		break;
	}

	P_acid[site][acid] = p;

	return p;*/
}



void Sim_Nd::check() const
{
	Nd_with_Anc<Sim_Nd, Sim_Coal_Nd, Sim_Rec_Nd, Sim_NC, Sim_Coal_NC, Sim_Rec_NC, Sim_Intern_NC, Sim_Tree>::check();
	Nd_with_Seq<Sim_Nd, Sim_NC, Sim_Tree>::check();
}


// *****************************************************************************
// * Sim_Tip_Nd
// *****************************************************************************

Sim_NC* Sim_Tip_Nd::unspec_node_contnr() const { return node_contnr(); }



Sim_Tip_Nd::Sim_Tip_Nd(Sim_Tree* tr, Sim_Tip_NC* node_contnr, int num_sites, int id)
: Base_Nd<Sim_Nd, Sim_NC, Sim_Tree>(tr, num_sites, id),
Nd_with_Anc<Sim_Nd, Sim_Coal_Nd, Sim_Rec_Nd, Sim_NC, Sim_Coal_NC, Sim_Rec_NC, Sim_Intern_NC, Sim_Tree>(),
Nd_with_Seq<Sim_Nd, Sim_NC, Sim_Tree>(num_sites),
Sim_Nd(),
Nd_with_NC<Sim_Nd, Sim_NC, Sim_Tip_NC, Sim_Tree>(node_contnr)
{
	OUT_FNCTN_HEADER(logger());
	OUT_FNCTN_BOTTOM(logger());
}



// *****************************************************************************
// * Sim_Coal_Nd
// *****************************************************************************

Sim_NC* Sim_Coal_Nd::unspec_node_contnr() const { return node_contnr(); }



Sim_Coal_Nd::Sim_Coal_Nd(Sim_Tree* tr, Sim_Coal_NC* node_contnr, double birth_time, Sim_Nd* first_old_node,
		    			 Sim_Nd* second_old_node, int id)
: Base_Nd<Sim_Nd, Sim_NC, Sim_Tree>(tr, first_old_node, second_old_node, id),
Nd_with_Anc<Sim_Nd, Sim_Coal_Nd, Sim_Rec_Nd, Sim_NC, Sim_Coal_NC, Sim_Rec_NC, Sim_Intern_NC, Sim_Tree>(first_old_node, second_old_node),
Nd_with_Seq<Sim_Nd, Sim_NC, Sim_Tree>(first_old_node->num_sites()),
Sim_Nd(),
Nd_with_NC<Sim_Nd, Sim_NC, Sim_Coal_NC, Sim_Tree>(node_contnr)
{
	OUT_FNCTN_HEADER(logger());
	OUT_FNCTN_BOTTOM(logger());
}



// simulate mutation recursively
void Sim_Coal_Nd::sim_mutation(int site, double mutation_rate)
{
	OUT_FNCTN_HEADER(logger());
	LOG4CXX_DEBUG(logger(), "id: " << id() << ", site: " << site << ", mutation_rate: " << mutation_rate);
	if (first_child_node()->is_act_site(site))
	{
		double prob[NUM_NUCL];
		for (int j = 0; j < NUM_NUCL; j++)
			prob[j] = subst_prob(tree()->decomp(), mutation_rate, birth_time() - first_child_node()->birth_time(),
								 seq_pos(site), j);

		gsl_ran_discrete_t* grd = gsl_ran_discrete_preproc (NUM_NUCL, prob);
		int k = gsl_ran_discrete (Random::rng(), grd);
		first_child_node()->set_seq_pos(site, k);
		gsl_ran_discrete_free(grd);

		LOG4CXX_DEBUG(logger(), id() << " Cl1 " << site << " " << acid(site) << " "
								<< Constants::instance()->alphabet().character(k) << " "
								<< birth_time() << " " << first_child_node()->birth_time() << " " << endl
								<< prob[0] << " " << prob[1] << " " << prob[2] << " " << prob[3]);
		first_child_node()->sim_mutation(site, mutation_rate);
	}
	if (second_child_node()->is_act_site(site))
	{
		double prob[NUM_NUCL];
		for (int j = 0; j < NUM_NUCL; j++)
			prob[j] = subst_prob(tree()->decomp(), mutation_rate, birth_time() - second_child_node()->birth_time(),
						         seq_pos(site), j);

		gsl_ran_discrete_t* grd = gsl_ran_discrete_preproc (NUM_NUCL, prob);
		int k = gsl_ran_discrete (Random::rng(), grd);
		second_child_node()->set_seq_pos(site, k);
		gsl_ran_discrete_free(grd);

		LOG4CXX_DEBUG(logger(), id() << " Cl2 " << site << " " << acid(site) << " "
							    << Constants::instance()->alphabet().character(k) << " "
							    << birth_time() << " " << second_child_node()->birth_time() << " " << endl
							    << prob[0] << " " << prob[1] << " " << prob[2] << " " << prob[3]);
		second_child_node()->sim_mutation(site, mutation_rate);
	}

	OUT_FNCTN_BOTTOM(logger());
}



// *****************************************************************************
// * Sim_Rec_Nd
// *****************************************************************************

Sim_NC* Sim_Rec_Nd::unspec_node_contnr() const { return node_contnr(); }



Sim_Rec_Nd::Sim_Rec_Nd(Sim_Tree* tr, Sim_Rec_NC* node_contnr, double birth_time, Sim_Nd* old_node,
			   		   VECTOR(int) bp, BOOL has_first_segment, int id)
: Base_Nd<Sim_Nd, Sim_NC, Sim_Tree>(tr, old_node->num_sites(), id),
Nd_with_Anc<Sim_Nd, Sim_Coal_Nd, Sim_Rec_Nd, Sim_NC, Sim_Coal_NC, Sim_Rec_NC, Sim_Intern_NC, Sim_Tree>(old_node),
Nd_with_Seq<Sim_Nd, Sim_NC, Sim_Tree>(old_node->num_sites()),
Sim_Nd(),
Nd_with_NC<Sim_Nd, Sim_NC, Sim_Rec_NC, Sim_Tree>(node_contnr),
Rec_Nd<Sim_Nd, Sim_NC, Sim_Rec_NC, Sim_Tree>(has_first_segment)
{
	OUT_FNCTN_HEADER(logger());

	VECTOR(BOOL) is_act_site(num_sites());
	int left_most_act_site, right_most_act_site;

	BOOL is_active = has_first_segment;
	for (int j = 0; j <= bp[0]; j++)
		is_act_site[j] = is_active && old_node->is_act_site(j);
	is_active = not(is_active);

	for (int i = 1; i < (int)bp.size(); i++)
	{
		for (int j = bp[i - 1] + 1; j <= bp[i]; j++)
			is_act_site[j] = is_active && old_node->is_act_site(j);
		is_active = not(is_active);
	}

	for (int j = bp[bp.size() - 1] + 1; j < num_sites(); j++)
		is_act_site[j] = is_active && old_node->is_act_site(j);

	BOOL is_first = true;
	for (int i = 0; i < num_sites(); i++)
	{
		if (is_act_site[i])
		{
			if (is_first)
			{
				left_most_act_site = i;
				is_first = false;
			}
			right_most_act_site = i;
		}
	}

	Base_Nd<Sim_Nd, Sim_NC, Sim_Tree>::init_act_sites(is_act_site, left_most_act_site, right_most_act_site);

	OUT_FNCTN_BOTTOM(logger());
}



// simulate mutation recursively
void Sim_Rec_Nd::sim_mutation(const int site, const double mutation_rate)
{
	OUT_FNCTN_HEADER(logger());

	if (child_node()->is_act_site(site))
	{
		double prob[NUM_NUCL];
		for (int j = 0; j < NUM_NUCL; j++)
			prob[j] = subst_prob(tree()->decomp(), mutation_rate, birth_time() - child_node()->birth_time(),
								 seq_pos(site), j);

		gsl_ran_discrete_t* grd = gsl_ran_discrete_preproc (NUM_NUCL, prob);
		int k = gsl_ran_discrete (Random::rng(), grd);
		child_node()->set_seq_pos(site, k);
		gsl_ran_discrete_free(grd);

		LOG4CXX_DEBUG(logger(), id() << " Rec " << site << " " << acid(site) << " "
								<< Constants::instance()->alphabet().character(k) << " "
								<< birth_time() << " " << child_node()->birth_time() << " " << endl
								<< prob[0] << " " << prob[1] << " " << prob[2] << " " << prob[3]);
		child_node()->sim_mutation(site, mutation_rate);

	}

	OUT_FNCTN_BOTTOM(logger());
}




