#ifndef SIM_NODE_H_
#define SIM_NODE_H_

#include "standard.h"

#include "base_nd.h"

using namespace std;

class Sim_Coal_Nd;
class Sim_Rec_Nd;
class Sim_Tip_NC;
class Sim_Intern_NC;
class Sim_Coal_NC;
class Sim_Rec_NC;



class Sim_Nd
: public virtual Nd_with_Anc<Sim_Nd, Sim_Coal_Nd, Sim_Rec_Nd, Sim_NC, Sim_Coal_NC, Sim_Rec_NC, Sim_Intern_NC, Sim_Tree>,
public virtual Nd_with_Seq<Sim_Nd, Sim_NC, Sim_Tree>
{
private:
	static LoggerPtr m_logger;
protected:
	Sim_Nd() { }
	static LoggerPtr& logger() { return m_logger; }
public:
	BOOL is_first() const;
	BOOL is_last() const;
	string to_dot_string(Dot_Para dot_para) const;
	double acid_prob(int site, int acid);
	virtual void sim_mutation(int site, double mutation_rate) = 0;

	virtual void check() const;
};



class Sim_Tip_Nd : public virtual Sim_Nd, public virtual Tip_Nd<Sim_Nd, Sim_NC, Sim_Tip_NC, Sim_Tree>
{
private:
protected:
public:
	Sim_Tip_Nd(Sim_Tree* tr, Sim_Tip_NC* node_contnr, int num_sites, int id);
	using Sim_Nd::logger;
	Sim_NC* unspec_node_contnr() const;
	void sim_mutation(int site, double mutation_rate) { }
	void check() const { Sim_Nd::check(); }
};



class Sim_Intern_Nd : public virtual Sim_Nd
{
private:
protected:
public:
	void check() const { Sim_Nd::check(); }
};



class Sim_Coal_Nd : public virtual Coal_Nd<Sim_Nd, Sim_NC, Sim_Coal_NC, Sim_Tree>, public virtual Sim_Intern_Nd
{
private:
protected:
public:
	Sim_Coal_Nd(Sim_Tree* tr, Sim_Coal_NC* node_contnr, double birth_time, Sim_Nd* first_old_node,
			    Sim_Nd* second_old_node, int id) ;
	using Sim_Nd::logger;
	Sim_NC* unspec_node_contnr() const;
	void sim_mutation(int site, double mutation_rate);
	void check() const { Sim_Intern_Nd::check(); }
};



class Sim_Rec_Nd : public virtual Rec_Nd<Sim_Nd, Sim_NC, Sim_Rec_NC, Sim_Tree>, public virtual Sim_Intern_Nd
{
private:
protected:
public:
	Sim_Rec_Nd(Sim_Tree* tr, Sim_Rec_NC* node_contnr, double birth_time, Sim_Nd* old_node,
			   VECTOR(int) bp, BOOL has_first_segment, int id);
	using Sim_Nd::logger;
	Sim_NC* unspec_node_contnr() const;
	void sim_mutation(int site, double mutation_rate);
	void check() const { Sim_Intern_Nd::check(); }
};



#endif
