#include "standard.h"

#include "constants.h"
#include "sum_probs.h"

using namespace std;



// auxiliary function for Rev_Tree::log_prob_acids for summing up the acid probabilities (given in an array of size NUM_NUCL)
// p = r_1 + ... + r_{NUM_NUCL}
void sum_probs(double* p, double* r)
{
	double max = r[0];
	int argmax = 0;
	for (int k = 1; k < NUM_NUCL; k++)
	{
		if (r[k] > max)
		{
			max = r[k];
			argmax = k;
		}
	}

	*p = max;
	double q = 1;
	for (int k = 0; k < NUM_NUCL; k++)
		if (k != argmax)
		{
			q += exp(r[k] - max);
		}
	*p += log(q);
}



// auxiliary function for Rev_Tree::log_prob_acids for summing up the acid probabilities (given in an arbitrary vector)
// p *= v_1 + ... + v_n
void sum_probs(double* p, VECTOR(double) v)
{
	double max = v[0];
	int argmax = 0;
	for (int k = 1; k < v.size(); k++)
	{
		if (v[k] > max)
		{
			max = v[k];
			argmax = k;
		}
	}

	*p = max;
	double q = 1;
	for (int k = 0; k < v.size(); k++)
		if (k != argmax)
		{
			q += exp(v[k] - max);
		}
	*p += log(q);
}

