#include "standard.h"

#include "files.h"
#include "random.h"

using namespace std;

unsigned int Random::rng_seed = 0;
BOOL Random::is_init = false;
gsl_rng* Random::internal_rng = NULL;


// initialize Random generator
void Random::init(unsigned int s)
{
	ofstream* out_strm = In_Out_Streams::instance()->ostrm(OUTPUT_FILE);
	internal_rng = gsl_rng_alloc(gsl_rng_mt19937);
	rng_seed = s;
	gsl_rng_set(internal_rng, rng_seed);
	*out_strm << "rng_seed: " << rng_seed << endl << endl;
	is_init = true;
}

// get Random generator
gsl_rng* Random::rng()
{
	if (!is_init)
		init();
	return internal_rng;
}



// get a Random pair of unequal ints <= n
void Random::get_rnd_pair(int n, int* x, int* y)
{
	*x = gsl_rng_uniform_int(rng(), n);
	*y = gsl_rng_uniform_int(rng(), n - 1);
	if (*y >= *x)
		(*y)++;
}



// get a Random partition of a set into two non-empty sets
void Random::get_rnd_partition(const set<int> old, set<int>* first_new, set<int>* second_new)
{
	set<int> s = old;

	if (old.size() > 0)
	{
		int idx = gsl_rng_uniform_int(rng(), s.size());
		int i = 0;
		set<int>::iterator jt;
		for (set<int>::iterator it = s.begin(); it != s.end(); ++it, ++i)
			if (i == idx)
				jt = it;
		first_new->insert(*jt);
		s.erase(*jt);

		idx = gsl_rng_uniform_int(rng(), s.size());
		i = 0;
		for (set<int>::iterator it = s.begin(); it != s.end(); ++it, ++i)
			if (i == idx)
				jt = it;
		second_new->insert(*jt);
		s.erase(*jt);
	}

	for (set<int>::const_iterator it = s.begin(); it != s.end(); ++it)
		if (gsl_rng_uniform(rng()) > 0.5)
			first_new->insert(*it);
		else
			second_new->insert(*it);
}

