#include "standard.h"

#include "classftn.h"
#include "constants.h"
#include "dot.h"
#include "files.h"
#include "MCMC.h"
#include "random.h"
#include "tip_seqs.h"
#include "test.h"



#ifdef TEST_MODE
	#define BOOST_TEST_DYN_LINK
	#define BOOST_TEST_MODULE general

	#include <boost/test/included/unit_test.hpp>
	#include <boost/test/floating_point_comparison.hpp>



	struct Main_Config {
		Main_Config()
		{
			const string LOGGER_NAME = "test";
			Constants::instance()->init();
			In_Out_Streams::instance()->init();
			PropertyConfigurator::configure(In_Out_Streams::instance()->filename(LOG_CONFIG_FILE));
			LoggerPtr logger(Logger::getLogger(LOGGER_NAME));

			Constants::instance()->single_arg_file() = true;

			Dot::set_para(1, 0,	   // anc
						  0, 0,    // suc
						  0,       // anc_suc_num
						  0, 0,    // next
						  0, 0,    // prev
						  0, 0,    // seq (tips, interior)
						  1,       // sub
						  0,       // affected
						  0,       // alias_groups
						  0, 0,	   // first_last
						  0, 0,    // mut_rate (int), is log prob
						  0, 0, 0, // acid_prob_grp, acid_prob, br_ac_probs,
						  0, 0,    // old_acid_prob, old_br_ac_probs
						  0, 0, 0, // prior_prob, nodes/sites
						  2.0);    // min_edge_len
			Constants::instance()->messages_on() = true;
			Constants::instance()->compile_dot() = true;
			Constants::instance()->compile_dot_immediately() = true;
			Constants::instance()->use_common_output_file() = true;
			Constants::instance()->delete_spec_output_file() = false;
			Dot::compile_to_printable() = true;
			Constants::instance()->compile_sim_tree() = true;
			Constants::instance()->compile_best_tree() = true;

			int seed = 0;
			Random::init(seed);
		}

		~Main_Config()
		{
			In_Out_Streams::instance()->close();
		}

		LoggerPtr logger;
	};

	BOOST_GLOBAL_FIXTURE( Main_Config );

	BOOST_AUTO_TEST_CASE( Classftn_Basic7 )
	{
		int num_sites = 6;
		Tip_Seqs tip_seqs(7, num_sites);
		Classftn cls("input/classifications/Basic7.txt", tip_seqs.num_sites());
		Results_MCMC res_mcmc = cls.log_prob_var(tip_seqs, 30, 1);
		cout << "ln L: " << setprecision(8) << res_mcmc.log_p()
			 << " (prior: " << res_mcmc.log_p_prior() << ", acids: " << res_mcmc.log_p_acids() << ")" << endl;
	}

#endif
