#ifndef BASE_TREE_H_
#define BASE_TREE_H_

#include "standard.h"

using namespace std;

class Decomposition;
class Tip_Seqs;
class Constants;
class Partition_Aliasing;




template<class N, class CN, class RN, class C> class Base_Tree
{
	friend class Rev_Nd;
	friend class Rev_NC;
	friend class Sim_Nd;
	friend class Sim_NC;
private:
	Decomposition* m_decomp;
	VECTOR(double) m_log_prob_mutation_rates;
	int m_num_mutation_rates;
	VECTOR(double) m_mutation_rates;
	static LoggerPtr m_logger;
	LoggerPtr& logger() const { return m_logger; }

	virtual void check_node_contnr() const = 0;
protected:
	void calc_num_elig_nd_st(C* start, C* end);
public:
	int m_num_sites, m_num_nodes, m_num_act_nodes, m_num_tips;   // private
	C* m_first_node_contnr;   // private
	C* m_last_node_contnr;   // private
	void set_first_node_contnr(C* first) { m_first_node_contnr = first; }   // protected
	void set_last_node_contnr(C* last) { m_last_node_contnr = last; }   // protected
	void calc_prior_prob(C* start);   // protected

	Base_Tree(const int num_sites_para, const int num_mutation_rates);
	virtual ~Base_Tree();

	void set_num_sites(int num_sites_para) { m_num_sites = num_sites_para; }
	void set_num_nodes(int num_nodes_para) { m_num_nodes = num_nodes_para; }
	void set_num_act_nodes(int num_act_nodes_para) { m_num_act_nodes = num_act_nodes_para; }
	void set_num_tips(int num_tips_para) { m_num_tips = num_tips_para; }
	int num_sites() const { return m_num_sites; }
	int num_nodes() const { return m_num_nodes; }
	int num_act_nodes() const { return m_num_act_nodes; }
	int num_tips() const { return m_num_tips; }
	N* first_node() const { return m_first_node_contnr ? m_first_node_contnr->unspec_nodes().front() : 0; }
	N* last_node() const { return m_last_node_contnr ? m_last_node_contnr->unspec_nodes().back() : 0; }
	C* first_node_contnr() const { return m_first_node_contnr; }
	C*& ref_first_node_contnr() { return m_first_node_contnr; }
	C* last_node_contnr() const { return m_last_node_contnr; }
	C*& ref_last_node_contnr() { return m_last_node_contnr; }
	N* find_node(int id) const;
	const Decomposition& decomp() const { return *m_decomp; }

	virtual double log_prior_prob() = 0;
	int num_mutation_rates() const { return m_num_mutation_rates; }
	double log_prob_mutation_rate(const int i) const { return m_log_prob_mutation_rates[i]; }
	double prob_mutation_rate(const int i) const { return exp(m_log_prob_mutation_rates[i]); }
	double mutation_rate(const int i) const { return m_mutation_rates[i]; }

	virtual void check() const;
};

#endif
