#ifndef CHANGES_H
#define CHANGES_H

#include "standard.h"

using namespace std;

class Rev_Nd;
class Rev_Tree;
class Rev_NC;
class Rev_Intern_NC;



class Changes
{
	friend ostream& operator<<(ostream& s, const Changes& changes);
private:
	BOOL m_rearranging_done;
	BOOL m_adjusting_pos_necessary;
	set<Rev_NC*> m_old_node_contnrs;
	set<Rev_NC*> m_new_node_contnrs;
	set<Rev_Nd*> m_anc_aff_nodes;
	set<Rev_Nd*> m_ac_pr_aff_nodes;
	set<Rev_Intern_NC*> m_suc_aff_node_contnrs;
	set<Rev_NC*> m_pos_aff_node_contnrs;
	set<Rev_NC*> m_time_aff_node_contnrs;
	set<Rev_NC*> m_nd_st_aff_node_contnrs;
	Rev_Tree* m_tree;
	static LoggerPtr s_logger;

	LoggerPtr& logger() const { return s_logger; }
public:
	Changes(Rev_Tree* tree);

	BOOL rearranging_done() const { return m_rearranging_done; }
	BOOL adjusting_pos_necessary() const { return m_adjusting_pos_necessary; }
	const set<Rev_NC*>& old_node_contnrs() { return m_old_node_contnrs; }
	const set<Rev_NC*>& new_node_contnrs() { return m_new_node_contnrs; }

	void set_rearranging_done() { m_rearranging_done = true; }
	void set_adjusting_pos_necessary() { m_adjusting_pos_necessary = true; }
	void add_to_old(Rev_NC* c) { m_old_node_contnrs.insert(c); }
	void add_to_new(Rev_NC* c) { m_new_node_contnrs.insert(c); }
	void add_to_anc_aff(Rev_Nd* n) { m_anc_aff_nodes.insert(n); }
	void add_to_ac_pr_aff(Rev_Nd* n) { m_ac_pr_aff_nodes.insert(n); }
	void add_to_suc_aff(Rev_Intern_NC* c) { m_suc_aff_node_contnrs.insert(c); }
	void add_to_pos_aff(Rev_NC* c) { m_pos_aff_node_contnrs.insert(c); }
	void add_to_time_aff(Rev_NC* c) { m_time_aff_node_contnrs.insert(c); }
	void add_to_nd_st_aff(Rev_NC* c) { m_nd_st_aff_node_contnrs.insert(c); }

	void accept();
	void reverse();
	void adjust_pos();
	void delete_new();
	void delete_old();
	void restore_anc();
	void restore_ac_pr();
	void restore_suc();
	void restore_pos();
	void restore_time();
	void restore_nd_st();
	void unmark_new();
	void unmark_old();
	void reset_anc();
	void reset_ac_pr();
	void reset_suc();
	void reset_pos();
	void reset_time();
 	void reset_nd_st();
};

ostream& operator<<(ostream& s, const Changes& changes);

#endif
