#include "standard.h"

#include "constants.h"
#include "decomposition.h"
#include "files.h"

using namespace std;



Decomposition::Decomposition()
{
    ifstream* eig_val_strm = In_Out_Streams::instance()->istrm(EIG_VAL_FILE);
    ifstream* eig_vec_strm = In_Out_Streams::instance()->istrm(EIG_VEC_FILE);
    ifstream* inv_eig_vec_strm = In_Out_Streams::instance()->istrm(INV_EIG_VEC_FILE);
    eig_val_strm->seekg (0, ios::beg);
    eig_vec_strm->seekg (0, ios::beg);
    inv_eig_vec_strm->seekg (0, ios::beg);

	m_T = gsl_matrix_alloc (NUM_NUCL, NUM_NUCL);
	m_inv_T = gsl_matrix_alloc (NUM_NUCL, NUM_NUCL);
	for (int i = 0; i < NUM_NUCL; i++)
		for (int j = 0; j < NUM_NUCL; j++)
		{
			double x;
			*eig_vec_strm >> x;
			gsl_matrix_set(m_T, i, j, x);
			*inv_eig_vec_strm >> x;
			gsl_matrix_set(m_inv_T, i, j, x);
		}

	m_eig_val.resize(NUM_NUCL);
	for (int i = 0; i < NUM_NUCL; i++)
		*eig_val_strm >> m_eig_val[i];
}



Decomposition::~Decomposition()
{
	gsl_matrix_free(m_T);
	gsl_matrix_free(m_inv_T);
}



ostream& operator<<(ostream& s, const Decomposition& dcmp)
{
	for (int i = 0; i < NUM_NUCL; i++)
		for (int j = 0; j < NUM_NUCL; j++)
		{
			s << i << " "  << j << " " << gsl_matrix_get(dcmp.m_T, i, j) << endl;
		}
	s << endl;

	return s;
}
