#include "standard.h"

#include "constants.h"
#include "decomposition.h"
#include "files.h"
#include "subst_prob.h"

using namespace std;



// calculate log substitution probability matrix
void get_subst_prob_matrix(gsl_matrix* Q, const Decomposition& decomp, double mutation_rate, double time)
{
	OUT_FNCTN_HEADER(subst_prob_logger);

	gsl_matrix* P = gsl_matrix_alloc (NUM_NUCL, NUM_NUCL);
	gsl_matrix* D = gsl_matrix_alloc (NUM_NUCL, NUM_NUCL);
	gsl_matrix_set_zero(D);
	for (int i = 0; i < NUM_NUCL; i++)
		gsl_matrix_set(D, i, i, exp(Constants::instance()->theta() * mutation_rate * time * decomp.eig_val()[i]));

	for (int i = 0; i < NUM_NUCL; i++)
		LOG4CXX_DEBUG(subst_prob_logger, i << " " << exp(Constants::instance()->theta() * mutation_rate * time * decomp.eig_val()[i]));

	gsl_blas_dgemm (CblasNoTrans, CblasNoTrans, 1.0, &decomp.T(), D, 0.0, P);
	gsl_blas_dgemm (CblasNoTrans, CblasNoTrans, 1.0, P, &decomp.inv_T(), 0.0, Q);

	for (int i = 0; i < NUM_NUCL; i++)
		for (int j = 0; j < NUM_NUCL; j++)
		{
			gsl_matrix_set(Q, i, j, log(gsl_matrix_get(Q, i, j)));
		}

	gsl_matrix_free(P);
	gsl_matrix_free(D);
	
	OUT_FNCTN_BOTTOM(subst_prob_logger);
}



// calculate substitution probability
double subst_prob(const Decomposition& decomp, double mutation_rate, double time, int first_acid, int second_acid)
{
	OUT_FNCTN_HEADER(subst_prob_logger);
	LOG4CXX_DEBUG(subst_prob_logger, "mutation_rate: " << mutation_rate << ", time: " << time 
									 << ", first_acid: " << first_acid << ", second_acid: " << second_acid);  
			
	gsl_matrix* P = gsl_matrix_alloc (NUM_NUCL, NUM_NUCL);
	gsl_matrix* Q = gsl_matrix_alloc (NUM_NUCL, NUM_NUCL);
	gsl_matrix* D = gsl_matrix_alloc (NUM_NUCL, NUM_NUCL);
	gsl_matrix_set_zero(D);
	for (int i = 0; i < NUM_NUCL; i++)
		gsl_matrix_set(D, i, i, exp(Constants::instance()->theta() * mutation_rate * time * decomp.eig_val()[i]));

	for (int i = 0; i < NUM_NUCL; i++)
		LOG4CXX_DEBUG(subst_prob_logger, i << " " << exp(Constants::instance()->theta() * mutation_rate * time * decomp.eig_val()[i]));

	gsl_blas_dgemm (CblasNoTrans, CblasNoTrans, 1.0, &decomp.T(), D, 0.0, P);
	gsl_blas_dgemm (CblasNoTrans, CblasNoTrans, 1.0, P, &decomp.inv_T(), 0.0, Q);

	double p = gsl_matrix_get(Q, first_acid, second_acid);

	gsl_matrix_free(P);
	gsl_matrix_free(Q);
	gsl_matrix_free(D);

	OUT_FNCTN_BOTTOM(subst_prob_logger);

	return(p);
}

