#include "standard.h"

#include "changes.h"
#include "constants.h"
#include "dot.h"
#include "files.h"
#include "nc.h"
#include "nd.h"
#include "random.h"
#include "tr_rearr_brch.h"
#include "tree.h"

using namespace std;

class Rev_Intern_Nd;
class Rev_Nd;

LoggerPtr tree_rearranger_brch::s_logger(Logger::getLogger("tree.tr_rearr_brch"));



// perform rearrange opeartion "branch"
void tree_rearranger_brch::branch(Rev_Tree* tr, Rev_Coal_Nd* target)
{
	OUT_FNCTN_HEADER(logger());
	LOG4CXX_DEBUG(logger(), "target id: " << target->id());

	if (tr->changes()->rearranging_done())
		throw ERROR(logger(), "Rearranging coalescent: Illegal Rev_Tree");

	tr->changes()->set_rearranging_done();
	tr->save_first_last();

	const int subtype = (target->has_crf() ? 0 : target->sub());
	const int crf_id = (target->has_crf() ? target->crf_id() : 0);
	const double target_time = target->birth_time();
	const BOOL target_child_is_first = (gsl_rng_uniform(Random::rng()) > 0.5 ? true : false);

	Rev_Coal_Nd* target_parent = target->parent_node();
	Rev_Nd* target_child;
	Rev_Nd* other_child;
	if (target_child_is_first)
	{
		target_child = target->first_child_node();
		other_child = target->second_child_node();
	}
	else
	{
		target_child = target->second_child_node();
		other_child = target->first_child_node();
	}

	// find possible destinations for branch
	VECTOR(Rev_Nd*) elig_dests;
	for (Rev_Nd* n = tr->first_node(); !n->is_last() && (n->birth_time() < target_time); n = n->next())
	{
		if (((target->is_pure_node()) && (n->is_pure_node()) && (n->sub() == subtype) ||
			 (target->is_crf_node()) && (n->is_crf_node()) && (n->crf_id() == crf_id)) &&
			(n->time_of_parent() > target_time) && (n->id() != target->id()) && (n->id() != target_child->id()) &&
			(n->id() != other_child->id()))
			elig_dests.push_back(n);
	}

	// output possible destination nodes
	stringstream ss;
	ss << "elig_dest_nodes: ";
	for (VECTOR(Rev_Nd*)::iterator it = elig_dests.begin(); it != elig_dests.end(); it++)
		ss << (*it)->id() << " ";
	LOG4CXX_DEBUG(logger(), ss.str());

	if (elig_dests.size() > 0)
	{
		// set nodes affected at old location
		target_parent->node_contnr()->set_suc_affected();
		other_child->set_anc_affected();
		target->set_anc_affected();
		target->node_contnr()->set_suc_affected();

		// output children and parent of target
		LOG4CXX_DEBUG(logger(), "parent id: " << target_parent->id());
		LOG4CXX_DEBUG(logger(), "target_child id: " << target_child->id());
		LOG4CXX_DEBUG(logger(), "other_child id: " << other_child->id());

		// reconnect nodes at old location
		if (target->is_first_child())
		{
			target_parent->first_child_node() = other_child;
			other_child->set_to_first_coal_child();
		}
		else
		{
			target_parent->second_child_node() = other_child;
			other_child->set_to_second_coal_child();
		}
		other_child->ref_parent_contnr() = target_parent->node_contnr();

		// set nodes affected at destination of branch
		Rev_Nd* dest = elig_dests[gsl_rng_uniform_int(Random::rng(), elig_dests.size())];
		Rev_Coal_Nd* dest_parent = dest->parent_node();
		dest->set_anc_affected();
		dest_parent->node_contnr()->set_suc_affected();

		// output destination
		LOG4CXX_DEBUG(logger(), "dest: " << dest->id());

		// connect nodes at destination of branch
		BOOL dest_is_first_child = dest->is_first_child();
		target->m_anc = dest_parent->node_contnr();
		if (target_child_is_first)
		{
			target->second_child_node() = dest;
			dest->set_to_second_coal_child();
		}
		else
		{
			target->first_child_node() = dest;
			dest->set_to_first_coal_child();
		}
		dest->m_anc = target->node_contnr();

		if (dest_is_first_child)
		{
			dest_parent->first_child_node() = target;
			target->set_to_first_coal_child();
		}
		else
		{
			dest_parent->second_child_node() = target;
			target->set_to_second_coal_child();
		}

		tr->recalc_act_sites_grps();
		tr->recalc_occur_subs();
	}

	OUT_FNCTN_BOTTOM(logger());
}



