#include "standard.h"

#include "changes.h"
#include "crf.h"
#include "constants.h"
#include "dot.h"
#include "nc.h"
#include "nd.h"
#include "files.h"
#include "part_tree.h"
#include "prt_nd_conn.h"
#include "random.h"
#include "tr_rearr_rec.h"
#include "tr_rearr_sngl.h"
#include "tree.h"

LoggerPtr tree_rearranger_rec::s_logger(Logger::getLogger("tree.tr_rearr_rec"));




// get all possible targets for rearrange operation "recombination"
void tree_rearranger_rec::get_recomb_nodes_rearr(Rev_Tree* tr, VECTOR(Rev_Nd*) &recomb_nodes, Rev_Nd* n)
{
	if (n->is_crf_node())
	{
		recomb_nodes.push_back(n);
		get_recomb_nodes_rearr(tr, recomb_nodes, n->first_parent_node());
		get_recomb_nodes_rearr(tr, recomb_nodes, n->second_parent_node());
	}
}


void tree_rearranger_rec::get_elig_targets_hook(Rev_Tree* tr, enum_direction dir, VECTOR(Rev_Coal_NC*)* elig_targets)
{
	OUT_FNCTN_HEADER(logger());
	LOG4CXX_DEBUG(logger(), "dir: " << dir << endl);

	for (Rev_NC* c = tr->first_node_contnr(); c; c = c->next())
	{
		LOG4CXX_DEBUG(logger(), c->str_id());
		if (c->has_coal_suc())
		{
			Rev_Coal_NC* d = dynamic_cast<Rev_Coal_NC*>(c);
			switch (dir)
			{
				case DIR_DOWN:
					if ((d->first_child()->is_single() && d->second_child()->is_double() && d->second_child()->is_new() ||
						 d->first_child()->is_double() && d->second_child()->is_single() && d->first_child()->is_new()))
						if 	(d->first_child()->is_single() && (d->prev() == d->first_child()->rev_node_contnr()) ||
							 d->second_child()->is_single() && (d->prev() == d->second_child()->rev_node_contnr()))
							elig_targets->push_back(d);
					break;
				case DIR_UP:
					if ((d->second_child()->is_double() && d->second_child()->is_new() ||
						 d->first_child()->is_double() && d->first_child()->is_new()))
						if (d->node()->parent_node()->is_pure_node() &&
							d->node()->has_coal_anc() && (d->next() == d->parent_contnr()))
							elig_targets->push_back(d);
					break;
				default:
					break;
			}
		}
	}

	OUT_FNCTN_BOTTOM(logger());
}



// move a node which was used to hook in a recombination
void tree_rearranger_rec::move_hook_node(Rev_Tree* tr, Rev_Coal_Nd* target, BOOL is_move_down, stringstream& ss_dot)
{
	OUT_FNCTN_HEADER(logger());

	if (is_move_down)
	{
		LOG4CXX_DEBUG(logger(), "DIR_DOWN");
		LOG4CXX_DEBUG(logger(), "target id: " << target->id());
		ss_dot << "DOWN ";
		ss_dot << target->id() << " ";

		double* t_new = new double;
		double r = tree_rearranger_sngl::get_ratio_mcmc(target->node_contnr(), true, t_new);

		if (r > gsl_rng_uniform(Random::rng()))
		{
			BOOL target_is_first = target->is_first_child();
			Rev_Coal_Nd* target_parent = target->parent_node();

			Rev_Coal_Nd* dest;
			if (target->first_child_node()->is_single())
				dest = dynamic_cast<Rev_Coal_Nd*>(target->first_child_node());
			else
				dest = dynamic_cast<Rev_Coal_Nd*>(target->second_child_node());
			if (!dest)
				throw Error("dest == 0 in move_hook_node");
			BOOL dest_is_first = dest->is_first_child();

			Rev_Nd* dest_child;
			BOOL move_to_first_child;
			if (gsl_rng_uniform(Random::rng()) > 0.5)
			{
				move_to_first_child = true;
				dest_child = dest->first_child_node();
			}
			else
			{
				move_to_first_child = false;
				dest_child = dest->second_child_node();
			}

			LOG4CXX_DEBUG(logger(), "t_new: " << *t_new);
			LOG4CXX_DEBUG(logger(), "target: " << target->id());
			LOG4CXX_DEBUG(logger(), "target_is_first: " << target_is_first);
			LOG4CXX_DEBUG(logger(), "target_parent: " << target_parent->id());
			LOG4CXX_DEBUG(logger(), "dest: " << dest->id());
			LOG4CXX_DEBUG(logger(), "dest_is_first: " << dest_is_first);
			LOG4CXX_DEBUG(logger(), "dest_child: " << dest_child->id());
			LOG4CXX_DEBUG(logger(), "move_to_first_child: " << move_to_first_child << endl);

			dest->set_coal_anc(target_parent->node_contnr(), target_is_first);
			target->set_coal_anc(dest->node_contnr(), move_to_first_child);
			dest_child->set_coal_anc(target->node_contnr(), dest_is_first);

			target->correct_after_move_hook_node(dest, true);
			target->recalc_act_sites_grps();
			target->recalc_occur_subs();

			target->change_time(*t_new);
			tree_rearranger_sngl::adjust_prev_and_next(tr, target->node_contnr(), is_move_down);
		}
	}
	else
	{
		LOG4CXX_DEBUG(logger(), "DIR_UP");
		LOG4CXX_DEBUG(logger(), "target id: " << target->id());
		ss_dot << "UP ";
		ss_dot << target->id() << " ";

		double* t_new = new double;
		double r = 0.5 * tree_rearranger_sngl::get_ratio_mcmc(target->node_contnr(), false, t_new);

		if (r > gsl_rng_uniform(Random::rng()))
		{
			Rev_Coal_Nd* dest = target->parent_node();
			Rev_Coal_Nd* dest_parent = dest->parent_node();
			BOOL dest_is_first = dest->is_first_child();
			BOOL target_is_first = target->is_first_child();

			Rev_Nd* target_child;
			BOOL target_child_is_first;
			if (target->first_child_node()->is_single() || !target->first_child_node()->is_new())
			{
				target_child = target->first_child_node();
				target_child_is_first = true;
			}
			else
			{
				target_child = target->second_child_node();
				target_child_is_first = false;
			}

			LOG4CXX_DEBUG(logger(), "t_new: " << *t_new);
			LOG4CXX_DEBUG(logger(), "target: " << target->id());
			LOG4CXX_DEBUG(logger(), "target_is_first: " << target_is_first);
			LOG4CXX_DEBUG(logger(), "target_child: " << target_child->id());
			LOG4CXX_DEBUG(logger(), "target_child_is_first: " << target_child_is_first);
			LOG4CXX_DEBUG(logger(), "dest: " << dest->id());
			LOG4CXX_DEBUG(logger(), "dest_is_first: " << dest_is_first);
			LOG4CXX_DEBUG(logger(), "dest_parent: " << dest_parent->id() << endl);

			dest->set_coal_anc(target->node_contnr(), target_child_is_first);
			target_child->set_coal_anc(dest->node_contnr(), target_is_first);
			target->set_coal_anc(dest_parent->node_contnr(), dest_is_first);

			target->correct_after_move_hook_node(dest, false);
			dest->recalc_act_sites_grps();
			dest->recalc_occur_subs();

			target->change_time(*t_new);
			tree_rearranger_sngl::adjust_prev_and_next(tr, target->node_contnr(), is_move_down);
		}
	}

	OUT_FNCTN_BOTTOM(logger());
}




// sample time of events for rearrange operation "recombination"
void tree_rearranger_rec::sample_time_of_events(Rev_Tree* tr, const set<Rev_NC*>& new_nodes, int num_iter,
												Rev_Nd* lower_bound, BOOL output_dot)
{
	OUT_FNCTN_HEADER(logger());
	const BOOL OUTPUT_DOT = output_dot;

	LOG4CXX_DEBUG(logger(), "lower_bound: " << lower_bound->id());
	stringstream ss;
	ss << "new_nodes: ";
	for (set<Rev_NC*>::const_iterator it = new_nodes.begin(); it != new_nodes.end(); it++)
		ss << (*it)->str_id() << " ";
	LOG4CXX_DEBUG(logger(), ss.str());

	const int NUM_OPS = 3;   // number of different alteration operations
	double FRACTION_OPS[NUM_OPS] = { 1, 1, 1 };   // proportion of each operation

	for (int i = 0; i < num_iter; i++)
	{
		stringstream ss_dot;
		LOG4CXX_DEBUG(logger(), "iter: " << i);
		ss_dot << i << " ";

		gsl_ran_discrete_t* grd = gsl_ran_discrete_preproc (NUM_OPS, FRACTION_OPS);
		enum_direction chosen_dir = (enum_direction)gsl_ran_discrete (Random::rng(), grd);
		gsl_ran_discrete_free(grd);
		VECTOR(Rev_Intern_NC*) v;
		tree_rearranger_sngl::get_elig_targets_single(tr, chosen_dir, &v);
		VECTOR(Rev_Intern_NC*) elig_targets_single;
		for (VECTOR(Rev_Intern_NC*)::iterator it = v.begin(); it != v.end(); it++)
			if ((*it)->is_new())
				elig_targets_single.push_back(*it);

		LOG4CXX_DEBUG(logger(), "dir: " << chosen_dir);
		stringstream ss;
		ss << "elig_targets_single: ";
		for (VECTOR(Rev_Intern_NC*)::iterator it = elig_targets_single.begin(); it != elig_targets_single.end(); ++it)
			ss << (*it)->str_id() << " ";
		LOG4CXX_DEBUG(logger(), ss.str());

		VECTOR(Rev_Coal_NC*) elig_targets_hook;
		if ((chosen_dir == DIR_UP) || (chosen_dir == DIR_DOWN))
		{
			get_elig_targets_hook(tr, chosen_dir, &elig_targets_hook);
			stringstream ss;
			ss << "elig_targets_hook: ";
			for (VECTOR(Rev_Coal_NC*)::iterator it = elig_targets_hook.begin(); it != elig_targets_hook.end(); ++it)
				ss << (*it)->str_id() << " ";
			LOG4CXX_DEBUG(logger(), ss.str());
		}

		if (elig_targets_single.size() + elig_targets_hook.size() > 0)
		{
			if (gsl_rng_uniform(Random::rng()) > (double)elig_targets_single.size() / (elig_targets_hook.size() + elig_targets_single.size()))
			{
				Rev_Coal_NC* target = elig_targets_hook[gsl_rng_uniform_int(Random::rng(), elig_targets_hook.size())];
				BOOL is_move_down = (chosen_dir == DIR_DOWN);
				move_hook_node(tr, target->node(), is_move_down, ss_dot);
			}
			else
			{
				Rev_Intern_NC* target = elig_targets_single[gsl_rng_uniform_int(Random::rng(), elig_targets_single.size())];
				tree_rearranger_sngl::move_node_contnr(tr, target, chosen_dir, ss_dot);
			}

			if (OUTPUT_DOT)
				Dot::to_dot(tr, ss_dot.str());
			if (SAFE_MODE)
				tr->check();
		}

		//Dot::to_dot(tr, "TGF");
		//tr->check();

		LOG4CXX_DEBUG(logger(), "");
	}

	OUT_FNCTN_BOTTOM(logger());
}



// generate coalescent and recombination events for rearrange operation "recombination"
void tree_rearranger_rec::generate_events_rearr_recomb(Rev_Tree* tr, Rev_Nd* start, Rev_Nd* end, Partition_Tree* part_tree,
													   const BOOL output_dot)
{
	OUT_FNCTN_HEADER(logger());
	LOG4CXX_DEBUG(logger(), "start: " << start->id());
	LOG4CXX_DEBUG(logger(), "end: " << end->id() << endl);

	const VECTOR(Part_Node_Conn) conns = part_tree->conns();
	const Part_Node_Conn chosen_conn = conns[gsl_rng_uniform_int(Random::rng(), conns.size())];

	LOG4CXX_DEBUG(logger(), "conn:");
	LOG4CXX_DEBUG(logger(), chosen_conn);

	// get crf and final coalescence nodes
	const CRF* crf = tr->crf(start->crf_id());
	map<int, Rev_Nd*> first_final_nodes;
	for (set<int>::iterator it = crf->occur_subs().begin(); it != crf->occur_subs().end(); it++)
		if (!tr->sub_only_in_crfs(*it))
				first_final_nodes[*it] = tr->get_first_final(*it);

	LOG4CXX_DEBUG(logger(), "first_final_nodes: ");
	for (map<int, Rev_Nd*>::const_iterator it = first_final_nodes.begin(); it != first_final_nodes.end(); it++)
		LOG4CXX_DEBUG(logger(), "sub: " << it->first << "  Rev_Nd id: " << it->second->id());
	LOG4CXX_DEBUG(logger(), "");

	// get indices of final coalescence nodes
	map<int, int> first_final_nodes_idx;
	int index = 0;
	Rev_Nd* n = start;
	for (Rev_Nd* n = start; n != end; n = n->next(), index++)
	{
		for (set<int>::iterator it = crf->occur_subs().begin(); it != crf->occur_subs().end(); it++)
			if (!tr->sub_only_in_crfs(*it) && (n->next()->id() == first_final_nodes[*it]->id()))
				first_final_nodes_idx[*it] = index;
	}

	// get nodes with their probabilities
	VECTOR(pair<Rev_Nd* COMMA double> ) weighted_nodes;
	for (Rev_Nd* n = start; n != end; n = n->next())
	{
		weighted_nodes.push_back(pair<Rev_Nd*, double>(n, n->next()->birth_time() - n->birth_time()));
	}

	stringstream ss;
	ss << "weighted_nodes: " << endl;
	for (VECTOR(pair<Rev_Nd* COMMA double> )::const_iterator it = weighted_nodes.begin(); it != weighted_nodes.end(); it++)
		ss << "Rev_Nd id: " << it->first->id() << "  weight: " << it->second << endl;
	ss << endl;
	ss << "first_final_nodes_idx: " << endl;
	for (map<int, int>::const_iterator it = first_final_nodes_idx.begin(); it != first_final_nodes_idx.end(); it++)
		if (!tr->sub_only_in_crfs(it->first))
			ss << "sub: " << it->first << "  Rev_Nd idx: " << it->second << endl;
	ss << endl;
	LOG4CXX_DEBUG(logger(), ss.str());

	// calculate time of events
	list<double> time_of_events;
	double P[weighted_nodes.size()];
	for (int i = 0; i < weighted_nodes.size(); i++)
		P[i] = weighted_nodes[i].second;
	int max_idx = weighted_nodes.size() - 1;
	for (VECTOR(Conn_Point)::const_reverse_iterator it = chosen_conn.conn_points().rbegin();
		 it != chosen_conn.conn_points().rend(); it++)
	{
		if (it->is_coal())
			max_idx = GSL_MIN(max_idx, first_final_nodes_idx[it->sub()]);
		LOG4CXX_DEBUG(logger(), "part_node id: " << it->id() << "  max_idx: " << max_idx);

		gsl_ran_discrete_t* grd = gsl_ran_discrete_preproc (max_idx + 1, P);
		int chosen_idx = gsl_ran_discrete (Random::rng(), grd);
		gsl_ran_discrete_free(grd);
		chosen_idx = max_idx;
		time_of_events.push_back(weighted_nodes[chosen_idx].first->birth_time() +
				                 weighted_nodes[chosen_idx].second * gsl_rng_uniform(Random::rng()));
	}
	time_of_events.sort();

	ss.str("");
	ss << time_of_events;
	LOG4CXX_DEBUG(logger(), "time_of_events: " << ss.str());

	// generate new nodes and connect them to the arg
	list<double>::iterator jt = time_of_events.begin();
	for (VECTOR(Conn_Point)::const_iterator it = chosen_conn.conn_points().begin();
	     it != chosen_conn.conn_points().end(); it++, jt++)
	{
		if (it->is_coal()) {
			LOG4CXX_DEBUG(logger(), "Coal");
			LOG4CXX_DEBUG(logger(), "target Rev_Nd: " << part_tree->node(it->sub())->id());

			VECTOR(Rev_Nd*) hook_nodes;
			tr->get_hook_in_targets(&hook_nodes, *jt, it->sub());
			Rev_Nd* target = hook_nodes[gsl_rng_uniform_int(Random::rng(), hook_nodes.size())];
			LOG4CXX_DEBUG(logger(), "hook nodes: " << hook_nodes)
			LOG4CXX_DEBUG(logger(), "target: " << target->id());
			Rev_Coal_Nd* new_node = part_tree->node(it->sub())->hook_in(target, *jt);

			LOG4CXX_DEBUG(logger(), "new Rev_Nd: " << *new_node << endl);
		}
		else
		{
			LOG4CXX_DEBUG(logger(), "Recomb");
			LOG4CXX_DEBUG(logger(), "target Rev_Nd: " << part_tree->node(it->subs())->id());

			set<int> subtypes_first = it->first_subs();
			set<int> subtypes_second = it->second_subs();
			Rev_Rec_NC* new_node_contnr = new Rev_Rec_NC(tr, *jt, part_tree->node(it->subs()), subtypes_first, subtypes_second);
			part_tree->set_node(subtypes_first, new_node_contnr->first_node());
			part_tree->set_node(subtypes_second, new_node_contnr->second_node());
			LOG4CXX_DEBUG(logger(), "new_nodes: " << new_node_contnr->first_node()->id() << " "
						            << new_node_contnr->second_node()->id() << endl);
		}
	}

	tr->recalc_act_sites_grps();
	tr->recalc_occur_subs();

	if (output_dot)
		Dot::to_dot(tr, "tree_rearranger_rec::generate_events_rearr_recomb");

	//Dot::to_dot(tr, "WMX");
	//tr->check();

	sample_time_of_events(tr, tr->changes()->new_node_contnrs(), 100, start, output_dot);

	OUT_FNCTN_BOTTOM(logger());
}



// perform rearrange operation "recombination"
void tree_rearranger_rec::recomb(Rev_Tree* tr, Rev_Nd* target, const BOOL output_dot)
{
	OUT_FNCTN_HEADER(logger());

	if (tr->changes()->rearranging_done())
		throw ERROR(logger(), "Rearranging recombination: Illegal Rev_Tree");
	if (!(target->is_rec_node() || target->is_crf_node()) && (target->has_rec_anc()))
		throw ERROR(logger(), "Rearranging recombination: Illegal Rev_Nd");

	tr->changes()->set_rearranging_done();
	tr->save_first_last();

	tr->set_nodes_old_recomb(target);
	tr->adjust_pos();
	tr->adjust_anc_suc(tr->changes()->old_node_contnrs());
	const CRF* crf = target->tree()->crf(target->crf_id());
	Partition_Tree* part_tree = crf->part_tree();
	int part_tree_size = part_tree->num_part_nodes();
	part_tree->set_root(target);

	// find most top Rev_Nd involved in rearrange operation
	Rev_Nd* end = 0;
	BOOL is_first = true;
	for (set<int>::iterator it = crf->occur_subs().begin(); it != crf->occur_subs().end(); it++)
		if (!tr->sub_only_in_crfs(*it)) {
			Rev_Nd* n = tr->get_first_final(*it);
			if (is_first || (n->birth_time() > end->birth_time()))
				end = n;
			is_first = false;
		}

	Dot::to_dot(part_tree);
	generate_events_rearr_recomb(tr, target, end, part_tree, output_dot);

	tr->calc_num_elig_nd_st(tr->first_node_contnr(), tr->last_node_contnr());
	tr->generate_unknown();
	tr->recalc_act_sites_grps();
	tr->recalc_occur_subs();
	tr->generate_final();

	OUT_FNCTN_BOTTOM(logger());
}


