#include "standard.h"

#include "changes.h"
#include "dot.h"
#include "files.h"
#include "nc.h"
#include "random.h"
#include "tr_rearr_scl.h"
#include "tree.h"

LoggerPtr tree_rearranger_scl::s_logger(Logger::getLogger("tree.tr_rearr_scl"));



// perform rearrange operation "scale"
void tree_rearranger_scl::scale(Rev_Tree* tr)
{
	OUT_FNCTN_HEADER(logger());

	const double MAX_DOWN = 0.05;
	
	if (tr->changes()->rearranging_done())
		throw ERROR(logger(), "Rearranging coalescent: Illegal Rev_Tree");
	tr->changes()->set_rearranging_done();
	tr->save_first_last();

	
	double scale = (1 - MAX_DOWN) + gsl_rng_uniform(Random::rng()) * (1 / (1 - MAX_DOWN) - (1 - MAX_DOWN));
	for (Rev_NC* c = tr->first_node_contnr(); c; c = c->next())
		c->change_time(scale * c->birth_time());

	OUT_FNCTN_HEADER(logger());
}
