#ifndef REV_TREE_H
#define REV_TREE_H

#include "standard.h"

#include "base_tree_def.h"


enum enum_direction { DIR_STAY, DIR_UP, DIR_DOWN };

ostream& operator<<(ostream& s, const enum_direction& dir);

using namespace log4cxx;
using namespace std;

class Rev_Nd;
class Rev_Coal_Nd;
class Rev_Rec_Nd;
class Rev_NC;
class Sim_Nd;
class Sim_NC;
class CRF;
class Partition_Tree;
class Tip_Seqs;
class Constants;
class Changes;
class Partition_Aliasing;




class Rev_Tree : public Base_Tree<Rev_Nd, Rev_Coal_Nd, Rev_Rec_Nd, Rev_NC>
{
	friend class CRF;
	friend class Rev_Nd;
	friend class Rev_NC;

	friend Rev_Tree* load_tree();
private:
	Rev_NC* m_old_first_node_contnr;
	Rev_NC* m_old_last_node_contnr;
	int m_num_subtypes;
	int m_num_crfs;
	Changes* m_changes;
	VECTOR(CRF*) m_crfs;
	Partition_Aliasing* m_part_alias;
	VECTOR( set<int> ) m_crfs_containing_subtype;
	set<int> m_subs_only_in_crfs;
	BOOL m_all_tips_added;
	BOOL m_all_tip_laps_calced;
	BOOL m_part_alias_calced;
	BOOL m_acid_prob_calced;
	static LoggerPtr s_logger;

	Rev_Tree& operator=(const Rev_Tree& tr) { throw Error("Trying to call assignment operator of Rev_Tree"); }   // no assignment possible

	static LoggerPtr& logger() { return s_logger; }
	void check_num_elig_sites_nodes();
	void check_node_contnr() const;
	void log_prob_acids_impl();

	void init();
public:
	Rev_Tree(const int num_sites, const int num_mutations_rates);
	Rev_Tree(const Rev_Tree& tr);
	~Rev_Tree();

	using Base_Tree<Rev_Nd, Rev_Coal_Nd, Rev_Rec_Nd, Rev_NC>::calc_num_elig_nd_st;

	Changes* changes() { return m_changes; }   // private
	BOOL acid_prob_calced() const { return m_acid_prob_calced; }
	BOOL all_tips_added() const { return m_all_tips_added; }
	BOOL all_tip_laps_calced() const { return m_all_tip_laps_calced; }
	BOOL part_alias_calced() const { return m_part_alias_calced; }

	Rev_Nd* old_first_node() const;
	Rev_Nd* old_last_node() const;
	Rev_NC* old_first_node_contnr() const { return m_old_first_node_contnr; }
	Rev_NC* old_last_node_contnr() const { return m_old_last_node_contnr; }
	Rev_Nd* final_node(const int s);

	int num_subtypes() const { return m_num_subtypes; }
	int num_crfs() const { return m_num_crfs; }
	CRF* crf(const int i) const { return m_crfs.at(i); }   // const CRF*
	const set<int>* crfs_containing_subtype(const int i) { return &m_crfs_containing_subtype.at(i); }
	const Partition_Aliasing* part_alias() const { return m_part_alias; }
	BOOL sub_only_in_crfs(const int i) const { return m_subs_only_in_crfs.find(i) != m_subs_only_in_crfs.end(); }

	void accept_changes();
	void reverse_changes();

	void generate_final();
	void generate_unknown();
	void get_recomb_nodes_rearr(VECTOR(Rev_Nd*) &recomb_nodes, Rev_Nd* n);
	void calc_num_subtypes_crfs();
	BOOL check_pure_final();
	void set_to_all_tips_added();
	void calc_tip_laps();
	void calc_part_alias();
	void recalc_act_sites_grps();
	void recalc_occur_subs();
	void set_old_first_last_to_zero();
	void save_first_last();
	void adjust_pos();
	Rev_Nd* get_first_final(const int i) const;
	void get_hook_in_targets(VECTOR(Rev_Nd*)* hook_nodes, const double time, const int sub) const;
	void set_nodes_old_recomb(Rev_Nd* n);
	void adjust_anc_suc(const set<Rev_NC*>& old_nodes);

	double log_prob_acids();
	VECTOR(double) log_prob_acids_pos();
	virtual double log_prior_prob();
	double log_prob() { return log_prob_acids() + log_prior_prob(); }

	void save();

	void check() const;
};

void to_dot(Rev_Nd* first, Rev_Nd* last, Rev_Nd* third_child);

Rev_Tree* load_tree();
ostream& operator<<(ostream& s, const Rev_Tree& tr);

#endif

