/*********************************************************************************
 filename	: alignment.hh
 licence        : GNU Licence, see file LICENCE

 author		: Anne-Kathrin Schultz, anne@gobics.de
 date		: Jan. 2004

content: classes Alignment, Profile

  |   date    |    author            |  changes
 -|-----------|----------------------|-------------------------------
  | April 06  | Anne-Kathrin Schultz | reading in LINUX AND windows alignments 

**********************************************************************************/
#ifndef _ALIGNMENT_HH         
#define _ALIGNMENT_HH

#include "const_def.hh"
#include <iostream>
#include <fstream>
#include <vector>

using namespace std;

class Profile;

/************************************************************************************************************************************************************************************************
description:

Alignment:
----------
The class Alignment reads in the given alignment and determines the number of subtypes, rows and columns in the alignment:

 prof:
 anz_Subtypes:          number of subtypes read in 
 anzSequenzen:          number of sequences read in
 anz_Spalten:           number of columns of the alignment
 anz_Residues:          number of residues in the alignment (without gaps!)
 first_ConsensusColumn: first shared consensus column in all subtypes, i.e. the maximum of the first consensus columns
 last_ConsensusColumn:  last shared consensus column in all subtypes, i.e. the minimum of the last consensus columns
************************************************************************************************************************************************************************************************/

/************************************************************************************************************************************************************************************************
Profile:
--------
The class Profile determines the profile of the alignment and of the sub-alignments/subtypes respectively,
i.e. in each column of the alignment/ a subtype the number of respective residues is determined
**************************************************************************************************************************************************************************************************/

/*****************************************************\
 *                  class Alignment                  *
\*****************************************************/

class Alignment {
public:                                      
  Alignment(ifstream& input);                                           // default-construktor  
  Alignment(ifstream& input, int subtypes, int sequenzen, int spalten); // construktor 
  ~Alignment();                                                         // destruktor
  void eval_Alignment(ifstream& input);                                 // Evaluation of the sub-alignments: num columns&rows, reads in the sub-alignments, generation of the profile (?)
  Profile *eval_Subalignment(int noOfSubtype);
  int  anzahlZeilenUndSpalten(ifstream& eingabe);                       // num rows&columns of the file read in
                                                                        // returns the max. number of columns
  void speicherBereitstellen();                                         // memory allocation for alignment[][]
  void dateiEinlesen(ifstream& eingabe, int max_anz_Spalten);           // reading in the alignment file
  void determineConsensusColumns();                                     // determination of the first and last consensus columns

  int  getAnzSubtypes(){return anz_Subtypes;};  
  int  getAnzSpalten(){return anz_Spalten;};
  int  getAnzSequenzen(){return anz_Sequenzen;};
  int  getAnzResidues(){return anz_Residues;};
  int  getFirstConsensus(){return first_ConsensusColumn;};
  int  getLastConsensus(){return last_ConsensusColumn;};
  Profile *getProfil(char** &alignment, int numSequenzen, int numSpalten);
  char **alignment;                                                     // stores the alignment read in
  vector<int>    numSeqsOfSubtypes;
  vector<string> names;
  
private:      
  Profile *prof;
  int      anz_Subtypes;
  int      anz_Sequenzen;                           // num sequences in the alignment
  int      anz_Spalten;                             // length of the sequences
  int      anz_Residues;                            // num residues in the alignment (without gaps!)
  int      first_ConsensusColumn;                   // first shared consensus column off all subtypes, i.e. maximum of the first consensus columns 
  int      last_ConsensusColumn;                    // last shared consensus column of all subtypes, i.e. minimum of the last consensus columns
};


/*****************************************************\
 *                     class Profile                 *
\*****************************************************/

class Profile {
public:
  Profile(char** &alignment, int numSequences, int numColumns, int first_ConsensusColumn, int last_ConsensusColumn);
  ~Profile();
  void  profilErstellen(char** &alignment);
  void  eval_Profil();
  int **getProfil(){return profil;};
  int   getNumSpalten(){return numSpalten;};
  int  *getKonsensusColumns(){return konsensusColumns;};                // decides: alignment column = consensus/insertion column ?
  
private:
  int **profil;
  int  *konsensusColumns;                           // stores: alignment column = consensus/insertion column ? 
  int   numSequenzen;
  int   numSpalten;
  int   first_Consensus;                            // first shared consensus column of all subtypes
  int   last_Consensus;                             // last shared consensus column all subtypes
};

#endif   // _ALIGNMENT_HH 
