/**************************************************************
 file   	: CONST_DEF.HH
 licence        : GNU Licence, see file LICENCE

 author		: Anne-Kathrin Schultz, anne@gobics.de
 date		: 12.03.04


  |   date    |    author            |  changes
 -|-----------|----------------------|-------------------------------
  | 31.05.05  | Anne-Kathrin Schultz | general_Alphabet added

***************************************************************/

/************************************************************************************************************************************************************************************************
description:
The class Const_Def declares all constants used in the jumping profile HMM:

HMM_StateType    lists all types of states in the jpHMM;
Alphabetic_Mode  defines the possible alphabets of the considered sequences, consisting of nucleotides (dna) or amino acids (protein);
Profile_Mode     defines the possibility to generate, in addition to the profile HMMs of the subtypes, a profile HMM of the whole alignment (common) or not (nocommon).
Consensus_Mode   defines the possibility that the first and last consensus column of each subtype must be the same (commonConsensus) or not (noCommonConsensus).
mytype           defines the data type used in the jpHMM for the computation of all probabilities etc., (here: float);
**************************************************************************************************************************************************************************************************/



#ifndef _CONST_DEF_HH
#define _CONST_DEF_HH

#include <fstream>


#define NUM_AA 20          // number of amino acids
#define NUM_NUCL 4         // number of nucleotides
#define NUM_GAPS 2         // gaps: '.' and '*'
#define NUM_ACT_STATES 3   // 'active' states of an HMM: MATCH, INSERT, DELETE
#define INF FLT_MAX        // if mytype==double => #define INF DBL_MAX 


using namespace std;

enum HMM_StateType {Begin = -3, N_Delete, N_Insert, Match, Insert, Delete, C_Insert, C_Delete, End}; // state types
enum Alphabetic_Mode { dna, protein};
enum Profile_Mode {common, nocommon};
enum Consensus_Mode {commonConsensus, noCommonConsensus};
enum Virus_Type {HIV, HCV};

extern long appl_id;

// in maintest.cc:
extern Alphabetic_Mode alphabet;  
extern Profile_Mode    profile_mode;
extern Consensus_Mode  consensus;
extern float           consensus_Percentage;
extern int             consensus_Threshold;

typedef float mytype;

const char nucleic[NUM_NUCL + NUM_GAPS + 1] = {'A','C','G','T','X','.','-'};
const char amino[NUM_AA + NUM_GAPS + 1]     = {'A','C','D','E','F','G','H','I','K','L','M','N','P','Q','R','S','T','V','W','Y','X','.','-'}; 
const char general_Alphabet[26]             = {'A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','W','X','Y','Z'}; 

class Const_Def{
public:
  Const_Def();
  ~Const_Def(){};
  
  static int indexOfAlphabet[256];
  static void initialize();
  static void fillAlphabet();
  
private:

};

#endif        // _CONST_DEF_HH
