/**************************************************************
 filename	: HMM.HH
 licence        : GNU Licence, see file LICENCE

 author		: Anne-Kathrin Schultz, anne@gobics.de
 date		: 03/04 - 08/05

content: classes HMM, ActiveStates

  |   date    |    author            |  changes
 -|-----------|----------------------|-------------------------------
  | April 06  | Anne-Kathrin Schultz | sequences containing gaps

***************************************************************/

/************************************************************************************************************************************************************************************************
description:

HMM:
----
The class HMM generates the jpHMM:

funktions:
 provideMemory()              memory allocation for the local array allStates which contains all(!) states of the jpHMM;
 fill_HMM_States()            transfers all states from allStates into the list HMM_States;
 addTransitions()             determines the transitions in the jpHMM:
                              in addition to the transitions determined for each profile HMM seperately in the class ProfileHMM transitions between the different profile HMMs of the subtypes 
                              and the transitions to the special states BEGIN, N_DELETE, N_INSERT, C_INSERT, C_DELETE and END are determined; 
                              the two lists containing the predecessors and successors of a state determines in class ProfileHMM are extended accordingly;
 createJumpingProfileHMM()    generates the jpHMM by calling the class ProfileHMM which generates a profile HMM for each subtype;
 evaluationOfHMM()            reads in the query sequence, contains the forward and Viterbi algorithm, including the BEAM algortithm, and computes the Viterbi path;  
 deleteActivationRecursion()  activates for each position of the query sequence the successional DELETE states of an active MATCH or DELETE state which satisfies the BEAM condition (recursively);
 viterbiAndForward()          computes the Viterbi and the forward variable for each position of the query sequence and each state of the jphmm;

variables:
 HMM_States                   list of all states of the jpHMM; HMM_States = (BEGIN, N_DELETE, N_INSERT, ..., C_INSERT, C_DELETE, END)
 querySeq                     contains the query sequence read in;
************************************************************************************************************************************************************************************************/

/************************************************************************************************************************************************************************************************
ActiveStates:
-------------
The class ActiveStates assigns to each active state three values (it is called in the class HMM in the forward/Viterbi algorithm):

variables:
 posState                     position of the state in the vector HMM_States;
 posBestPred                  position of the best predecessor in the vector HMM_States, determined in the Viterbi algorithm;
 forwardValue                 forward variable computed in the forward algorithm;

These variables are not private, in order that they can be called in class HMM;
**************************************************************************************************************************************************************************************************/



#ifndef _HMM_HH
#define _HMM_HH

#include "const_def.hh"
#include "alignment.hh"
#include "profileHMM.hh"
#include <vector>
#include <list>
#include <iostream>

using namespace std;

extern float BEAM_WIDTH;                           // ->maintest.cc
extern mytype epsilon;                             // ->maintest.cc
extern mytype d_init;
extern mytype d_ext;

/******************************************************\
 *                   class HMM                        *
\******************************************************/

class HMM {
public:
  HMM(Alignment* al, int numSubtypes, ifstream& emissionPriors, ifstream& querySequence);
  ~HMM();

  void provideMemory(State**** &allStates, int numSubtypes, int numSpalten);     
  void fill_HMM_States(State**** &allStates);                                  
  void addTransitions(State* akt_Zustand, State**** &allStates, int numSubtypes, int numSpalten, int &num_JumpsToProfile, int &num_JumpsToSubtypes);  
  void createJumpingProfileHMM(Alignment* al, int numSubtypes, ifstream& emissionPriors, ifstream& querySequence); 
  void evaluationOfHMM(Alignment *al, int numSubtypes, ifstream& querySequence);
  void deleteActivationRecursion(State* akt_Zustand, unsigned int queryPos, mytype viterbi_Max, int** &activatedStates, mytype** &viterbi, mytype** &forward, int** &posOfBestPred, list<int> &possibleActiveStates, mytype beam_width); 
  void viterbiAndForward(int posOfState, unsigned int queryPos, int** &activatedStates, int** &posOfBestPred, mytype** &viterbi, mytype** &forward); 
 
private:
  vector<State*> HMM_States;                                                             
  vector<char>   querySeq;                                                               
};

/******************************************************\
 *               class ActiveStates                   *
\******************************************************/

class ActiveStates {
public:
  ActiveStates(int pos, int bestPred, mytype forward_Value);
  ~ActiveStates(){};

  int posState;
  int posBestPred;
  mytype forwardValue;
};

#endif        // _HMM_HH
