/**************************************************************
 filename	: PROFILEHMM.CC 
 licence        : GNU Licence, see file LICENCE
 description    : -> profileHMM.hh
 author		: Anne-Kathrin Schultz, anne@gobics.de
 date		: 03/04 - 08/05

Content:
 classes: ProfileHMM, 
          Transition,
          State

  |   date    |    author            |  changes
 -|-----------|----------------------|-------------------------------------------
  | 06.10.06  | Anne-Kathrin Schultz | now also accepts only 1 subtype in the alignment
 
***************************************************************/

#include "const_def.hh"
#include "profileHMM.hh"
#include "alignment.hh"
#include "hmm.hh"
#include <math.h>
#include <float.h>
#include <vector>
#include <iostream>
#include <string>
#include <cstdlib>


using namespace std;


/******************************************************\

 *                   class profileHMM                 *

\******************************************************/

ProfileHMM::ProfileHMM() {  
}

ProfileHMM::~ProfileHMM() {
 
}

//--------------------------------------------------------------------------------------------------------------------------------------------------------------------
// provideMemory(State***&, int):  
//
//    memory allocation for allStates[][];
//
//--------------------------------------------------------------------------------------------------------------------------------------------------------------------
void ProfileHMM::provideMemory(State*** &allStates, 
			       int       numSpalten) {
  
  anz_Spalten = numSpalten;
  allStates = new State**[numSpalten]; 
  
  for (int i=0; i<numSpalten; i++) {
    allStates[i] = new State*[NUM_ACT_STATES];
    // Rest siehe in fillStates(), da Unterschied zw. "leer" und Match 
  }
}

//--------------------------------------------------------------------------------------------------------------------------------------------------------------------
// fillStates(Alignment, State***&, int**&, int*&, int, int, int, ifstream&):  
//
//--------------------------------------------------------------------------------------------------------------------------------------------------------------------
void ProfileHMM::fillStates(Alignment *al,
			    State***  &allStates,
			    int**     &profil,
			    int*      &gaps,
			    int        numResidues,
			    int        numSubtype,
			    int        anz_SubProfiles,
			    ifstream&  emissionPriors) {

  vector<int> numSequences        = al->numSeqsOfSubtypes;   // vector contains the number of sequences for each subtype
  int         numSpalten          = al->getAnzSpalten();
 
  int NUM;
  if (alphabet == protein)
    NUM = NUM_AA;
  else 
    NUM = NUM_NUCL;
  
  // frequencies of characters not included in the considered alphabet are added up (ueber gesamtes Alignment) and stored in the variable num_N 
  int         num_X      = 1;         // for DNA and protein sequences   =1, =>wie Pseudocount gewertet, denn Wkt fuer N und X muss gegeben sein!
  for (int i=0; i<numSpalten; i++) {
    num_X     += profil[i][NUM];    // profil[i][NUM] = column in the profile which contains the number of characters not member of the considered alphabet (stored as 'X')      
  }
  mytype         prob_X              = (mytype)num_X/((mytype)numResidues);
  int            last_active_State   = 0;
  int            first_active_State  = 0;
  int            count_active_States = 0;
  mytype        *aa_Frequencies      = new mytype[NUM];
  vector<mytype> emProbs;
  int            luecken_Laenge      = 0;
  int            succ_Spalte         = 0;

  for (int j=0; j<numSpalten; j++) {      // Loop over the columns of the corresponding subtype
    for (int typ = (int)Match; typ<=(int)Delete; typ++) {
      if (gaps[j] == 1) { // MATCH-Spalte
	allStates[j][typ] = new State((HMM_StateType)typ, numSubtype, j);
	if (typ == (int)Match) {
	  // MATCH emission probabilities:
	  for (int k=0; k<NUM; k++)
	    aa_Frequencies[k] = profil[j][k];
	}
	else if (typ == (int)Insert) {
	  // INSERT emission probabilities
	  // Initialization:
	  for (int i=0; i<NUM; i++)
	    aa_Frequencies[i] =0;
	  luecken_Laenge=0;
	  succ_Spalte = j+1;
	  // Determination of the length of the gap (= number of subsequent non-consensus columns)
	  while ( gaps[succ_Spalte] != 1 && succ_Spalte < numSpalten){
	    luecken_Laenge++;
	    if ((j + luecken_Laenge) < numSpalten) {
	      // for the number of insertions in the gap the profile of the corresponding (gap)column must be considered!
	      for (int i=0; i<NUM; i++)
		aa_Frequencies[i] += profil[succ_Spalte][i];
	    }
	    succ_Spalte++;
	  }
	}
	if ((typ == (int)Match) || (typ == (int)Insert)) {
	  emProbs     = State::dirichlet_EmissionPriors((HMM_StateType)typ, aa_Frequencies, emissionPriors); 
	  for (int i=0; i<NUM; i++) 
	    emProbs[i] = ((mytype)1 - prob_X)*emProbs[i];
	  emProbs.push_back(prob_X);
	  allStates[j][typ]->log_emProb = State::log_Probs(emProbs);
	}
	count_active_States++;
	last_active_State = j;
	if (count_active_States == 1)
	  first_active_State = j;
      }
      else { // INSERT, thus no HMM node
	allStates[j][typ] = NULL;
      }
    }
  }  
  // Delete last INSERTs since common C_INSERT exists
  allStates[last_active_State][ (int) Insert] = NULL;
}

//--------------------------------------------------------------------------------------------------------------------------------------------------------------------
// addStates(State*, State***&, int, vector<mytype>): 
//
//    determines for each state the successor states
//
//--------------------------------------------------------------------------------------------------------------------------------------------------------------------
void ProfileHMM::addStates(State         *akt_Zustand, 
			   State***      &allStates,
			   int            numSpalten,
			   vector<mytype> innerTransProbs) {
  
  
  // innerTransProbs = {p_MM, p_MI, p_MD, p_IM, p_II, p_DM, p_DD};
  Transition *Uebergang; 
  int         spalte          = akt_Zustand->column;
  int         gap_Laenge      = 0;

  // successional INSERT as potential successor
  if ( (akt_Zustand->stateType == Match) && (allStates[spalte][(int)Insert] != NULL)) { 
    Uebergang = new Transition(akt_Zustand, allStates[spalte][(int)Insert]);
    Uebergang->transProb = innerTransProbs[1];
    akt_Zustand->succ.push_back( Uebergang );
    allStates[spalte][(int)Insert]->pred.push_back( Uebergang);
  }
  
  if (akt_Zustand->stateType == Insert) { // Transition to itself
    Uebergang = new Transition(akt_Zustand, akt_Zustand);
    Uebergang->transProb = innerTransProbs[4];
    akt_Zustand->succ.push_back( Uebergang);
    akt_Zustand->pred.push_back( Uebergang);
  }
  
  if (spalte != numSpalten-1) {
    // Determination of the length of this row:
    // Attention: DELETE and INSERT can be NULL but MATCH not (last node!)
    // therefore the length of the following gap in this row has to be determined (using the MATCH state)
    
    while (((spalte+gap_Laenge) < numSpalten-2) && (allStates[spalte+gap_Laenge+1][(int)Match] == NULL)) 
      gap_Laenge++;
    if (allStates[spalte+gap_Laenge+1][(int)Match] != NULL) {
      // for each state (M, I, D) successional MATCH in same row/subtype:
      Uebergang = new Transition(akt_Zustand, allStates[spalte+gap_Laenge+1][(int)Match]);
      if (akt_Zustand->stateType == Match)
	Uebergang->transProb = innerTransProbs[0];
      else if (akt_Zustand->stateType == Insert)
	Uebergang->transProb = innerTransProbs[3];
      else
	Uebergang->transProb = innerTransProbs[5];
      akt_Zustand->succ.push_back( Uebergang);
      allStates[spalte+gap_Laenge+1][(int)Match]->pred.push_back( Uebergang);
    }
    if (allStates[spalte+gap_Laenge+1][(int)Delete] != NULL) {
      if ((akt_Zustand->stateType == Delete) || (akt_Zustand->stateType == Match)) {
	// if DELETE or MATCH successional DELETE in same row/subtype:
	Uebergang = new Transition(akt_Zustand, allStates[spalte+gap_Laenge+1][(int)Delete]);
	if (akt_Zustand->stateType == Match)
	  Uebergang->transProb = innerTransProbs[2];
	else
	  Uebergang->transProb = innerTransProbs[6];     // p_DD
	akt_Zustand->succ.push_back( Uebergang);
	allStates[spalte+gap_Laenge+1][(int)Delete]->pred.push_back( Uebergang);
      }
    }
  }
}

//--------------------------------------------------------------------------------------------------------------------------------------------------------------------
// createProfileHMM(Alignment*, int, int, int, ifstream&): 
//
//    provides for each state a list of all potential successors and corresponding transition and emission probabilities
//--------------------------------------------------------------------------------------------------------------------------------------------------------------------
vector<State*> ProfileHMM::createProfileHMM(Alignment *al, 
					    int        numSubtypes,
					    int        noOfSubtype,
					    int        anz_SubProfiles,
					    ifstream&  emissionPriors) {

  // anz_SubProfile = number of considered profiles, can be numSubtypes+1 if the profile of the whole alignment is also considered (profile_mode == common)

  vector<State*> ProfileHMM_States;
  int            numSpalten         = al->getAnzSpalten();
  State***       allStates;
  Profile       *p;
  int          **profil;
  int           *gaps;
  int            NUM;
  int            numResidues        = al->getAnzResidues();
  State         *akt_Zustand;
  vector<int>    innerTransFrequencies(7);
  vector<mytype> innerTransProbs(7);
  if (alphabet == protein)
    NUM = NUM_AA;
  else 
    NUM = NUM_NUCL;
  
  if (noOfSubtype == numSubtypes) {
    p      = al->getProfil(al->alignment, al->getAnzSequenzen(), numSpalten);
    profil = p->getProfil();
    gaps   = p->getKonsensusColumns();
  }
  else {
    p      = al->eval_Subalignment(noOfSubtype);
    profil = p->getProfil();
    gaps   = p->getKonsensusColumns();
  }

  provideMemory(allStates, numSpalten);
  fillStates(al, allStates, profil, gaps, numResidues, noOfSubtype, anz_SubProfiles, emissionPriors);
  
  innerTransFrequencies[0] = 1;
  for (int i=1; i<7; i++)
    innerTransFrequencies[i] = 0; 
  innerTransProbs = Transition::inner_Transition_Probs(innerTransFrequencies);
  // noOfSubtype entspricht im Fall der ZeilenHMMs der aktuellen Zeile!!
  // im Fall der ZeilenHMMs wird nur fuer das gesamte ProfilHMM diese innerTransProbs berechnet, 
  // die einzelnen Zeilen erhalten als Uebergangshaeufigkeiten (1,0,0,0,0,0,0)
  // wird hier trotzdem nochmal extra berechnet  => Fallunterscheidung machen
  for (int spalte=0; spalte<numSpalten; spalte++) {
    if (allStates[spalte][(int) Match] != NULL) {
      innerTransFrequencies = Transition::profileHMM_TransitionFrequencies(al, noOfSubtype, spalte, numSubtypes, gaps, profil);
      innerTransProbs       = Transition::inner_Transition_Probs(innerTransFrequencies);
    }
    for (int typ=(int)Match; typ<=(int)Delete; typ++) {
      if (allStates[spalte][typ] != NULL) {
	akt_Zustand = allStates[spalte][typ];
	ProfileHMM_States.push_back(akt_Zustand);
	addStates(akt_Zustand, allStates, numSpalten, innerTransProbs);
      }
    }
  }
  return ProfileHMM_States;
}

/******************************************************\

 *                 class Transition                   *

\******************************************************/

Transition::Transition() {
}

Transition::Transition(State *woher, 
		       State  *wohin) {
  whereFrom = woher;
  whereTo   = wohin;
}

Transition::~Transition() {
}

//--------------------------------------------------------------------------------------------------------------------------------------------------------------------
// profileHMM_TransitionFrequencies(Alignment*, int, int, int, int*&, int**&):
//
//    counts the transition frequencies within the considered profile HMM/subtype
//    on the basis of the aligned sequences of this subtype (see Wistrand et al.)
//--------------------------------------------------------------------------------------------------------------------------------------------------------------------
vector<int> Transition::profileHMM_TransitionFrequencies(Alignment *al, 
							 int        akt_Zeile,
							 int        akt_Spalte,
							 int        numSubtypes,
							 int*      &konsensus_Spalte,
							 int**     &profil) {

  // fuer jeden Subtype muessen vollstaendige Luecken im Subalignment "gestrichen", d.h. nicht beachtet werden!! 
  // (passiert automatisch, da hier keine Inserts gefunden werden und Uebergaenge M->D, D->M, M->M, D->D nur fuer diese und die naechste Konsensusspalte betrachtet werden
  // d.h. das in der Luecke spielt fuer diese Uebergaenge keine Rolle) 
  // koennen dafuer Anzahl der Luecken im Profil des Subalignments zaehlen: 

  // brauchen noch Position in Alignment, also welcher Subtype vorliegt und wieviele Zeilen er umfasst!
  vector<int> transFrequencies(7);         // n = (n_MM, n_MI, n_MD, n_IM, n_II, n_DM, n_DD)
  int         numSpalten              = al->getAnzSpalten();
  int         numSequences            = 0;              // number of considered rows/sequences (of one subtype or, if the general profile HMM of the MSA is needed, the number of sequences in the MSA)
  int         start                   = 0;              // first considered row/sequence in the alignment (= first sequence belonging to the considered subtype)
  vector<int> numSequencesOfSubtype   = al->numSeqsOfSubtypes;
  int  succ_Spalte      = akt_Spalte+1;
  int  luecken_Laenge   = 0;
  int  count_MM_trans   = 0;
  int  count_MI_trans   = 0;
  int  count_MD_trans   = 0;
  int  count_IM_trans   = 0;
  int  count_II_trans   = 0; 
  int  count_DM_trans   = 0;
  int  count_DD_trans   = 0;
  int  spalte;
  int  count_inserts_proZeile;
  int  NUM;
  if (alphabet == protein)
    NUM = NUM_AA;
  else 
    NUM = NUM_NUCL;
  
  if (akt_Zeile == numSubtypes){
    numSequences          = al->getAnzSequenzen();
    start                 = 0;
  }
  else {
    numSequences = numSequencesOfSubtype[akt_Zeile];
    for (int i=0; i<akt_Zeile; i++)
      start += numSequencesOfSubtype[i];
  }
  
  // Determination of the length of the gap (number of subsequent non-consensus columns)
  while ( (konsensus_Spalte[succ_Spalte] != 1) && (succ_Spalte < numSpalten+1)){  // END in column numSpalten+1
    luecken_Laenge++;
    succ_Spalte++;
  }
  if ((akt_Spalte + luecken_Laenge) < numSpalten+1) {
    // akt_Spalte is the considered consensus column -> we examine the subsequent non-consenus columns, beginning at akt_Spalte+1 
    // (=>  spalte=1) and counting up to lueckenlaenge
    for (int zeile=0; zeile<numSequences; zeile++){
      spalte                 = 1;
      count_inserts_proZeile = 0;
      
      // Counting the INSERTs in this gap
      while((spalte!=luecken_Laenge+1) && (akt_Spalte+spalte < numSpalten)) {
	if ((al->alignment[start+zeile][akt_Spalte+spalte] != '-') && (al->alignment[start+zeile][akt_Spalte+spalte] != '*')) {   // or another gap character
	    count_II_trans++;
	    count_inserts_proZeile++;
	}
	spalte++;
      }
      if (count_inserts_proZeile != 0) { // at least 1 INSERT in this gap
	count_II_trans--;                       // if at least one INSERT is found in this row => subtract 1 for initiation of gap
	if ((spalte == -1) || (al->alignment[start+zeile][akt_Spalte] != '-') && (al->alignment[start+zeile][akt_Spalte] != '*')) 
	  count_MI_trans++;                       //  i.e. MATCH->INSERT in this row or BEGIN->INSERT (column = -1)
	if ((akt_Spalte+spalte == numSpalten) || ((akt_Spalte+spalte < numSpalten) && ((al->alignment[start+zeile][akt_Spalte+spalte] != '-') && (al->alignment[start+zeile][akt_Spalte+spalte] != '*')))) 
	  count_IM_trans++;
      }
      else {  // no INSERT found in this row/sequence of the alignment
	if ((spalte != -1) && (al->alignment[start+zeile][akt_Spalte] == '-') || (al->alignment[start+zeile][akt_Spalte] == '*')) {
	  // DELETE in akt_Spalte 
	  if ((akt_Spalte+spalte == numSpalten) || ((akt_Spalte+spalte < numSpalten) && ((al->alignment[start+zeile][akt_Spalte+luecken_Laenge+1] != '-') && (al->alignment[start+zeile][akt_Spalte+luecken_Laenge+1] != '*')))) 
	    count_DM_trans++; 
	  else 
	    count_DD_trans++;
	}
	else { // BEGIN or MATCH in current consensus column
	  if ((akt_Spalte+spalte == numSpalten) || ((akt_Spalte+spalte < numSpalten) && ((al->alignment[start+zeile][akt_Spalte+luecken_Laenge+1] != '-') && (al->alignment[start+zeile][akt_Spalte+luecken_Laenge+1] != '*')))) 
	    count_MM_trans++;
	  else 
	    count_MD_trans++;
	}
      }
    }
    transFrequencies[0] = count_MM_trans;                      // n_MM
    transFrequencies[1] = count_MI_trans;                      // n_MI 
    transFrequencies[2] = count_MD_trans;                      // n_MD       =0, if other residues in this gap 
    transFrequencies[3] = count_IM_trans;                      // n_IM  
    transFrequencies[4] = count_II_trans;                      // n_II
    transFrequencies[5] = count_DM_trans;                      // n_DM     
    transFrequencies[6] = count_DD_trans;                      // n_DD     
  }

  //  AENDERN: andererseits: ob man es nun berechnet oder nicht, ist auch egal, selbst bei TransFreq = 0 erhaelt dieser Uebergang eine Wahrscheinlichkeit!, 
  // Match-Wkt aendert sich natuerlich, da sich |n| aendert. Aber wesentlich?
  // t_MM aendert sich nicht, wenn sich t_DM aendert, denn t_DM+t_DD=1. Die Wkten t_Mx und t_Dx werden mit unabhaengigen Dichten berechnet!!
  // Profile:
  //if ((allStates[akt_Spalte][(int) Delete] == NULL) && (allStates[akt_Spalte][(int) Insert] != NULL)) {
  //transFrequencies[5] = 0;                    // n_DM
  // transFrequencies[6] = 0;                    // n_DD  
  //}
  return transFrequencies;
}

//--------------------------------------------------------------------------------------------------------------------------------------------------------------------
// inner_TransitionProbs(vector<int>):
//
//    computes the transition probabilities within a profile HMM (=subtype or general profile HMM of the alignment)
//
//     TRANSITION PRIORS (see Wistrand et al.)
//--------------------------------------------------------------------------------------------------------------------------------------------------------------------
vector<mytype> Transition::inner_Transition_Probs(vector<int> transition_frequencies ) {

  // transition_frequencies = {n_MM, n_MI, n_MD, n_IM, n_II, n_DM, n_DD};
  vector<mytype> transProbs;
  const mytype   trans_alphas[7] =  {0.794, 0.095, 0.005, 0.333, 0.667, 0.278, 0.222};
  // Summe der Alphas vielleicht schon vorher ausrechnen, oder mit in Tabelle!
  int            sum_trans_freqs = 0;
  mytype         sum_alphas      = 0;
  
  // MATCH:
  for (unsigned int u=0; u<3; u++) {
    sum_trans_freqs += transition_frequencies[u];
    sum_alphas      += trans_alphas[u];
  }
  for (unsigned int v=0; v<3; v++) 
    transProbs.push_back(((mytype)transition_frequencies[v] + trans_alphas[v])/((mytype)sum_trans_freqs + sum_alphas));
   
  // INSERT:
  sum_trans_freqs = 0;
  sum_alphas      = 0;
  for (unsigned int u=3; u<5; u++) {
    sum_trans_freqs += transition_frequencies[u];
    sum_alphas      += trans_alphas[u];
  }
  for (unsigned int v=3; v<5; v++) 
    transProbs.push_back(((mytype)transition_frequencies[v] + trans_alphas[v])/((mytype)sum_trans_freqs + sum_alphas));
   
  // DELETE:
  sum_trans_freqs = 0;
  sum_alphas      = 0;
  for (unsigned int u=5; u<7; u++) {
    sum_trans_freqs += transition_frequencies[u];
    sum_alphas      += trans_alphas[u];
  }
  for (unsigned int v=5; v<7; v++) 
    transProbs.push_back(((mytype)transition_frequencies[v] + trans_alphas[v])/((mytype)sum_trans_freqs + sum_alphas));

  return transProbs;
}

//--------------------------------------------------------------------------------------------------------------------------------------------------------------------
// ermittleTransProbs(State*, State****&, int, int, int**&, int, int):
//
//    computes the transition probabilities for all successors of a state
//--------------------------------------------------------------------------------------------------------------------------------------------------------------------
void Transition::ermittleTransProbs(State*     akt_Zustand, 
				    State**** &allStates, 
				    int        numSubtypes,
				    int        numSpalten,
				    int**     &gaps,
				    int        num_JumpsToProfile,
				    int        num_JumpsToSubtypes) { 

  // possible transitions between subtypes: M -> M,D
  //                                        I -> M
  //                                        D -> M
  
  Transition    *akt_Uebergang;
  State         *akt_Nachfolger;
  mytype      local_jumpToRow           = 0;
  mytype      local_jumpToProfile       = num_JumpsToProfile*jumpRowToProfile;
  mytype      trans_MM_InSelberZeile    = 0, 
              trans_MD_InSelberZeile    = 0, 
              trans_MI_InSelberZeile    = 0, 
              trans_MM                  = 0, 
              trans_MD                  = 0;
  mytype      jump_To_C_Delete          = 0;
  int         numSuccConsensusColumns   = 0;
  int         numPrevConsensusColumns   = 0;
  int         spalte                    = 0;
  mytype jumpProb = 0;

  if (akt_Zustand->row == numSubtypes) { // profile HMM of whole alignment
    local_jumpToRow = (jumpProfileToRow/((mytype)numSubtypes));
  }
  else {
    if (numSubtypes == 1)
      local_jumpToRow = 0;
    else
      local_jumpToRow = (jumpRowToRow/((mytype)(numSubtypes-1)));
  }
 
  if (akt_Zustand->row == numSubtypes) { // general profile HMM (if ProfileMode == common)
    jumpProb = local_jumpToRow*((mytype)num_JumpsToSubtypes);
  }
  else { // profile HMM of a subtype (ort_vonNachfolgern[1] = 0 if ProfileMode == common)
    jumpProb = local_jumpToRow*((mytype)num_JumpsToSubtypes) + local_jumpToProfile*((mytype)num_JumpsToProfile);
  }
  
  if (akt_Zustand->stateType == Match) {
    spalte = numSpalten-1;
    numSuccConsensusColumns = 0;
    while (spalte > akt_Zustand->column) {
      if (gaps[akt_Zustand->row][spalte] == 1) { // passt auch fuer allg. ProfilHMM
	numSuccConsensusColumns++;
      }
      spalte--;
    }
    jump_To_C_Delete = exp( log(d_init)+(((mytype)(numSuccConsensusColumns-1))*(log(d_ext)))); 
    // -1 da Uebergang ueberhaupt zu C_Delete extra betrachtet wird ???
    for (unsigned int j=0; j<akt_Zustand->succ.size(); j++) {
      if ((akt_Zustand->succ[j]->whereTo->row == akt_Zustand->row)) { 
	for (unsigned int k=0; k<akt_Zustand->succ[j]->whereTo->pred.size(); k++){
	  if ((akt_Zustand->succ[j]->whereTo->pred[k]->whereFrom->row == akt_Zustand->row) && (akt_Zustand->succ[j]->whereTo->pred[k]->whereFrom->stateType == Match)) {
	    if (akt_Zustand->succ[j]->whereTo->stateType == Match)
	      trans_MM_InSelberZeile = akt_Zustand->succ[j]->whereTo->pred[k]->transProb;
	    else if (akt_Zustand->succ[j]->whereTo->stateType == Insert)
	      trans_MI_InSelberZeile = akt_Zustand->succ[j]->whereTo->pred[k]->transProb;
	    else
	      trans_MD_InSelberZeile = akt_Zustand->succ[j]->whereTo->pred[k]->transProb;        // can only be DELETE
	  }
	}
      }
    }
  }
  for  (unsigned int i=0; i<akt_Zustand->succ.size(); i++) {
    akt_Uebergang = akt_Zustand->succ[i];
    akt_Nachfolger = akt_Uebergang->whereTo;
    if (akt_Zustand->stateType == N_Insert) {
      if (akt_Nachfolger->stateType == N_Insert) 
	akt_Uebergang->transProb = (mytype)1-epsilon;
      else { // N_INSERT->MATCH
	if (profile_mode == nocommon)
	  akt_Uebergang->transProb = epsilon/((mytype)numSubtypes);
	else
	  if (akt_Nachfolger->row == numSubtypes) { // profile HMM of the whole alignment
	    akt_Uebergang->transProb = 0.5*epsilon;
	  }	    
	  else { // profile HMMs of subtypes
	    akt_Uebergang->transProb = 0.5*epsilon/((mytype)numSubtypes);
	  }
      }
    }
    else if (akt_Zustand->stateType == N_Delete) { // successors = each MATCH state
      spalte = 0;
      numPrevConsensusColumns = 0;
      while (spalte < akt_Nachfolger->column) {
	if (gaps[akt_Nachfolger->row][spalte] == 1) { // consensus column
	  numPrevConsensusColumns++;
	}
	spalte++;
      }
      // no difference between general profile HMM and profile HMMs of the subtypes
      if (profile_mode == nocommon)
	akt_Uebergang->transProb = exp((((mytype)numPrevConsensusColumns)*log(d_ext)) + log((mytype)1/((mytype)numSubtypes))); // aendern???
      else
	akt_Uebergang->transProb = exp((((mytype)numPrevConsensusColumns)*log(d_ext)) + log((mytype)1/((mytype)(numSubtypes+1))));
    }
    else if (akt_Zustand->stateType == Match) {
      if (akt_Nachfolger->row == akt_Zustand->row) {
	// innerhalb der einzelnen Sub-ProfilHMMs schon berechnet, jetzt Beruecksichtigung der Spruenge
	//akt_Uebergang->transProb = akt_Uebergang->transProb-(jump_To_C_Delete/((mytype)3))-(jumpProb/((mytype)3));  
	akt_Uebergang->transProb = akt_Uebergang->transProb*((mytype)1-jump_To_C_Delete-jumpProb);
	
	//akt_Uebergang->transProb = akt_Uebergang->transProb;
	// wenn es Nachfolgern in der selben Zeile gibt, dann gibt es 3 Nachfolger 
	// deshalb von jedem Uebergang nur 1/3 der Sprung- und 1/3 der jump_To_C_Delete-Wkt abgezogen! 
	// nicht akt_Zustand->column sondern wievielte Konsensusspalte?
      }
      else {
	if (akt_Nachfolger->stateType == End) 
	  akt_Uebergang->transProb = epsilon;
	else if (akt_Nachfolger->stateType == C_Insert) { // fixed prob
	  akt_Uebergang->transProb = ((mytype)1-jumpProb-jump_To_C_Delete-epsilon);
	  //akt_Uebergang->transProb = ((mytype)1-epsilon);
	}
	else if (akt_Nachfolger->stateType == C_Delete)
	  akt_Uebergang->transProb = jump_To_C_Delete;
	else { // transition to other subtypes(profile HMMs) or general profile HMM
	  // jump to M and D possible
	  // die Wkt fuer den Sprung in ein anderes ProfilHMM wird aufgeteilt entsprechend der Wkten fuer M und D an dieser Stelle des anderen ProfilHMMs 
	  // suchen dafuer entsprechenden Uebergang innerhalb des anderen ProfilHMMs
	  for (unsigned int j=0; j<akt_Zustand->succ.size(); j++) {
	    if ((akt_Zustand->succ[j]->whereTo->row == akt_Uebergang->whereTo->row)) { // && (akt_Zustand->succ[j]->whereTo->column == akt_Uebergang->whereTo->column)) {
	      for (unsigned int k=0; k<akt_Zustand->succ[j]->whereTo->pred.size(); k++){
		if ((akt_Zustand->succ[j]->whereTo->pred[k]->whereFrom->row == akt_Uebergang->whereTo->row) && (akt_Zustand->succ[j]->whereTo->pred[k]->whereFrom->stateType == Match)) {
		  if (akt_Zustand->succ[j]->whereTo->stateType == Match)
		    trans_MM = akt_Zustand->succ[j]->whereTo->pred[k]->transProb;
		  else if (akt_Zustand->succ[j]->whereTo->stateType == Delete)
		    trans_MD = akt_Zustand->succ[j]->whereTo->pred[k]->transProb;  
		}
	      }
	    }
	  }
	  if (trans_MM == 0 && trans_MD == 0) {
	    // falls in Nachfolger-Zeile noch keine Vorgaenger ausser Begin oder N_Insert vorhanden sind
	    // d.h. falls z.B. aus Spalte 0 in Zeile 0 in Spalte 1 in Zeile 1 gewechselt wird, in Zeile 1 aber in Spalte 0 Luecke war,  
	    // dann gibt es dort keinen entsprechenden Uebergang innerhalb des Modells, und in Delete und Match dieser Zeile 1 
	    // wird mit "normaler Anfangswkt." gewechselt: (epsilon/2 (fuer M und D))/numSubtypes 
	    trans_MM = epsilon/((mytype)2);
	    trans_MD = epsilon/((mytype)2);
	  }
	  if ((akt_Zustand->row != numSubtypes) && (akt_Uebergang->whereTo->row == numSubtypes)) { // transition to general profile HMM   
	    if (akt_Uebergang->whereTo->stateType == Match) 
	      akt_Uebergang->transProb = local_jumpToProfile * (trans_MM/(trans_MM+trans_MD));
	    else
	      akt_Uebergang->transProb = local_jumpToProfile * (trans_MD/(trans_MM+trans_MD)); 	
	  }
	  else { // transition to other subtypes(profile HMMs)
	    if (akt_Uebergang->whereTo->stateType == Match) 
	      akt_Uebergang->transProb =  local_jumpToRow * (trans_MM/(trans_MM+trans_MD));
	    else 
	      akt_Uebergang->transProb = local_jumpToRow * (trans_MD/(trans_MM+trans_MD));
	  }
	}
      }
    }
    else if (akt_Zustand->stateType == Insert) {
      if (akt_Nachfolger->row == akt_Zustand->row) {
	// already computed within the different profile HMMs/subtypes, now we must take into account the jumps
	akt_Uebergang->transProb = ((mytype)1-jumpProb)*akt_Uebergang->transProb;         
	//akt_Uebergang->transProb = akt_Uebergang->transProb; 
      }
      else if ((akt_Zustand->row != numSubtypes) && (akt_Uebergang->whereTo->row == numSubtypes)) { // transition to general profile HMM
	akt_Uebergang->transProb = local_jumpToProfile;   // INSERT->MATCH  
      }
      else { // transition to other subtypes(profile HMMs)
	akt_Uebergang->transProb = local_jumpToRow;
      }
    }
    else if (akt_Zustand->stateType == Delete) {
      if (akt_Nachfolger->row == akt_Zustand->row) {
	akt_Uebergang->transProb = ((mytype)1-jumpProb)*akt_Uebergang->transProb;       //already computed within the different profile HMMs/subtypes, now we must take into account the jumps
      }
      else {
	if (akt_Nachfolger->stateType == End) { // only END or other rows/subtypes = successors
	  akt_Uebergang->transProb = (mytype)1-jumpProb; 
	}
	else if ((akt_Zustand->row != numSubtypes) && (akt_Uebergang->whereTo->row == numSubtypes)) { // transition to general profile HMM
	  akt_Uebergang->transProb = local_jumpToProfile;   //DELETE->MATCH
	}
	else { // transition to other subtypes(profile HMMs)
	  akt_Uebergang->transProb = local_jumpToRow;
	}
      }
    }
  } 
}
  

/******************************************************\

 *                   class State                      *

\******************************************************/

State::State() {
}

State::State(HMM_StateType type, 
	     int           zeile, 
	     int           spalte) {
  
  stateType = type;
  row       = zeile;
  column    = spalte;
}

State::~State() {
}

//--------------------------------------------------------------------------------------------------------------------------------------------------------------------
// dirichlet_EmissionPriors(HMM_StateType, mytype*, ifstream&):
//
//    reads in the Dirichlet priors for the emission probabilities
//    (a 9-component Dirichlet mixture(for proteins) or a single-component Dirichlet mixture (for DNA))
//    and computes the emission probabilities of a MATCH or an INSERT state
//--------------------------------------------------------------------------------------------------------------------------------------------------------------------
vector<mytype> State::dirichlet_EmissionPriors(HMM_StateType type,
					       mytype* frequencies, 
					       ifstream& emissionPriors) {
  
  vector<mytype>  emProbs;
  const  int      MAX_ROW_LEN        = 8192;
  static mytype** match_Alphas       = NULL;         // table for MATCH emission priors 
  static mytype** insert_Alphas      = NULL;         // table for INSERT emission priors
  static mytype*  q                  = NULL;         // vector of the (9) weight vectors
  static mytype*  sum_Match_Alphas   = NULL;         // sum of the MATCH emission priors for each of the (9) components
  static mytype*  sum_Insert_Alphas  = NULL;         // sum of the INSERT emission priors for each of the (9) components
        
  // entspricht Term in exp() mit Beta-Funktion
  static int      numComponents      = 0;
  static int      numResidues        = 0;

  if ((match_Alphas == NULL) || (insert_Alphas == NULL) || (q == NULL) || (sum_Match_Alphas == NULL) || (sum_Insert_Alphas == NULL)) {
    char *comment = new char[MAX_ROW_LEN];
    string name;
    
    while ( (emissionPriors >> ws) && (emissionPriors.peek() == '#')) 
      emissionPriors.getline(comment, MAX_ROW_LEN);
   
    emissionPriors >> name >> numComponents;
    emissionPriors >> name >> numResidues;
    emissionPriors >> ws;
    emissionPriors.getline(comment, MAX_ROW_LEN);
    //delete [] comment;

    match_Alphas  = new mytype*[numComponents];
    insert_Alphas = new mytype*[numComponents];
    for (int i=0; i<numComponents; i++) {
      match_Alphas[i]  = new mytype[numResidues];
      insert_Alphas[i] = new mytype[numResidues];
    }
    q                 = new mytype[numComponents];
    sum_Match_Alphas  = new mytype[numComponents];   
    sum_Insert_Alphas = new mytype[numComponents]; 
    
    emissionPriors >> name;
    for (int k=0; k<numComponents; k++) 
      emissionPriors >> q[k];

    // MATCHes:
    emissionPriors >> ws;
    emissionPriors.getline(comment, MAX_ROW_LEN);
    
    emissionPriors >> ws;
    for (int j=0; j<numResidues; j++){
      emissionPriors >> name;
      for (int i=0; i<numComponents; i++) 
	emissionPriors >> match_Alphas[i][j];
    }
    emissionPriors >> ws;
    emissionPriors >> name;
    for (int k=0; k<numComponents; k++) 
      emissionPriors >> sum_Match_Alphas[k];

    // INSERTs:
    emissionPriors >> ws;
    emissionPriors.getline(comment, MAX_ROW_LEN);
    
    emissionPriors >> ws;
    for (int j=0; j<numResidues; j++){
      emissionPriors >> name;
      for (int i=0; i<numComponents; i++) 
	emissionPriors >> insert_Alphas[i][j];
    }
    emissionPriors >> ws;
    emissionPriors >> name;
    for (int k=0; k<numComponents; k++) 
      emissionPriors >> sum_Insert_Alphas[k];
    
  }
  mytype   sum_frequencies = 0;
  mytype** alphaAndFreqs   = new mytype*[numComponents];
  for (int i=0; i<numComponents; i++) 
      alphaAndFreqs[i] = new mytype[numResidues];
  mytype*  X          = new mytype[numResidues];
  mytype   sum_X      = 0;
  mytype*  temp_var   = new mytype[numComponents];                
  mytype** alphas     = NULL;
  mytype*  sum_Alphas = NULL;
  if (type == Match) {
    alphas     = match_Alphas;
    sum_Alphas = sum_Match_Alphas;
  }
  else if (type == Insert) {
    alphas     = insert_Alphas;
    sum_Alphas = sum_Insert_Alphas;
  }

  for (int u=0; u<numResidues; u++)
    sum_frequencies += frequencies[u];
  
  for (int s=0; s<numComponents; s++) 
    for (int t=0; t<numResidues; t++) 
      alphaAndFreqs[s][t] = alphas[s][t] + frequencies[t];
  
  for (int r=0; r<numComponents; r++) {
    temp_var[r] = 0;
    for (int z=0; z<numResidues; z++) 
      temp_var[r] += lgamma(alphaAndFreqs[r][z]) - lgamma(alphas[r][z]);
    temp_var[r] += lgamma(sum_Alphas[r] + sum_frequencies) - lgamma(sum_Alphas[r]);
  }
  
  mytype max_BetaFct = temp_var[0];                  // see Dirichlet p.13 near 'efficiency improvements'
  for (int v=1; v<numComponents; v++) 
    if (temp_var[v] > max_BetaFct)
      max_BetaFct = temp_var[v];
  
  for (int k=0; k<numResidues; k++) {
    X[k] = 0;
    for (int l=0; l<numComponents; l++)
      X[k] +=  q[l] * exp(temp_var[l]-max_BetaFct) * (alphaAndFreqs[l][k]/(sum_Alphas[l]+ sum_frequencies));
  }
  for (int m=0; m<numResidues; m++) 
    sum_X += X[m]; 
  
  for (int n=0; n<numResidues; n++) 
    emProbs.push_back(X[n]/sum_X);
  
  return emProbs;
}

//--------------------------------------------------------------------------------------------------------------------------------------------------------------------
// log_Probs(vector<mytype>):
// 
//    computes the logarithm for each entry of a vector
//--------------------------------------------------------------------------------------------------------------------------------------------------------------------
vector<mytype> State::log_Probs(vector<mytype> probs) {

  vector<mytype> log_Probs;
  
  for (unsigned int i=0; i<probs.size(); i++) 
    log_Probs.push_back(log(probs[i]));
  return log_Probs;
}
/*****************************************************************/
